# generated by datamodel-codegen:
#   filename:  webscene_schema.json

from __future__ import annotations

from enum import Enum
from typing import Any, Literal, Optional, Annotated, Union
from pydantic import BaseModel as PydanticBaseModel, model_validator
from pydantic import (
    field_validator,
    ConfigDict,
    Field,
    confloat,
    conint,
    constr,
)

common_config = ConfigDict(
    extra="ignore",
    populate_by_name=True,
    use_enum_values=True,
)


class BaseModel(PydanticBaseModel):
    @field_validator("id", mode="before", check_fields=False)
    @classmethod
    def check_id(cls, v):
        # ensure that the id is a string, else make it a str
        if not isinstance(v, str):
            return str(v)
        return v

    @field_validator("item_id", mode="before", check_fields=False)
    @classmethod
    def check_item_id(cls, v):
        # ensure that the id is a string, else make it a str
        if not isinstance(v, str):
            return str(v)
        return v

    def _handle_enums(self, data):
        for key, value in data.items():
            if isinstance(value, Enum):
                data[key] = value.value
            elif isinstance(value, dict):
                data[key] = self._handle_enums(value)
        return data

    def dict(
        self,
        *,
        mode: str = "python",
        include=None,
        exclude=None,
        by_alias: bool = True,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = True,
        round_trip: bool = False,
        warnings: bool = False,
    ) -> dict[str, Any]:
        d = super().model_dump(
            mode=mode,
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
            round_trip=round_trip,
            warnings=warnings,
        )

        return self._handle_enums(d)


class SymbolValidatorMixin(BaseModel):
    model_config = common_config

    @model_validator(mode="before")
    def validate_symbol_type(cls, values):
        # Handle both instance and dict cases
        # if dataclass, we can skip whole thing
        if isinstance(values, BaseModel):
            return values
        symbol_data = values.get("symbol")

        symbol_types_lookup = {
            "esriSFS": SimpleFillSymbolEsriSFS,
            "esriSLS": SimpleLineSymbolEsriSLS,
            "esriSMS": SimpleMarkerSymbolEsriSMS,
            "esriPMS": PictureMarkerSymbolEsriPMS,
            "esriTS": TextSymbolEsriTS,
            "LineSymbol3D": LineSymbol3D,
            "MeshSymbol3D": MeshSymbol3D,
            "PointSymbol3D": PointSymbol3D,
            "PolygonSymbol3D": PolygonSymbol3D,
        }

        if symbol_data and isinstance(symbol_data, dict):
            symbol_type = symbol_data.get("type")

        # Use the lookup dictionary to get the appropriate class and instantiate it
        if symbol_type in symbol_types_lookup:
            values["symbol"] = symbol_types_lookup[symbol_type](**symbol_data)

        return values

    @model_validator(mode="before")
    def validate_default_symbol_type(cls, values):
        # Handle both instance and dict cases
        # if dataclass, we can skip whole thing
        if isinstance(values, BaseModel):
            return values

        symbol_mapping = {
            "esriSFS": SimpleFillSymbolEsriSFS,
            "esriSLS": SimpleLineSymbolEsriSLS,
            "esriSMS": SimpleMarkerSymbolEsriSMS,
            "esriPMS": PictureMarkerSymbolEsriPMS,
            "esriTS": TextSymbolEsriTS,
            "esriPFS": PictureFillSymbolsEsriPFS,
        }
        # Handle both instance and dict cases
        symbol_data = None
        if not isinstance(values, dict) and hasattr(values, "default_symbol"):
            symbol_data = values.default_symbol
        if isinstance(values, dict):
            symbol_data = values.get("default_symbol")

        if symbol_data and isinstance(symbol_data, dict):
            symbol_type = symbol_data.get("type")
            symbol_class = symbol_mapping.get(symbol_type)
            if symbol_class:
                values["default_symbol"] = symbol_class(**symbol_data)
        return values


class ViewingMode(Enum):
    global_ = "global"
    local = "local"


class HeightModel(Enum):
    """
    The surface type or height model of the vertical coordinate system.
    """

    ellipsoidal = "ellipsoidal"
    gravity_related_height = "gravity_related_height"


class HeightUnit(Enum):
    """
    The unit of the vertical coordinate system.<a href="#heightUnit"><sup>1</sup></a>
    """

    field_150_kilometers = "150-kilometers"
    field_50_kilometers = "50-kilometers"
    benoit_1895_b_chain = "benoit-1895-b-chain"
    clarke_foot = "clarke-foot"
    clarke_link = "clarke-link"
    clarke_yard = "clarke-yard"
    foot = "foot"
    gold_coast_foot = "gold-coast-foot"
    indian_1937_yard = "indian-1937-yard"
    indian_yard = "indian-yard"
    meter = "meter"
    sears_1922_truncated_chain = "sears-1922-truncated-chain"
    sears_chain = "sears-chain"
    sears_foot = "sears-foot"
    sears_yard = "sears-yard"
    us_foot = "us-foot"


class HeightModelInfo(BaseModel):
    """
    An object that defines the characteristics of the vertical coordinate system used by the web scene.
    """

    model_config = common_config
    height_model: HeightModel | None = Field(
        HeightModel.ellipsoidal,
        validate_default=True,
        alias="heightModel",
        description="The surface type or height model of the vertical coordinate system.",
    )
    height_unit: HeightUnit = Field(
        ...,
        alias="heightUnit",
        description='The unit of the vertical coordinate system.<a href="#heightUnit"><sup>1</sup></a>',
    )
    vert_crs: str | None = Field(
        None,
        alias="vertCRS",
        description="(Optional) The datum realization of the vertical coordinate system.",
    )


class MapRangeInformation(BaseModel):
    """
    Map range information
    """

    model_config = common_config
    active_range_name: str = Field(
        ...,
        alias="activeRangeName",
        description="Active range ID that slider/picker acts upon.",
    )
    current_range_extent: list[float] | None = Field(
        None,
        alias="currentRangeExtent",
        description="Current range for the active range.",
        max_length=2,
        min_length=2,
    )
    full_range_extent: list[float] | None = Field(
        None,
        alias="fullRangeExtent",
        description="Full range extent for the active range to be presented in the UI.",
        max_length=2,
        min_length=2,
    )


class SpatialReference(BaseModel):
    """
    The spatialReference object is located at the top level of the JSON hierarchy. In addition to this, it is also available within the operationalLayer and basemap objects.

    Many predefined spatial references have already been defined and are available for use. A spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT). The default tolerance and resolution values for the associated coordinate system are used.

    A spatial reference can optionally include a definition for a vertical coordinate system (VCS), which is used to interpret the z values of a geometry.
    """

    model_config = common_config
    latest_vcs_wkid: int | None = Field(
        None,
        alias="latestVcsWkid",
        description="(Optional) The current wkid value of the vertical coordinate system.",
    )
    latest_wkid: int | None = Field(
        None,
        alias="latestWkid",
        description="(Optional) Identifies the current wkid value associated with the same spatial reference. For example a WKID of '102100' (Web Mercator) has a latestWKid of '3857'.",
    )
    vcs_wkid: int | None = Field(
        None,
        alias="vcsWkid",
        description="(Optional) The wkid value of the vertical coordinate system.",
    )
    wkid: int | None = Field(
        None,
        description="The well-known ID (WKID) of the coordinate system. Specify either WKID or the well-known text (WKT) of the coordinate system.",
    )
    wkt: str = Field(
        None,
        description="The well-known text (WKT) of the coordinate system. Specify either WKT or WKID of the coordinate system.",
    )


class TransportationNetwork(BaseModel):
    """
    The transportation network used for routing in the scene.
    """

    model_config = common_config
    name: str = Field(
        ...,
        description="The name of the transportation network. The name must be unique within a scene.",
    )
    path: constr(pattern=r"^file:.+\.geodatabase?$") = Field(
        ...,
        description="The path to the geodatabase which contains the network.",
    )
    title: str = Field(
        ...,
        description="A title or alias of the network layer. This can be used in the client user interface.",
    )


class Version(Enum):
    """
    Root element in the web scene specifying a string value indicating the web scene version.
    """

    field_1_8 = "1.8"
    field_1_9 = "1.9"
    field_1_10 = "1.10"
    field_1_11 = "1.11"
    field_1_12 = "1.12"
    field_1_13 = "1.13"
    field_1_14 = "1.14"
    field_1_15 = "1.15"
    field_1_16 = "1.16"
    field_1_17 = "1.17"
    field_1_18 = "1.18"
    field_1_19 = "1.19"
    field_1_20 = "1.20"
    field_1_21 = "1.21"
    field_1_22 = "1.22"
    field_1_23 = "1.23"
    field_1_24 = "1.24"
    field_1_25 = "1.25"
    field_1_26 = "1.26"
    field_1_27 = "1.27"
    field_1_28 = "1.28"
    field_1_29 = "1.29"
    field_1_30 = "1.30"
    field_1_31 = "1.31"
    field_1_32 = "1.32"
    field_1_33 = "1.33"
    field_1_34 = "1.34"


class Extent(BaseModel):
    """
    This object defines the bounding geometry given the lower-left and upper-right corners of the bounding box. A [spatial reference](spatialReference.md) is also required.
    """

    model_config = common_config
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="An object used to specify the spatial reference of the given geometry.",
        title="spatialReference",
    )
    xmax: float | None = Field(
        None,
        description="A numeric value indicating the top-right X-coordinate of an extent envelope.",
    )
    xmin: float | Literal["NaN"] | None = Field(
        None,
        description="A numeric value indicating the bottom-left X-coordinate of an extent envelope.",
    )
    ymax: float | None = Field(
        None,
        description="A numeric value indicating the top-right Y-coordinate of an extent envelope.",
    )
    ymin: float | None = Field(
        None,
        description="A numeric value indicating the bottom-left Y-coordinate of an extent envelope.",
    )


class Type(Enum):
    none = "none"
    stay_above = "stayAbove"


class NavigationConstraint(BaseModel):
    """
    Object determining whether the camera is constrained to navigate only above, or also under the ground surface.
    """

    model_config = common_config
    type: Type


class FacilityLayer(BaseModel):
    """
    Defines the layer and field properties for the Facility layer used for floor filtering.
    """

    model_config = common_config
    facility_id_field: str = Field(
        ...,
        alias="facilityIdField",
        description="The name of the attribute field that contains a facility feature's unique identifier.",
    )
    layer_id: str = Field(
        ...,
        alias="layerId",
        description="A layer ID that refers to an operational layer in the map. The layer provides access to Facility features to use for floor filtering.",
    )
    name_field: str = Field(
        ...,
        alias="nameField",
        description="The name of the attribute field that contains a facility feature's name.",
    )
    site_id_field: str | None = Field(
        None,
        alias="siteIdField",
        description="The name of the attribute field that contains a facility feature's site identifier (a foreign key to the Site layer).",
    )


class LevelLayer(BaseModel):
    """
    Defines the layer and field properties for the Level layer used for floor filtering.
    """

    model_config = common_config
    facility_id_field: str = Field(
        ...,
        alias="facilityIdField",
        description="The name of the attribute field that contains a level feature's facility identifier (a foreign key to the Facility layer).",
    )
    layer_id: str = Field(
        ...,
        alias="layerId",
        description="A layer ID that refers to an operational layer in the map. The layer provides access to Level features to use for floor filtering.",
    )
    level_id_field: str = Field(
        ...,
        alias="levelIdField",
        description="The name of the attribute field that contains a level feature's unique identifier.",
    )
    level_number_field: str = Field(
        ...,
        alias="levelNumberField",
        description="The name of the attribute field that contains a level feature's level number specific to its facility.",
    )
    long_name_field: str = Field(
        ...,
        alias="longNameField",
        description="The name of the attribute field that contains a level feature's long name.",
    )
    short_name_field: str = Field(
        ...,
        alias="shortNameField",
        description="The name of the attribute field that contains a level feature's short name.",
    )
    vertical_order_field: str = Field(
        ...,
        alias="verticalOrderField",
        description="The name of the attribute field that contains a level feature's vertical order. The vertical order defines the order of display in the floor filter widget, and it also references the floor levels of an Indoor Positioning System.",
    )


class SiteLayer(BaseModel):
    """
    Defines the layer and field properties for the Site layer used for floor filtering.
    """

    model_config = common_config
    layer_id: str = Field(
        ...,
        alias="layerId",
        description="A layer ID that refers to an operational layer in the map. The layer provides access to Site features to use for floor filtering.",
    )
    name_field: str = Field(
        ...,
        alias="nameField",
        description="The name of the attribute field that contains a site feature's name.",
    )
    site_id_field: str = Field(
        ...,
        alias="siteIdField",
        description="The name of the attribute field that contains a site feature's unique identifier.",
    )


class ListMode(Enum):
    """
    To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.
    """

    hide = "hide"
    hide_children = "hide-children"
    show = "show"


class ColumnDelimiter(Enum):
    """
    A string defining the character used to separate columns in a CSV file.
    """

    field_ = "\t"
    field__1 = " "
    field__2 = ","
    field__3 = ";"
    field__4 = "|"


class VisibilityMode(Enum):
    """
    How visibility of children is affected:independent, exclusive, inherited
    """

    exclusive = "exclusive"
    independent = "independent"
    inherited = "inherited"


class Format(Enum):
    """
    String value representing image format.
    """

    bip = "bip"
    bmp = "bmp"
    bsq = "bsq"
    gif = "gif"
    jpg = "jpg"
    jpgpng = "jpgpng"
    lerc = "lerc"
    png = "png"
    png24 = "png24"
    png32 = "png32"
    png8 = "png8"
    tiff = "tiff"


class Interpolation(Enum):
    """
    The algorithm used for interpolation.
    """

    rsp_bilinear_interpolation = "RSP_BilinearInterpolation"
    rsp_cubic_convolution = "RSP_CubicConvolution"
    rsp_majority = "RSP_Majority"
    rsp_nearest_neighbor = "RSP_NearestNeighbor"


class NoDataInterpretation(Enum):
    """
    A string value of interpretation of noData setting. Default is `esriNoDataMatchAny` when noData is a number, and `esriNoDataMatchAll` when noData is an array.
    """

    esri_no_data_match_all = "esriNoDataMatchAll"
    esri_no_data_match_any = "esriNoDataMatchAny"


class PixelType(Enum):
    """
    Pertains to the type of values stored in the raster, such as signed integer, unsigned integer, or floating point.
    """

    c128 = "C128"
    c64 = "C64"
    f32 = "F32"
    f64 = "F64"
    s16 = "S16"
    s32 = "S32"
    s8 = "S8"
    u1 = "U1"
    u16 = "U16"
    u2 = "U2"
    u32 = "U32"
    u4 = "U4"
    u8 = "U8"
    unknown = "UNKNOWN"


class KMLLayerKML(BaseModel):
    """
    Keyhole Markup Language (KML) is an XML-based format for storing geographic data and associated content and is an official Open Geospatial Consortium (OGC) standard. KML is a common format for sharing geographic data with non-GIS users as it can be easily delivered on the Internet.
    """

    model_config = common_config
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="A string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["KML"] = Field(
        "KML",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        alias="listMode",
        description="To show or hide the sublayer in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.km[lz]$") | None = Field(
        None,
        description="For offline data, a path to a KML file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.kml` or `.kmz`.  A relative path must be from the file which defines the layer. For example `file:../commondata/kml/paris.kml`.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Indicates whether to allow scene authors the ability to control what layers should be shown in a client's legend.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visible_folders: list[int] | None = Field(
        None,
        alias="visibleFolders",
        description="Array of numeric IDs of folders that will be made visible.",
    )


class KMLLayerKML1(BaseModel):
    """
    Keyhole Markup Language (KML) is an XML-based format for storing geographic data and associated content and is an official Open Geospatial Consortium (OGC) standard. KML is a common format for sharing geographic data with non-GIS users as it can be easily delivered on the Internet.
    """

    model_config = common_config
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="A string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["KML"] = Field(
        "KML",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        alias="listMode",
        description="To show or hide the sublayer in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.km[lz]$") = Field(
        ...,
        description="For offline data, a path to a KML file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.kml` or `.kmz`.  A relative path must be from the file which defines the layer. For example `file:../commondata/kml/paris.kml`.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Indicates whether to allow scene authors the ability to control what layers should be shown in a client's legend.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str | None = Field(None, description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visible_folders: list[int] | None = Field(
        None,
        alias="visibleFolders",
        description="Array of numeric IDs of folders that will be made visible.",
    )


class RasterDataLayer(BaseModel):
    """
    A layer for displaying raster data. This layer only applies to offline data.
    """

    model_config = common_config
    copyright: str | None = Field(
        None,
        description="Attribution to the Raster Data Layer provider. It is displayed in the attribution on the scene. Input required by the user when the layer is added to the scene.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    layer_type: Literal["RasterDataLayer"] = Field(
        "RasterDataLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        alias="listMode",
        description="To show or hide the sublayer in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+$") = Field(
        ...,
        description="For offline data, a path to a raster data file. A URI format is used, starting with `file:` followed by a file system path. A relative path must be from the file which defines the layer. For example `file:../commondata/raster_data/beijing.tif`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the scene.",
    )


class Interpolation1(Enum):
    """
    String indicating the interpolation type.
    """

    rsp_bilinear_interpolation = "RSP_BilinearInterpolation"
    rsp_cubic_convolution = "RSP_CubicConvolution"
    rsp_majority = "RSP_Majority"
    rsp_nearest_neighbor = "RSP_NearestNeighbor"


class Format1(Enum):
    """
    A string containing the image format to be requested from a WMS service. Default is `png`.
    """

    bmp = "bmp"
    gif = "gif"
    jpg = "jpg"
    png = "png"
    svg = "svg"


class HtmlPopupType(Enum):
    """
    String value indicating the HTML popup type.
    """

    esri_server_html_popup_type_as_html_text = "esriServerHTMLPopupTypeAsHTMLText"
    esri_server_html_popup_type_as_url = "esriServerHTMLPopupTypeAsURL"
    esri_server_html_popup_type_none = "esriServerHTMLPopupTypeNone"


class Type1(Enum):
    """
    Indicates whether the layerDefinition applies to a Feature Layer or a Table.
    """

    feature_layer = "Feature Layer"
    table = "Table"


class FloorFilter(BaseModel):
    """
    Configurable properties used by the floor filter widget.
    """

    model_config = common_config
    enabled: bool | None = Field(
        False,
        description="Indicates whether the floor filter is actively filtering the displayed content according to the floor filter selection.",
    )
    facility: str | None = Field(
        None,
        description="Contains a facility identifier for the initially selected facility in the floor filter.",
    )
    level: str | None = Field(
        None,
        description="Contains a level identifier for the initially selected floor, which is used when filtering floor-aware layers.",
    )
    long_names: bool | None = Field(
        False,
        alias="longNames",
        description="Indicates whether the floor filter is showing long names instead of short names for floor levels.",
    )
    minimized: bool | None = Field(
        False,
        description="Indicates whether the floor filter has been minimized to show only the levels list instead of showing the full set of breadcrumbs.",
    )
    pinned_levels: bool | None = Field(
        False,
        alias="pinnedLevels",
        description="Indicates whether the floor filter has been pinned to show the floor levels list, instead of including the levels as a breadcrumb dropdown.",
    )
    site: str | None = Field(
        None,
        description="Contains a site identifier for the initially selected site in the floor filter.",
    )


class InteractionMode(Enum):
    """
    Indicates the mode in which the active range should be presented to the user.
    """

    picker = "picker"
    slider = "slider"


class Range(BaseModel):
    """
    Range object.
    """

    model_config = common_config
    interaction_mode: InteractionMode = Field(
        ...,
        alias="interactionMode",
        description="Indicates the mode in which the active range should be presented to the user.",
    )
    number_of_stops: int = Field(
        ...,
        alias="numberOfStops",
        description="This is used to generate the activeRangeValues if activeRangeValues are not specified.",
    )
    stop_interval: float | None = Field(
        None,
        alias="stopInterval",
        description="Interval in which stops should be generated.",
    )


class Range1(BaseModel):
    """
    Range object.
    """

    model_config = common_config
    interaction_mode: InteractionMode = Field(
        ...,
        alias="interactionMode",
        description="Indicates the mode in which the active range should be presented to the user.",
    )
    number_of_stops: int | None = Field(
        None,
        alias="numberOfStops",
        description="This is used to generate the activeRangeValues if activeRangeValues are not specified.",
    )
    stop_interval: float = Field(
        ...,
        alias="stopInterval",
        description="Interval in which stops should be generated.",
    )


class RasterDataElevationLayer(BaseModel):
    """
    RasterDataElevationLayer is a single-band raster layer used for rendering elevation.
    """

    model_config = common_config
    id: str | None = Field(
        None, description="A unique identifying string for the layer"
    )
    layer_type: Literal["RasterDataElevationLayer"] = Field(
        "RasterDataElevationLayer",
        alias="layerType",
        description="String indicating the layer type",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the elevation layer in the layer list",
    )
    path: constr(pattern=r"^file:.+$") = Field(
        ...,
        description="For offline data, a path to an ArcGIS Runtime supported raster data file. A URI format is used, starting with file: followed by a file system path. A relative path must be from the file which defines the layer. For example `file:../commondata/raster_data/beijing.tif`",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )


class TimeSlider(BaseModel):
    """
    Time animation is controlled by a configurable time slider. Those configurations are saved to the web scene as a timeSlider widget.
    """

    model_config = common_config

    properties: dict[str, Any] | TimeSliderProperties = Field(
        ..., description="The properties of the time slider."
    )


class TimeSliderProperties(BaseModel):
    """
    Configurable properties for the time slider.
    """

    model_config = common_config

    current_time_extent: list[int] | None = Field(
        None,
        alias="currentTimeExtent",
        description="The current time extent of the time slider.",
    )
    end_time: int | None = Field(
        None, alias="endTime", description="The end time of the time slider."
    )
    loop: bool | None = Field(
        False, description="Indicates whether the time slider should loop."
    )
    number_of_stops: int | None = Field(
        None,
        alias="numberOfStops",
        description="The number of stops in the time slider.",
    )
    start_time: int | None = Field(
        None,
        alias="startTime",
        description="The start time of the time slider.",
    )
    stops: list[int] | None = Field(None, description="The stops of the time slider.")
    thumb_count: int | None = Field(
        2,
        alias="thumbCount",
        description="The number of thumbs in the time slider.",
    )
    thumb_moving_rate: int | None = Field(
        2000,
        alias="thumbMovingRate",
        description="The rate at which the thumb moves in the time slider.",
    )
    time_stop_interval: TimeStopInterval | None = Field(
        None,
        alias="timeStopInterval",
        description="The time stop interval of the time slider.",
    )


class IntervalUnits(Enum):
    """
    The units of the time stop.
    """

    centuries = "esriTimeUnitsCenturies"
    days = "esriTimeUnitsDays"
    decades = "esriTimeUnitsDecades"
    hours = "esriTimeUnitsHours"
    milliseconds = "esriTimeUnitsMilliseconds"
    minutes = "esriTimeUnitsMinutes"
    months = "esriTimeUnitsMonths"
    seconds = "esriTimeUnitsSeconds"
    weeks = "esriTimeUnitsWeeks"
    years = "esriTimeUnitsYears"


class TimeStopInterval(BaseModel):
    """
    The interval set for the time slider widget
    """

    model_config = common_config

    interval: int | None = Field(None, description="The interval of the time stop.")
    units: str | IntervalUnits | None = Field(
        None, description="The units of the time stop."
    )


class TiledElevationLayer(BaseModel):
    """
    Tiled Elevation Layer is a tile layer used for rendering elevation.
    """

    model_config = common_config

    id: str | None = Field(
        None, description="A unique identifying string for the layer"
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["ArcGISTiledElevationServiceLayer"] = Field(
        "ArcGISTiledElevationServiceLayer",
        alias="layerType",
        description="String indicating the layer type",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the elevation layer in the layer list",
    )
    path: constr(pattern=r"^file:.+\.tpkx?$") | None = Field(
        None,
        description="For offline data, a path to a tile package containing elevation data. A URI format is used, starting with `file:` followed by a file system path with an extension of `.tpk` or `.tpkx`. A relative path must be from the file which defines the layer. For example `file:../p20/portland.tpk`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str = Field(
        ...,
        description="The URL to the layer.If the layer is not from a web service but rather a feature collection, then the url property is omitted",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[str] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Array of two strings representing the start and end time of the visibility of the layer.",
    )


class ColorBackground(BaseModel):
    """
    Specifies a single color to fill the background of the scene with. The scene background is displayed behind any scene objects, stars and atmosphere.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="The value has to lie between `100` (full transparency) and `0` (full opacity).",
    )
    type: Literal["color"] = Field("color", description="The type of background.")


class CloudyWeather(BaseModel):
    """
    Object containing information for changing the weather conditions in the scene to cloudy.
    """

    model_config = common_config
    cloud_cover: confloat(ge=0.0, le=1.0) | None = Field(
        0.5,
        alias="cloudCover",
        description="Specifies the amount of cloud cover in the sky.",
    )
    type: Literal["cloudy"] = Field("cloudy", description="The type of weather.")


class FoggyWeather(BaseModel):
    """
    Object containing information for changing the weather conditions in the scene to foggy.
    """

    model_config = common_config
    fog_strength: confloat(ge=0.0, le=1.0) | None = Field(
        0.5,
        alias="fogStrength",
        description="Specifies the amount of fog used in the scene.",
    )
    type: Literal["foggy"] = Field("foggy", description="The type of weather.")


class RainyWeather(BaseModel):
    """
    Object containing information for changing the weather conditions in the scene to rainy.
    """

    model_config = common_config
    cloud_cover: confloat(ge=0.0, le=1.0) | None = Field(
        0.5,
        alias="cloudCover",
        description="Specifies the amount of cloud cover in the sky.",
    )
    precipitation: confloat(ge=0.0, le=1.0) | None = Field(
        0.5, description="Specifies the amount of rainfall in the scene."
    )
    type: Literal["rainy"] = Field("rainy", description="The type of weather.")


class SnowCover(Enum):
    """
    Display surfaces covered with snow.
    """

    disabled = "disabled"
    enabled = "enabled"


class SnowyWeather(BaseModel):
    """
    Object containing information for changing the weather conditions in the scene to snowy.
    """

    model_config = common_config
    cloud_cover: confloat(ge=0.0, le=1.0) | None = Field(
        0.5,
        alias="cloudCover",
        description="Specifies the amount of cloud cover in the sky.",
    )
    precipitation: confloat(ge=0.0, le=1.0) | None = Field(
        0.5, description="Specifies the amount of snowfall in the scene."
    )
    snow_cover: SnowCover | None = Field(
        SnowCover.disabled,
        validate_default=True,
        alias="snowCover",
        description="Display surfaces covered with snow.",
    )
    type: Literal["snowy"] = Field("snowy", description="The type of weather.")


class SunnyWeather(BaseModel):
    """
    Object containing information for changing the weather conditions in the scene to sunny.
    """

    model_config = common_config
    cloud_cover: confloat(ge=0.0, le=1.0) | None = Field(
        0.5,
        alias="cloudCover",
        description="Specifies the amount of cloud cover in the sky.",
    )
    type: Literal["sunny"] = Field("sunny", description="The type of weather.")


class LocationInfo(BaseModel):
    """
    Defines how location information will be retrieved from a [CSV](csvLayer.md) file referenced through the web, ie. referenced by URL.
    """

    model_config = common_config
    latitude_field_name: str | None = Field(
        None,
        alias="latitudeFieldName",
        description="A string defining the field name that holds the latitude (Y) coordinate.",
    )
    location_type: Literal["coordinates"] = Field(
        "coordinates",
        alias="locationType",
        description="String value indicating location type.",
    )
    longitude_field_name: str | None = Field(
        None,
        alias="longitudeFieldName",
        description="A string defining the field name that holds the longitude (X) coordinate.",
    )


class MeasureType(Enum):
    """
    Defines whether the horizontal, vertical or direct distance between the start and end points is measured.
    """

    direct = "direct"
    horizontal = "horizontal"
    vertical = "vertical"


class DimensionSimpleStyle(BaseModel):
    """
    Specification of how dimensions and their labels are displayed.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ..., description="Color of dimension lines.", title="color"
    )
    font_size: confloat(ge=0.0) = Field(
        ...,
        alias="fontSize",
        description="Font size of dimension label text in points.",
    )
    line_size: confloat(ge=0.0) = Field(
        ...,
        alias="lineSize",
        description="Width of dimension lines in points.",
    )
    text_background_color: list[conint(ge=0, le=255)] = Field(
        ...,
        alias="textBackgroundColor",
        description="Background color of dimension labels.",
        title="color",
    )
    text_color: list[conint(ge=0, le=255)] = Field(
        ...,
        alias="textColor",
        description="Color of dimension label text.",
        title="color",
    )
    type: Literal["simple"] = Field(
        "simple", description="Specifies the type of style used."
    )


class BlendMode(Enum):
    """
    Blend modes are used to create various effects by blending colors of top and background layers. `normal` blend mode is the default.
    """

    average = "average"
    color = "color"
    color_burn = "color-burn"
    color_dodge = "color-dodge"
    darken = "darken"
    destination_atop = "destination-atop"
    destination_in = "destination-in"
    destination_out = "destination-out"
    destination_over = "destination-over"
    difference = "difference"
    exclusion = "exclusion"
    hard_light = "hard-light"
    hue = "hue"
    invert = "invert"
    lighten = "lighten"
    lighter = "lighter"
    luminosity = "luminosity"
    minus = "minus"
    multiply = "multiply"
    normal = "normal"
    overlay = "overlay"
    plus = "plus"
    reflect = "reflect"
    saturation = "saturation"
    screen = "screen"
    soft_light = "soft-light"
    source_atop = "source-atop"
    source_in = "source-in"
    source_out = "source-out"
    vivid_light = "vivid-light"
    xor = "xor"


class MosaicMethod(Enum):
    """
    A string value that determines how the selected rasters are ordered.
    """

    esri_mosaic_attribute = "esriMosaicAttribute"
    esri_mosaic_center = "esriMosaicCenter"
    esri_mosaic_lock_raster = "esriMosaicLockRaster"
    esri_mosaic_nadir = "esriMosaicNadir"
    esri_mosaic_none = "esriMosaicNone"
    esri_mosaic_northwest = "esriMosaicNorthwest"
    esri_mosaic_seamline = "esriMosaicSeamline"
    esri_mosaic_viewpoint = "esriMosaicViewpoint"


class MosaicOperation(Enum):
    """
    Use the mosaic operation to resolve overlap pixel values: from first or last raster, use the min, max or mean of the pixel values, or blend them.
    """

    mt_blend = "MT_BLEND"
    mt_first = "MT_FIRST"
    mt_last = "MT_LAST"
    mt_max = "MT_MAX"
    mt_mean = "MT_MEAN"
    mt_min = "MT_MIN"
    mt_sum = "MT_SUM"


class OutputPixelType(Enum):
    """
    the output pixel type defines the output image's pixel type.
    """

    c128 = "C128"
    c64 = "C64"
    f32 = "F32"
    f64 = "F64"
    s16 = "S16"
    s32 = "S32"
    s8 = "S8"
    u1 = "U1"
    u16 = "U16"
    u2 = "U2"
    u32 = "U32"
    u4 = "U4"
    u8 = "U8"
    unknown = "UNKNOWN"


class Type2(Enum):
    """
    Type of the modification. `clip`: Anything inside of the polygon will be clipped out. `mask`: All areas outside of the polygon will be masked out. `replace`: Flattens or elevates the integrated mesh layer based on the polygon.
    """

    clip = "clip"
    mask = "mask"
    replace = "replace"


class DimensionalDefinition(BaseModel):
    """
    The dimensional definition defines a display filter based on one variable and one dimension. It is typically used when filtering data based on slices or ranges in one or more dimensions with [mosaicRule.description](mosaicRule.md). If the [multidimensionalSubset](multidimensionalSubset.md) is defined on the [TiledImageServiceLayer](tiledImageServiceLayer_schema.md) or the [mosaicRule](mosaicRule.md) then the multidimensionalDefinition must be within the defined multidimensionalSubset, otherwise nothing will be displayed.
    """

    model_config = common_config
    dimension_name: str | None = Field(
        None,
        alias="dimensionName",
        description="Type of dimension being used (ex. StdTime).",
    )
    is_slice: bool | None = Field(True, alias="isSlice", description="Is slice?")
    values: list[float | list[float]] | None = Field(
        None,
        description="Array where every element is either a single value, or an array of two values (lower and upper bound).",
    )
    variable_name: str = Field(
        ..., alias="variableName", description="Name of the variable."
    )


class DimensionalDefinition1(BaseModel):
    """
    The dimensional definition defines a display filter based on one variable and one dimension. It is typically used when filtering data based on slices or ranges in one or more dimensions with [mosaicRule.description](mosaicRule.md). If the [multidimensionalSubset](multidimensionalSubset.md) is defined on the [TiledImageServiceLayer](tiledImageServiceLayer_schema.md) or the [mosaicRule](mosaicRule.md) then the multidimensionalDefinition must be within the defined multidimensionalSubset, otherwise nothing will be displayed.
    """

    model_config = common_config
    dimension_name: str = Field(
        ...,
        alias="dimensionName",
        description="Type of dimension being used (ex. StdTime).",
    )
    is_slice: bool | None = Field(True, alias="isSlice", description="Is slice?")
    values: list[float | list[float]] = Field(
        ...,
        description="Array where every element is either a single value, or an array of two values (lower and upper bound).",
    )
    variable_name: str | None = Field(
        None, alias="variableName", description="Name of the variable."
    )


class WebMapTileServiceInfo(BaseModel):
    """
    Object defines and provides information about layers in a [WMTSLayer](webTiledLayer.md) service.
    """

    model_config = common_config
    custom_layer_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customLayerParameters",
        description="A sequence of parameters used to append different custom parameters to a WMTS tile request. These parameters are applied to `GetTile`. The `customLayerParameters` property takes precedence if `customParameters` is also present.",
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of parameters used to append custom parameters to all WMTS requests. These parameters are applied to `GetCapabilities` and `GetTile`. If used with the `customLayerParameters` property, `customParameters` will not take precedence.",
        title="customParameters",
    )
    layer_identifier: str | None = Field(
        None,
        alias="layerIdentifier",
        description="Identifier for the specific layer used in the WMTS service. Required input by the user.",
    )
    tile_matrix_set: str | None = Field(
        None,
        alias="tileMatrixSet",
        description="Tiling schema, set by the WMTS service.",
    )
    url: str | None = Field(
        None,
        description="URL to the WMTS web service. Required input by the user.",
    )


class WebFeatureServiceInfo(BaseModel):
    """
    Object that defines and provides information about layers in a WFS service.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of parameters used to append custom parameters to all WFS requests. These parameters are applied to `GetCapabilities`, `DescribeFeatureType`, and `GetFeatures`.",
    )
    feature_url: str | None = Field(
        None,
        alias="featureUrl",
        description="URL of the WFS service operation.",
    )
    max_features: int | None = Field(
        None,
        alias="maxFeatures",
        description="Set this to limit the number of requested features that a GetFeature request presents.",
    )
    name: str | None = Field(
        None,
        description="The name of the WFS layer. This is used to set layer visibility.",
    )
    supported_spatial_references: list[int] | None = Field(
        None,
        alias="supportedSpatialReferences",
        description="List of supported spatial reference IDs",
    )
    swap_xy: bool | None = Field(
        False,
        alias="swapXY",
        description="Boolean value indicating whether X and Y axis are flipped.",
    )
    version: str | None = Field(
        None,
        description="Value indicating which version of the WFS specification is used.",
    )
    wfs_namespace: str | None = Field(
        None,
        alias="wfsNamespace",
        description="String indicating namespace.",
    )


class WMSLayerLayer(BaseModel):
    """
    A layer object may allow overrides on popup content and drawing behavior for individual layers of a web service.
    """

    model_config = common_config
    legend_url: str | None = Field(
        None,
        alias="legendUrl",
        description="A string URL to a legend graphic for the layer.",
    )
    name: str | None = Field(
        None, description="A string containing a unique name for the layer."
    )
    queryable: bool | None = Field(
        None,
        description="Boolean specifying whether a layer is queryable or not.",
    )
    show_popup: bool | None = Field(
        None,
        alias="showPopup",
        description="Boolean specifying whether to display popup or not.",
    )
    title: str | None = Field(
        None,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )


class Description(BaseModel):
    """
    Description object with text.
    """

    model_config = common_config
    text: str = Field(..., description="Text to display as slide description.")


class Ground1(BaseModel):
    """
    Ground defines the main surface of the web scene, based on elevation layers. This object defines the ground properties to be set in the slides.
    """

    model_config = common_config
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="The transparency of the ground surface (including basemap). The value has to lie between `100` (full transparency) and `0` (full opacity).",
    )


class Thumbnail(BaseModel):
    """
    Object containing a thumbnail image.
    """

    model_config = common_config
    url: str = Field(
        ...,
        description="The URI pointing to the thumbnail image. Can be a URL or a base64-encoded image.",
    )


class Title(BaseModel):
    """
    Text for the title of the individual slide.
    """

    model_config = common_config
    text: str = Field(..., description="Text to display as slide title.")


class VisibleLayer(BaseModel):
    """
    Object with layer id, used to indicate layer visibility in a slide.
    """

    model_config = common_config
    id: str | None = Field(
        None,
        description="The id of the layer as listed on the operational layer.",
    )
    sub_layer_ids: list[int] | None = Field(
        None,
        alias="subLayerIds",
        description="List of visible sublayer ids, as listed on the service of the layer.",
    )


class FeatureExpression(BaseModel):
    """
    <em>Deprecated with 1.8, use `featureExpressionInfo` instead</em>. `{"value":0}` ignores geometry z-values.
    """

    model_config = common_config
    value: Literal[0] | None = None


class Mode(Enum):
    """
    Determines how the service elevation values are combined with the elevation of the scene.
    """

    absolute_height = "absoluteHeight"
    on_the_ground = "onTheGround"
    relative_to_ground = "relativeToGround"
    relative_to_scene = "relativeToScene"


class Type3(Enum):
    """
    A string defining the field type.
    """

    esri_field_type_big_integer = "esriFieldTypeBigInteger"
    esri_field_type_blob = "esriFieldTypeBlob"
    esri_field_type_date = "esriFieldTypeDate"
    esri_field_type_date_only = "esriFieldTypeDateOnly"
    esri_field_type_double = "esriFieldTypeDouble"
    esri_field_type_geometry = "esriFieldTypeGeometry"
    esri_field_type_global_id = "esriFieldTypeGlobalID"
    esri_field_type_guid = "esriFieldTypeGUID"
    esri_field_type_integer = "esriFieldTypeInteger"
    esri_field_type_long = "esriFieldTypeLong"
    esri_field_type_oid = "esriFieldTypeOID"
    esri_field_type_raster = "esriFieldTypeRaster"
    esri_field_type_single = "esriFieldTypeSingle"
    esri_field_type_small_integer = "esriFieldTypeSmallInteger"
    esri_field_type_string = "esriFieldTypeString"
    esri_field_type_time_only = "esriFieldTypeTimeOnly"
    esri_field_type_timestamp_offset = "esriFieldTypeTimestampOffset"
    esri_field_type_xml = "esriFieldTypeXML"


class FloorInfo(BaseModel):
    """
    Contains floor-awareness information for a layer.
    """

    model_config = common_config
    floor_field: str = Field(
        ...,
        alias="floorField",
        description="The name of the attribute field that contains a floor's level ID used for floor filtering.",
    )


class SpatialRelationship(Enum):
    """
    Specifies the spatial relationship used for the filter. `disjoint`: Display features that do not intersect any filter polygon. `contains`: Display features completely inside any filter polygon
    """

    contains = "contains"
    disjoint = "disjoint"


class PolygonFilter(BaseModel):
    """
    Filter features using polygons stored in external resources.
    """

    model_config = common_config

    geometries: str = Field(
        ...,
        description="URL to a polygon filter geometries json file, typically stored in `ITEM/resources`. Content of the file follows the $ref:[polygon filter geometries schema](polygonFilterGeometries_schema.json).",
    )
    spatial_relationship: SpatialRelationship = Field(
        ...,
        alias="spatialRelationship",
        description="Specifies the spatial relationship used for the filter. `disjoint`: Display features that do not intersect any filter polygon. `contains`: Display features completely inside any filter polygon",
    )


class RangeInformation(BaseModel):
    """
    Range Information.
    """

    model_config = common_config
    current_range_extent: list[float] | None = Field(
        None,
        alias="currentRangeExtent",
        description="Contains the min and max values within which the features are visible.",
        max_length=2,
        min_length=2,
    )
    field: str = Field(..., description="Field name to used for the range.")
    full_range_extent: list[float] | None = Field(
        None,
        alias="fullRangeExtent",
        description="Contains the min and max values of all the features for this rangeInfo.",
        max_length=2,
        min_length=2,
    )
    name: str = Field(
        ...,
        description="A unique name that can be referenced by webMap.activeRanges.",
    )
    type: Literal["rangeInfo"] = Field("rangeInfo", description="Type of range object.")


class ReturnType(Enum):
    """
    Return type of the Arcade expression, can be number or string. Defaults to string value. Number values are assumed to be `double`. This can be determined by the authoring client by executing the expression using a sample feature(s), although it can be corrected by the user. Knowing the returnType allows the authoring client to present fields in relevant contexts. For example, numeric fields in numeric contexts such as charts.
    """

    number = "number"
    string = "string"


class PopupExpressionInfo(BaseModel):
    """
    Arcade expression added to the pop-up.
    """

    model_config = common_config
    expression: str | None = Field(None, description="The Arcade expression.")
    name: str | None = Field(None, description="Unique identifier for the expression.")
    return_type: ReturnType | None = Field(
        None,
        alias="returnType",
        description="Return type of the Arcade expression, can be number or string. Defaults to string value. Number values are assumed to be `double`. This can be determined by the authoring client by executing the expression using a sample feature(s), although it can be corrected by the user. Knowing the returnType allows the authoring client to present fields in relevant contexts. For example, numeric fields in numeric contexts such as charts.",
    )
    title: str | None = Field(None, description="Title of the expression.")


class StatisticType(Enum):
    """
    Used in a 1:many or many:many relationship to compute the statistics on the field to show in the popup.
    """

    avg = "avg"
    count = "count"
    max = "max"
    min = "min"
    stddev = "stddev"
    sum = "sum"
    var = "var"


class StringFieldOption(Enum):
    """
    A string determining what type of input box editors see when editing the field. Applies only to string fields. Not applicable to Arcade expressions.
    """

    richtext = "richtext"
    textarea = "textarea"
    textbox = "textbox"


class LayerOptions(BaseModel):
    """
    Additional options available for the popup layer.
    """

    model_config = common_config

    return_topmost_raster: bool | None = Field(
        None,
        alias="returnTopmostRaster",
        description="Indicates whether or not only the topmost raster should be displayed.",
    )
    show_no_data_records: bool | None = Field(
        None,
        alias="showNoDataRecords",
        description="Indicates whether or not the NoData records should be displayed.",
    )


class SunLighting(BaseModel):
    """
    Object containing information for the sun lighting type. The position of the light is set to the sun's location.
    """

    model_config = common_config
    datetime: float | None = Field(
        None,
        description="The current date and time of the simulated sun. It is a number representing the number of milliseconds since 1 January, 1970 UTC.",
    )
    direct_shadows: bool | None = Field(
        False,
        alias="directShadows",
        description="Indicates whether to show shadows cast by the main light source.",
    )
    display_utc_offset: float | None = Field(
        None,
        alias="displayUTCOffset",
        description="UTC offset in decimal hours. Not to be applied to datetime for sun position, only to adjust display of datetime in UI. If displayUTCOffset is null, offset is calculated for the current location (approximate only).",
    )
    type: Literal["sun"] | None = Field("sun", description="The type of lighting")


class VirtualLighting(BaseModel):
    """
    Object containing information for the virtual lighting type. The position of the light follows the camera and is set behind the camera with a small offset to the left side.
    """

    model_config = common_config
    direct_shadows: bool | None = Field(
        False,
        alias="directShadows",
        description="Indicates whether to show shadows cast by the main light source.",
    )
    type: Literal["virtual"] = Field("virtual", description="The type of lighting")


class PointGeometry(BaseModel):
    """
    Defines the JSON formats of the point and spatial reference objects.
    """

    model_config = common_config
    m: float | None = Field(
        None,
        description="M coordinate which contains measures used for linear referencing.",
    )
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",
        title="spatialReference",
    )
    x: float | Literal["NaN"] | None = Field(
        ...,
        description="X coordinate which is measured along the east/west axis.",
    )
    y: float | None = Field(
        None,
        description="Y coordinate which is measured along the north/south axis.",
    )
    z: float | None = Field(
        None, description="Z coordinate which measures height or elevation."
    )


class MultipointGeometry(BaseModel):
    """
    Contains an array of points, along with a spatial reference field.
    """

    model_config = common_config
    has_m: bool | None = Field(
        None,
        alias="hasM",
        description="Indicates whether the geometry contains M coordinate values.",
    )
    has_z: bool | None = Field(
        None,
        alias="hasZ",
        description="Indicates whether the geometry contains Z coordinate values.",
    )
    points: list[list[float]] = Field(
        ..., description="An array that corresponds to 2D and 3D points."
    )
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",
        title="spatialReference",
    )


class PolygonGeometry(BaseModel):
    """
    A polygon contains an array of rings and a spatial reference.
    """

    model_config = common_config
    has_m: bool | None = Field(
        None,
        alias="hasM",
        description="Indicates whether the geometry contains M coordinate values.",
    )
    has_z: bool | None = Field(
        None,
        alias="hasZ",
        description="Indicates whether the geometry contains Z coordinate values.",
    )
    rings: list[list[Any]] = Field(
        ...,
        description="Represents an array of rings. Each ring is an array of points.",
        min_length=1,
    )
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",
        title="spatialReference",
    )


class PolylineGeometry(BaseModel):
    """
    Contains an array of paths and a spatialReference.
    """

    model_config = common_config
    has_m: bool | None = Field(
        None,
        alias="hasM",
        description="Indicates whether the geometry contains M coordinate values.",
    )
    has_z: bool | None = Field(
        None,
        alias="hasZ",
        description="Indicates whether the geometry contains Z coordinate values.",
    )
    paths: list[list[list[Any]]] = Field(
        ...,
        description="Three nested array levels describing a polyline. The first level describes individual paths; the second describes the points of these paths; and the third gives the coordinates of the points.",
    )
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",
        title="spatialReference",
    )


class LabelPlacement(Enum):
    """
    This string property specifies the label placement with respect to that of its feature. A list of label placement values categorized by feature geometry types.
    """

    esri_server_line_placement_above_after = "esriServerLinePlacementAboveAfter"
    esri_server_line_placement_above_along = "esriServerLinePlacementAboveAlong"
    esri_server_line_placement_above_before = "esriServerLinePlacementAboveBefore"
    esri_server_line_placement_above_end = "esriServerLinePlacementAboveEnd"
    esri_server_line_placement_above_start = "esriServerLinePlacementAboveStart"
    esri_server_line_placement_below_after = "esriServerLinePlacementBelowAfter"
    esri_server_line_placement_below_along = "esriServerLinePlacementBelowAlong"
    esri_server_line_placement_below_before = "esriServerLinePlacementBelowBefore"
    esri_server_line_placement_below_end = "esriServerLinePlacementBelowEnd"
    esri_server_line_placement_below_start = "esriServerLinePlacementBelowStart"
    esri_server_line_placement_center_after = "esriServerLinePlacementCenterAfter"
    esri_server_line_placement_center_along = "esriServerLinePlacementCenterAlong"
    esri_server_line_placement_center_before = "esriServerLinePlacementCenterBefore"
    esri_server_line_placement_center_end = "esriServerLinePlacementCenterEnd"
    esri_server_line_placement_center_start = "esriServerLinePlacementCenterStart"
    esri_server_point_label_placement_above_center = (
        "esriServerPointLabelPlacementAboveCenter"
    )
    esri_server_point_label_placement_above_left = (
        "esriServerPointLabelPlacementAboveLeft"
    )
    esri_server_point_label_placement_above_right = (
        "esriServerPointLabelPlacementAboveRight"
    )
    esri_server_point_label_placement_below_center = (
        "esriServerPointLabelPlacementBelowCenter"
    )
    esri_server_point_label_placement_below_left = (
        "esriServerPointLabelPlacementBelowLeft"
    )
    esri_server_point_label_placement_below_right = (
        "esriServerPointLabelPlacementBelowRight"
    )
    esri_server_point_label_placement_center_center = (
        "esriServerPointLabelPlacementCenterCenter"
    )
    esri_server_point_label_placement_center_left = (
        "esriServerPointLabelPlacementCenterLeft"
    )
    esri_server_point_label_placement_center_right = (
        "esriServerPointLabelPlacementCenterRight"
    )
    esri_server_polygon_placement_always_horizontal = (
        "esriServerPolygonPlacementAlwaysHorizontal"
    )


class FunctionType(Enum):
    """
    Defines whether the `function` is applied to a mosaic dataset. Indicates the level of the mosaic processing is used. Only applies to mosaic based image services. `0` - function is applied after mosaicking; `1` - function is applied on each raster item before mosaicking; `2` - function is applied to a group of raster items before mosaicking.
    """

    number_0 = 0
    number_1 = 1
    number_2 = 2


class Properties(BaseModel):
    """
    The properties used to define multidimensional dataset processing rules.
    """

    model_config = common_config

    field_object_id: int | None = Field(None, alias="_object_id")
    type: str


class FeatureReference(BaseModel):
    """
    References a feature by object and layer id.
    """

    model_config = common_config
    layer_id: float | str = Field(
        ...,
        alias="layerId",
        description="Identifies the layer to which the feature belongs.",
    )
    object_id: str | float = Field(
        ...,
        alias="objectId",
        description="ObjectId identifying the feature within the layer.",
    )


class ControlPoint(BaseModel):
    """
    A location in pixels in the media of the [MediaLayer](mediaLayer.md). The origin is located at the top-left corner of the media.
    """

    model_config = common_config
    x: float = Field(
        ...,
        description="The X coordinate in pixels. The value must be between 0, and the width of the [georeference](mediaLayer_georeference_schema.md)",
    )
    y: float = Field(
        ...,
        description="The Y coordinate in pixels. The value must be between 0, and the height of the [georeference](mediaLayer_georeference_schema.md)",
    )


class RenderMode(Enum):
    """
    Current rendering mode for the voxel layer. Depending on the rendering mode different voxel layer representations can be shown. `volume` draws the voxel layer as rectangular cuboid (but any slices defined for the voxel layer will change the volume to the area of interest). `surfaces` will represent the layer as a set of surfaces, for example, dynamic sections can define a plane through the volume or isosurfaces can show a specific value as surface.
    """

    surfaces = "surfaces"
    volume = "volume"


class Lod(BaseModel):
    model_config = common_config
    level: int | None = Field(None, description="ID for each level.")
    level_value: str | None = Field(
        None,
        alias="levelValue",
        description="String to be used when constructing URL to access a tile from this LOD.",
    )
    resolution: float | None = Field(
        None,
        description="Resolution in map units of each pixel in a tile for each level.",
    )
    scale: float | None = Field(None, description="Scale for each level.")


class FeatureExpressionInfo(BaseModel):
    """
    An object that defines an expression for per-feature elevation.
    """

    model_config = common_config
    expression: constr(min_length=1) = Field(
        ...,
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number.",
    )
    title: constr(min_length=1) | None = Field(
        None, description="Title of the expression."
    )


class FeatureReductionSelection(BaseModel):
    """
    Feature reduction of type `selection` declutters the screen by hiding features that would otherwise intersect with other features on screen. The default behavior of this feature reduction type is to select features by depth order, i.e. hide all features that would otherwise be overlapped by at least one other feature which is closer to the viewer.
    """

    model_config = common_config

    type: Literal["selection"] | None = Field(
        None,
        description="A string value indicating the feature reduction type.",
    )


class PointCloudBitfieldFilter(BaseModel):
    """
    Filters points based on the value of the specified bitfield attribute.
    """

    model_config = common_config
    field: str = Field(
        ..., description="The name of the field that is used for the filter."
    )
    required_clear_bits: list[int] = Field(
        ...,
        alias="requiredClearBits",
        description="List ALL bit numbers that must cleared (=0) for the point to be kept. bit 0 is LSB.",
    )
    required_set_bits: list[int] | None = Field(
        None,
        alias="requiredSetBits",
        description=" List ALL bit numbers that must set (=1) for the point to be kept. bit 0 is LSB.",
    )
    type: Literal["pointCloudBitfieldFilter"] = "pointCloudBitfieldFilter"


class PointCloudBitfieldFilter1(BaseModel):
    """
    Filters points based on the value of the specified bitfield attribute.
    """

    model_config = common_config
    field: str = Field(
        ..., description="The name of the field that is used for the filter."
    )
    required_clear_bits: list[int] | None = Field(
        None,
        alias="requiredClearBits",
        description="List ALL bit numbers that must cleared (=0) for the point to be kept. bit 0 is LSB.",
    )
    required_set_bits: list[int] = Field(
        ...,
        alias="requiredSetBits",
        description=" List ALL bit numbers that must set (=1) for the point to be kept. bit 0 is LSB.",
    )
    type: Literal["pointCloudBitfieldFilter"] = "pointCloudBitfieldFilter"


class IncludedReturn(Enum):
    first_of_many = "firstOfMany"
    last = "last"
    last_of_many = "lastOfMany"
    single = "single"


class PointCloudReturnFilter(BaseModel):
    """
    Filters points based on the value of the return number/return count.
    """

    model_config = common_config
    field: str = Field(
        ..., description="The name of the field that is used for the filter."
    )
    included_returns: list[IncludedReturn] = Field(
        ...,
        alias="includedReturns",
        description="All points with at least one specified return status will be kept. Status values: `last`, `firstOfMany`, `lastOfMany`, `single`",
    )
    type: Literal["pointCloudReturnFilter"] = "pointCloudReturnFilter"


class Mode1(Enum):
    """
    Defines if values should be included or excluded.
    """

    exclude = "exclude"
    include = "include"


class PointCloudValueFilter(BaseModel):
    """
    Filters points based on the value of an specified attribute.
    """

    model_config = common_config
    field: str = Field(
        ..., description="The name of the field that is used for the filter."
    )
    mode: Mode1 = Field(
        ..., description="Defines if values should be included or excluded."
    )
    type: Literal["pointCloudValueFilter"] = Field(
        "pointCloudValueFilter",
        description="Filters points based on the value of an specified attribute.",
    )
    values: list[float] = Field(..., description="list of values")


class DynamicMapLayer(BaseModel):
    """
    A dynamic map layer refers to a layer in the current map service. More information on this can be found in the [ArcGIS REST API help](http://resources.arcgis.com/en/help/rest/apiref/layersource.html).
    """

    model_config = common_config
    gdb_version: str | None = Field(
        None,
        alias="gdbVersion",
        description="If applicable, specify this to use an alternate geodatabase version.",
    )
    map_layer_id: int = Field(
        ..., alias="mapLayerId", description="The current map layer's id."
    )
    type: Literal["mapLayer"] = Field(
        "mapLayer", description="A string value indicating the type."
    )


class DrawingTool(Enum):
    """
    An optional string that can define a client-side drawing tool to be used with this feature.
    """

    esri_feature_edit_tool_auto_complete_polygon = (
        "esriFeatureEditToolAutoCompletePolygon"
    )
    esri_feature_edit_tool_circle = "esriFeatureEditToolCircle"
    esri_feature_edit_tool_down_arrow = "esriFeatureEditToolDownArrow"
    esri_feature_edit_tool_ellipse = "esriFeatureEditToolEllipse"
    esri_feature_edit_tool_freehand = "esriFeatureEditToolFreehand"
    esri_feature_edit_tool_left_arrow = "esriFeatureEditToolLeftArrow"
    esri_feature_edit_tool_line = "esriFeatureEditToolLine"
    esri_feature_edit_tool_none = "esriFeatureEditToolNone"
    esri_feature_edit_tool_point = "esriFeatureEditToolPoint"
    esri_feature_edit_tool_polygon = "esriFeatureEditToolPolygon"
    esri_feature_edit_tool_rectangle = "esriFeatureEditToolRectangle"
    esri_feature_edit_tool_right_arrow = "esriFeatureEditToolRightArrow"
    esri_feature_edit_tool_text = "esriFeatureEditToolText"
    esri_feature_edit_tool_triangle = "esriFeatureEditToolTriangle"
    esri_feature_edit_tool_up_arrow = "esriFeatureEditToolUpArrow"


class DateFormat(Enum):
    """
    A string used with date fields to specify how the date should be formatted.
    """

    day_short_month_year = "dayShortMonthYear"
    day_short_month_year_long_time = "dayShortMonthYearLongTime"
    day_short_month_year_long_time24 = "dayShortMonthYearLongTime24"
    day_short_month_year_short_time = "dayShortMonthYearShortTime"
    day_short_month_year_short_time24 = "dayShortMonthYearShortTime24"
    long_date = "longDate"
    long_date_long_time = "longDateLongTime"
    long_date_long_time24 = "longDateLongTime24"
    long_date_short_time = "longDateShortTime"
    long_date_short_time24 = "longDateShortTime24"
    long_month_day_year = "longMonthDayYear"
    long_month_day_year_long_time = "longMonthDayYearLongTime"
    long_month_day_year_long_time24 = "longMonthDayYearLongTime24"
    long_month_day_year_short_time = "longMonthDayYearShortTime"
    long_month_day_year_short_time24 = "longMonthDayYearShortTime24"
    long_month_year = "longMonthYear"
    short_date = "shortDate"
    short_date_le = "shortDateLE"
    short_date_le_long_time = "shortDateLELongTime"
    short_date_le_long_time24 = "shortDateLELongTime24"
    short_date_le_short_time = "shortDateLEShortTime"
    short_date_le_short_time24 = "shortDateLEShortTime24"
    short_date_long_time = "shortDateLongTime"
    short_date_long_time24 = "shortDateLongTime24"
    short_date_short_time = "shortDateShortTime"
    short_date_short_time24 = "shortDateShortTime24"
    short_month_year = "shortMonthYear"
    year = "year"


class Format2(BaseModel):
    """
    The format object can be used with numerical or date fields to provide more detail about how values should be formatted for display.
    """

    model_config = common_config
    date_format: DateFormat | None = Field(
        None,
        alias="dateFormat",
        description="A string used with date fields to specify how the date should be formatted.",
    )
    digit_separator: bool | None = Field(
        None,
        alias="digitSeparator",
        description="A Boolean used with numerical fields. A value of true allows the number to have a digit (or thousands) separator. Depending on the locale, this separator is a decimal point or a comma. A value of false means that no separator will be used.",
    )
    places: int | None = Field(
        None,
        description="An integer used with numerical fields to specify the number of decimal places. Any places beyond this value are rounded.",
    )


class Type5(Enum):
    """
    A string defining the type of media.
    """

    barchart = "barchart"
    columnchart = "columnchart"
    linechart = "linechart"
    piechart = "piechart"


class DisplayType(Enum):
    """
    This property applies to elements of type `attachments`. A string value indicating how to display the attachment. If `list` is specified, attachments show as links. If `preview` is specified, attachments expand to the width of the pop-up. Setting the value to `auto` allows applications to choose the most suitable default experience for their application.
    """

    auto = "auto"
    list = "list"
    preview = "preview"


class PopupElementAttachments(BaseModel):
    """
    Configures attachments in popup elements.
    """

    model_config = common_config
    description: str | None = Field(
        None,
        description="An optional string value describing the element in detail. This property does not apply when the `type` is `text` or `expression`.",
    )
    display_type: DisplayType | None = Field(
        None,
        alias="displayType",
        description="This property applies to elements of type `attachments`. A string value indicating how to display the attachment. If `list` is specified, attachments show as links. If `preview` is specified, attachments expand to the width of the pop-up. Setting the value to `auto` allows applications to choose the most suitable default experience for their application. ",
    )
    title: str | None = Field(
        None,
        description="An optional string value indicating what the element represents. This property does not apply when the `type` is `text` or `expression`.",
    )
    type: Literal["attachments"] = "attachments"


class PopupElementText(BaseModel):
    """
    Configures text in popup elements.
    """

    model_config = common_config
    text: str | None = Field(
        None,
        description="This property applies to elements of type `text`. This is string value indicating the text to be displayed within the popupElement. If no `text` property is provided, the popupElement will display whatever is specified in the [popupInfo.description](popupInfo.md) property.",
    )
    type: Literal["text"] = "text"


class Order(Enum):
    """
    Set the ascending or descending sort order of the returned related records.
    """

    asc = "asc"
    desc = "desc"


class OrderByField(BaseModel):
    """
    Object defining the display order of features or records based on a field value, and whether they should be sorted in ascending or descending order.
    """

    model_config = common_config

    field: str | None = Field(
        None,
        description="Name of a field. The value of this field will drive the sorting.",
    )
    order: Order | None = Field(
        None,
        description="Indicates whether features or records are sorted in ascending or descending order of the field values.",
    )


class Type6(Enum):
    """
    A string defining the field type.
    """

    esri_field_type_big_integer = "esriFieldTypeBigInteger"
    esri_field_type_blob = "esriFieldTypeBlob"
    esri_field_type_date = "esriFieldTypeDate"
    esri_field_type_date_only = "esriFieldTypeDateOnly"
    esri_field_type_double = "esriFieldTypeDouble"
    esri_field_type_geometry = "esriFieldTypeGeometry"
    esri_field_type_global_id = "esriFieldTypeGlobalID"
    esri_field_type_guid = "esriFieldTypeGUID"
    esri_field_type_integer = "esriFieldTypeInteger"
    esri_field_type_long = "esriFieldTypeLong"
    esri_field_type_oid = "esriFieldTypeOID"
    esri_field_type_raster = "esriFieldTypeRaster"
    esri_field_type_single = "esriFieldTypeSingle"
    esri_field_type_small_integer = "esriFieldTypeSmallInteger"
    esri_field_type_string = "esriFieldTypeString"
    esri_field_type_time_only = "esriFieldTypeTimeOnly"
    esri_field_type_timestamp_offset = "esriFieldTypeTimestampOffset"
    esri_field_type_xml = "esriFieldTypeXML"


class SearchField(BaseModel):
    """
    Contains information about an attribute field in layer and table search configurations.
    """

    model_config = common_config
    exact_match: bool | None = Field(
        None,
        alias="exactMatch",
        description="A Boolean defining whether or not the field is an exact match.",
    )
    name: str | None = Field(None, description="A string defining the field name.")
    type: Type6 | None = Field(None, description="A string defining the field type.")


class LabelExpressionInfo(BaseModel):
    """
    The labelExpressionInfo allows label text to be read similar to that of Popups's description property.
    """

    model_config = common_config
    expression: str | None = Field(
        None,
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to either a string or a number.",
    )


class NormalizationType(Enum):
    """
    Determine how the data was normalized.
    """

    esri_normalize_by_field = "esriNormalizeByField"
    esri_normalize_by_log = "esriNormalizeByLog"
    esri_normalize_by_percent_of_total = "esriNormalizeByPercentOfTotal"


class RasterFunctionTemplateArguments(BaseModel):
    """
    Raster function template or raster function variable used as argument.
    """

    model_config = common_config

    field_object_id: int | None = Field(
        None,
        alias="_object_id",
        description="The id of the raster function template argument.",
    )
    type: str = Field(..., description="Type of the raster function argument.")


class RasterFunctionTemplateArguments1(BaseModel):
    """
    Raster function template by reference
    """

    model_config = common_config
    field_object_ref_id: int = Field(
        ...,
        alias="_object_ref_id",
        description="The id of the raster function template by reference.",
    )


class PixelType1(Enum):
    """
    Pixel type of the output image processed by the raster function template.
    """

    c128 = "C128"
    c64 = "C64"
    f32 = "F32"
    f64 = "F64"
    s16 = "S16"
    s32 = "S32"
    s8 = "S8"
    u1 = "U1"
    u16 = "U16"
    u2 = "U2"
    u32 = "U32"
    u4 = "U4"
    u8 = "U8"
    unknown = "UNKNOWN"


class RasterFunctionInformation(BaseModel):
    """
    Information about the `function` referenced in a raster function template.
    """

    model_config = common_config
    field_object_id: int | None = Field(
        None,
        alias="_object_id",
        description="Optional. The id of the raster function info.",
    )
    description: str | None = Field(
        None, description="Description of the raster function."
    )
    name: str = Field(
        ...,
        description="Name of the raster function used by the raster function template.",
    )
    pixel_type: PixelType1 = Field(
        ...,
        alias="pixelType",
        description="Pixel type of the output image processed by the raster function template.",
    )
    type: str = Field(
        ...,
        description="Type of the raster function used by the raster function template.",
    )


class VoxelSlice(BaseModel):
    """
    A slice is a plane through the voxel layer. Slices visually cut the voxel layer by removing portion of it. A slice or a combination of slices can define an area of interest showing only a portion of the original extent or show the entire of a voxel layer.
    """

    model_config = common_config
    enabled: bool | None = Field(True, description="Boolean defining slice visibility.")
    label: str | None = Field(None, description="Label for the slice.")
    normal: list[float] = Field(
        ...,
        description="Normal vector to the plane in voxel space. Defining the orientation of the slice.",
        max_length=3,
        min_length=3,
    )
    point: list[float] = Field(
        ...,
        description="Point belonging to the section plane in voxel space. Defining the position of the slice.",
        max_length=3,
        min_length=3,
    )


class VoxelShading(BaseModel):
    """
    Describes the shading properties of the voxel layer. Voxel layers use a simple shading model which provides specular highlights and diffuse shading proportional to the opacity of the voxel being accumulated during raycasting. Opaque voxels are shaded like a surface while semi-transparent voxels mostly contribute their color.
    """

    model_config = common_config
    diffuse_factor: confloat(ge=0.0, le=1.0) | None = Field(
        0.5, alias="diffuseFactor", description="Diffuse light coefficient."
    )
    specular_factor: confloat(ge=0.0, le=1.0) | None = Field(
        0.5,
        alias="specularFactor",
        description="Specular highlight coefficient.",
    )


class ExaggerationMode(Enum):
    """
    Determines how the vertical exaggeration is applied. `scale-height` scales from the voxel dataset origin only, for example, if a voxel layer has its minimum at sea level the layer will be exaggerated starting from sea level. `scale-height` is the default. `scale-position` also scales the space between voxel dataset origin in the coordinate system origin. This exaggeration mode is identical with exaggeration applied to other layers like feature layers (use the scale position option if you want to draw the voxel layer together with feature based data).
    """

    scale_height = "scale-height"
    scale_position = "scale-position"


class InheritedDomain(BaseModel):
    """
    This domain applies to domains on subtypes. It implies that the domain for a field at the subtype level is the same as the domain for the field at the layer level.
    """

    model_config = common_config
    name: str | None = Field(None, description="The domain name.")
    type: Literal["inherited"] = Field(
        "inherited", description="String value representing the domain type."
    )


class RangeDomain(BaseModel):
    """
    Range domain specifies a range of valid values for a field.
    """

    model_config = common_config
    name: str | None = Field(None, description="The domain name.")
    range: list[float] = Field(
        ...,
        description="The first element is the minValue and the second element is the maxValue.",
        max_length=2,
        min_length=2,
    )
    type: Literal["range"] = Field(
        "range", description="String value representing the domain type."
    )


class MediaInfoChartValue(BaseModel):
    """
    The value object contains information for popup windows about how charts should be constructed.
    """

    model_config = common_config
    colors: list[list[conint(ge=0, le=255)]] | None = Field(
        None,
        description="Used with charts. An optional array of colors where each `color` sequentially corresponds to a field in the `fields` property. When the value for `mediaInfo.type` is `linechart`, the first color in the array will drive the line color. If `colors` is longer than `fields`, unmatched colors are ignored. If `colors` is shorter than `fields` or `colors` isn't specified, a default color ramp is applied",
    )
    fields: list[str] = Field(
        ...,
        description="An array of strings, with each string containing the name of a field to display in the chart.",
    )
    normalize_field: str | None = Field(
        None,
        alias="normalizeField",
        description="An optional string containing the name of a field. The values of all fields in the chart will be normalized (divided) by the value of this field.",
    )
    tooltip_field: str | None = Field(
        None,
        alias="tooltipField",
        description="String value indicating the tooltip for a chart specified from another field. This field is needed when related records are not used. It is used for showing tooltips from another field in the same layer or related layer/table.",
    )


class MediaInfoImageValue(BaseModel):
    """
    The value object contains information for popup windows about how images should be retrieved.
    """

    model_config = common_config
    link_url: str | None = Field(
        None,
        alias="linkURL",
        description="A string containing a URL to be launched in a browser when a user clicks the image.",
    )
    source_url: str = Field(
        ...,
        alias="sourceURL",
        description="A string containing the URL to the image.",
    )


class PopupElementExpressionInfo(BaseModel):
    """
    An Arcade expression that defines the pop-up element content. The return type will always be a `dictionary` that defines the desired pop-up element as outlined [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element).
    """

    model_config = common_config
    expression: str | None = Field(None, description="The Arcade expression.")
    return_type: Literal["dictionary"] | None = Field(
        None,
        alias="returnType",
        description="Return type of the Arcade expression, the expected return is always `dictionary.`",
    )
    title: str | None = Field(None, description="Title of the expression.")


class FilterType(BaseModel):
    """
    The file authoring information for a filter, including the filter type and its value settings.
    """

    model_config = common_config
    filter_type: str = Field(
        ...,
        alias="filterType",
        description="Represents the filter type name. Name is a unique identifier.",
    )
    filter_values: list[str] = Field(
        ...,
        alias="filterValues",
        description="Array of filter values. Filter values are the attributes that can be stored for individual fields in a layer.",
    )


class FilterModeSolid(BaseModel):
    """
    Draw elements of this filter block in solid mode. This mode does not change the display of features.
    """

    model_config = common_config
    type: Literal["solid"] = Field(
        "solid", description="Declares filter mode of type solid."
    )


class FilterModeXRay(BaseModel):
    """
    Draw elements of this filter block in x-ray mode. `x-ray` renders elements semi-transparent with white color.
    """

    model_config = common_config
    type: Literal["x-ray"] = Field(
        "x-ray", description="Declares filter mode of type x-ray."
    )


class HorizontalAlignment(Enum):
    """
    One of the following string values representing the horizontal alignment of the text.
    """

    center = "center"
    left = "left"
    right = "right"


class VerticalAlignment(Enum):
    """
    One of the following string values representing the vertical alignment of the text.
    """

    baseline = "baseline"
    bottom = "bottom"
    middle = "middle"
    top = "top"


class VerticalOffset(BaseModel):
    """
    Shifts the symbol along the vertical world axis by a given length. The length is set in screen space units.
    """

    model_config = common_config
    max_world_length: float | None = Field(
        None,
        alias="maxWorldLength",
        description="The maximum vertical symbol lift in world units. It acts as an upper bound to avoid lift becoming too big.",
    )
    min_world_length: float | None = Field(
        0,
        alias="minWorldLength",
        description="The minimum vertical symbol lift in world units. It acts as a lower bound to avoid lift becoming too small.",
    )
    screen_length: float = Field(
        ...,
        alias="screenLength",
        description="Maximal screen length of lift in points.",
    )


class ClassificationMethod(Enum):
    """
    Used for classed color or size. The default value is `esriClassifyManual`.
    """

    esri_classify_defined_interval = "esriClassifyDefinedInterval"
    esri_classify_equal_interval = "esriClassifyEqualInterval"
    esri_classify_manual = "esriClassifyManual"
    esri_classify_natural_breaks = "esriClassifyNaturalBreaks"
    esri_classify_quantile = "esriClassifyQuantile"
    esri_classify_standard_deviation = "esriClassifyStandardDeviation"


class Focus(Enum):
    """
    Optional. Used for Relationship renderer. If not set, the legend will default to being square.
    """

    hh = "HH"
    hl = "HL"
    lh = "LH"
    ll = "LL"


class LengthUnit(Enum):
    """
    Unit used in user interfaces to display world/map sizes and distances
    """

    centimeters = "centimeters"
    decimal_degrees = "decimal-degrees"
    decimeters = "decimeters"
    feet = "feet"
    inches = "inches"
    kilometers = "kilometers"
    meters = "meters"
    miles = "miles"
    millimeters = "millimeters"
    nautical_miles = "nautical-miles"
    yards = "yards"


class StandardDeviationInterval(Enum):
    """
    Use this property if the classificationMethod is `esriClassifyStandardDeviation`.
    """

    number_0_25 = 0.25
    number_0_33 = 0.33
    number_0_5 = 0.5
    number_1 = 1


class Type7(Enum):
    classed_color = "classedColor"
    classed_size = "classedSize"
    dot_density = "dotDensity"
    predominance = "predominance"
    relationship = "relationship"
    univariate_color_size = "univariateColorSize"


class RendererLegendOptions(BaseModel):
    """
    Options available for the renderer legend.
    """

    model_config = common_config
    title: str | None = Field(None, description="The title of the legend.")


class HeatmapColorStop(BaseModel):
    """
    A colorStop object describes the renderer's color ramp and how it is mapped to the intensity values. The color of the first stop (i.e. the stop with the lowest ratio value) must have an alpha value of 0 for the underlying basemap to be visible in the app.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    ratio: float = Field(
        ...,
        description="A number between 0-1. Describes what portion along the gradient the colorStop is added.",
    )


class LegendOptions1(BaseModel):
    """
    Options available for the legend.
    """

    model_config = common_config
    max_label: str | None = Field(
        None,
        alias="maxLabel",
        description="Text in the legend that describes the hottest (most dense) part of the heatmap. Only applicable to Heatmap renderers. If not specified, then a localized label, for 'High' will display on the legend.",
    )
    min_label: str | None = Field(
        None,
        alias="minLabel",
        description="Text in the legend that describes the coldest (least dense) part of the heatmap. Only applicable to Heatmap renderers. If not specified, then a localized label, for 'Low' will display on the legend.",
    )
    title: str | None = Field(None, description="The title of the legend.")


class FieldTransformType(Enum):
    """
    A transform that is applied to the field value before evaluating the renderer.
    """

    absolute_value = "absoluteValue"
    high_four_bit = "highFourBit"
    low_four_bit = "lowFourBit"
    modulo_ten = "moduloTen"
    none = "none"


class HillshadeType(Enum):
    """
    Use single (traditional), or multiple illumination sources to generate hillshade.
    """

    multi_directional = "multi-directional"
    traditional = "traditional"


class ScalingType(Enum):
    """
    Apply a constant or adjusted z-factor based on resolution changes. The `adjusted` type is suitable for worldwide elevation dataset. An adjusted `zFactor` is determined using this equation: `Adjusted Z-Factor = (Z Factor) + (Pixel Size)` <sup>(Pixel Size Power)</sup> `x Pixel Size Factor`.
    """

    adjusted = "adjusted"
    none = "none"


class StretchType(Enum):
    """
    The stretch types for stretch raster function.
    """

    histogram_equalization = "histogramEqualization"
    min_max = "minMax"
    none = "none"
    percent_clip = "percentClip"
    sigmoid = "sigmoid"
    standard_deviation = "standardDeviation"


class VoxelIsosurfaceStyle(BaseModel):
    """
    The isosurface style describes the value and coloring of the isosurface. For example, for a given variable temperature you can define up to four isosurfaces based on a temperature value.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    color_locked: bool | None = Field(
        False,
        alias="colorLocked",
        description="If false the isosurface color is automatically updated when the variable's colorStops are modified.",
    )
    enabled: bool | None = Field(
        True,
        description="Determines if the isosurface should be shown or hidden.",
    )
    label: str | None = Field(None, description="Label for the isosurface.")
    value: float = Field(..., description="Value of the variable.")


class Interpolation2(Enum):
    """
    Interpolation mode
    """

    linear = "linear"
    nearest = "nearest"


class VoxelUniqueValue(BaseModel):
    """
    Describes the unique value of a variable with a discrete data type.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    enabled: bool | None = Field(
        True, description="Show or hide all voxels equal to this value."
    )
    label: str | None = Field(None, description="Label for the unique value.")
    value: int = Field(..., description="Unique value of the variable.")


class VoxelDynamicSection(BaseModel):
    """
    A section is a plane through the voxel layer. A section can be moved by changing the position and orientation.
    """

    model_config = common_config
    enabled: bool | None = Field(
        True, description="Boolean defining section visibility."
    )
    label: str | None = Field(None, description="The label for the dynamic section.")
    normal: list[float] = Field(
        ...,
        description="Normal vector to the plane in voxel space. Defining the orientation of the dynamic section.",
        max_length=3,
        min_length=3,
    )
    point: list[float] = Field(
        ...,
        description="Point belonging to the section plane in voxel space. Defining the position of the dynamic section.",
        max_length=3,
        min_length=3,
    )


class CodedValue1(BaseModel):
    """
    A set of valid coded values with unique names.
    """

    model_config = common_config
    code: float | str = Field(
        ..., description="The value stored in the feature attribute."
    )
    name: str = Field(..., description="User-friendly name for what the code means.")


class JoinType(Enum):
    """
    The type of join (left outer or left inner).
    """

    esri_left_inner_join = "esriLeftInnerJoin"
    esri_left_outer_join = "esriLeftOuterJoin"


class GeometryType(Enum):
    """
    The geometry type. When querying a table that does not have a geometry column, do not include geometryType.
    """

    esri_geometry_multipoint = "esriGeometryMultipoint"
    esri_geometry_point = "esriGeometryPoint"
    esri_geometry_polygon = "esriGeometryPolygon"
    esri_geometry_polyline = "esriGeometryPolyline"


class QueryTableDataSource(BaseModel):
    """
    Query table data source is a layer/table that is defined by a SQL query.
    """

    model_config = common_config
    geometry_type: GeometryType | None = Field(
        None,
        alias="geometryType",
        description="The geometry type. When querying a table that does not have a geometry column, do not include geometryType.",
    )
    oid_fields: str | None = Field(
        None,
        alias="oidFields",
        description="Comma separated list of identifier fields. There are only certain field types that can be used as a unique identifier. These field types include integer, string, GUID, and date. If a single integer field is specified, map server uses the values in that field directly to uniquely identify all features and rows returned from a queryTable. However, if a single string field or a group of fields is used as the unique identifier, map server maps those unique values to an integer.",
    )
    query: str | None = Field(None, description="The SQL query.")
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="The spatial reference of the geometry column. When querying a table that does not have a geometry column, do not include spatialReference.",
        title="spatialReference",
    )
    type: Literal["queryTable"] = Field(
        "queryTable",
        description="String value indicating the type for the dataSource.",
    )
    workspace_id: str | None = Field(
        None,
        alias="workspaceId",
        description="The unique string value used to identify the datasource's workspace.",
    )


class RasterDataSource(BaseModel):
    """
    Raster data source is a file-based raster that resides in a registered raster workspace.
    """

    model_config = common_config
    data_source_name: str | None = Field(
        None,
        alias="dataSourceName",
        description="The raster datasource's name.",
    )
    type: Literal["raster"] = Field(
        "raster",
        description="String value indicating the type for the dataSource.",
    )
    workspace_id: str | None = Field(
        None,
        alias="workspaceId",
        description="The unique string value used to identify the datasource's workspace.",
    )


class TableDataSource(BaseModel):
    """
    Table data source is a table, feature class, or raster that resides in a registered workspace (either a folder or geodatabase). In the case of a geodatabase, if versioned, use version to switch to an alternate geodatabase version. If version is empty or missing, the registered geodatabase version will be used.
    """

    model_config = common_config
    data_source_name: str | None = Field(
        None,
        alias="dataSourceName",
        description="The fully-qualified string value used to specify where the dataSource is derived.",
    )
    gdb_version: str | None = Field(
        None,
        alias="gdbVersion",
        description="If applicable, the value indicating the version of the geodatabase.",
    )
    type: Literal["table"] = Field(
        "table",
        description="String value indicating the type for the dataSource. The value for a Table DataSource is `table`.",
    )
    workspace_id: str | None = Field(
        None,
        alias="workspaceId",
        description="The unique string value used to identify the datasource's workspace.",
    )


class Border(BaseModel):
    """
    Optional border on the line that is used to improve the contrast of the line color against various background colors.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="A value between `100` (full transparency) and `0` (full opacity).",
    )


class TextBackground(BaseModel):
    """
    Text background definition.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] | None = Field(
        None,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="A value between `100` (full transparency) and `0` (full opacity).",
    )


class Decoration(Enum):
    """
    The text decoration.
    """

    line_through = "line-through"
    none = "none"
    underline = "underline"


class Style(Enum):
    """
    The text style.
    """

    italic = "italic"
    normal = "normal"
    oblique = "oblique"


class Weight(Enum):
    """
    The text weight.
    """

    bold = "bold"
    bolder = "bolder"
    lighter = "lighter"
    normal = "normal"


class Font(BaseModel):
    """
    Font used for text symbols.
    """

    model_config = common_config
    decoration: Decoration | None = Field(
        Decoration.none,
        validate_default=True,
        description="The text decoration.",
    )
    family: str | None = Field(None, description="The font family.")
    size: float | None = Field(
        None,
        description="The font size in points. Ignored when font is used on TextSymbol3DLayer.",
    )
    style: Style | None = Field(
        Style.normal, validate_default=True, description="The text style."
    )
    weight: Weight | None = Field(
        Weight.normal, validate_default=True, description="The text weight."
    )


class Halo(BaseModel):
    """
    Halo definition.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] | None = Field(
        None,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    size: float | None = Field(None, description="Width of the halo in points.")
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="A value between `100` (full transparency) and `0` (full opacity).",
    )


class Material(BaseModel):
    """
    The material used to shade the geometry.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] | None = Field(
        None,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="A value between `100` (full transparency) and `0` (full opacity). Ignored if no color is specified.",
    )


class Style1(Enum):
    """
    (This property is used for comparison rendering). It is used to map the ratio between two numbers. It is possible to express that relationship as percentages, simple ratios, or an overall percentage.
    """

    percent = "percent"
    percent_total = "percentTotal"
    ratio = "ratio"


class Theme(Enum):
    """
    Theme to be used only when working with visual variables of type `colorInfo`. Default is `high-to-low`.
    """

    above_and_below = "above-and-below"
    centered_on = "centered-on"
    extremes = "extremes"
    high_to_low = "high-to-low"


class Type8(Enum):
    """
    A string value specifying the type of renderer's visual variable.
    """

    color_info = "colorInfo"
    rotation_info = "rotationInfo"
    size_info = "sizeInfo"
    transparency_info = "transparencyInfo"


class Units(Enum):
    """
    (This property is used only with age renderers.) Units for startTime and endTime.
    """

    days = "days"
    hours = "hours"
    minutes = "minutes"
    months = "months"
    seconds = "seconds"
    years = "years"


class AuthoringInfoVisualVariable(BaseModel):
    """
    This visual variable pertains specifically to [authoringInfo](authoringInfo.md) and is different from visual variables directly on the [renderer](renderer.md).
    """

    model_config = common_config
    end_time: float | str | None = Field(
        None,
        alias="endTime",
        description="A Unix stamp. Both `startTime` or `endTime` can be fields. If this is the case, their names must be different.",
    )
    field: str | None = Field(
        None,
        description="The attribute field the user chose in the Smart Mapping gallery. Must be the same as in either `startTime` or `endTime`.",
    )
    max_slider_value: float | None = Field(
        None,
        alias="maxSliderValue",
        description="A numeric value indicating the maximum value displayed.",
    )
    min_slider_value: float | None = Field(
        None,
        alias="minSliderValue",
        description="A numeric value indicating the minimum value displayed.",
    )
    start_time: float | str | None = Field(
        None,
        alias="startTime",
        description="A Unix time stamp. Both `startTime` or `endTime` can be fields. If this is the case, their names must be different.",
    )
    style: Style1 | None = Field(
        None,
        description="(This property is used for comparison rendering). It is used to map the ratio between two numbers. It is possible to express that relationship as percentages, simple ratios, or an overall percentage.",
    )
    theme: Theme | None = Field(
        None,
        description="Theme to be used only when working with visual variables of type `colorInfo`. Default is `high-to-low`.",
    )
    type: Type8 | None = Field(
        None,
        description="A string value specifying the type of renderer's visual variable.",
    )
    units: Units | None = Field(
        None,
        description="(This property is used only with age renderers.) Units for startTime and endTime.",
    )


class StyleOrigin(BaseModel):
    """
    The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.
    """

    model_config = common_config
    name: str = Field(..., description="Identifies a symbol in the style by name.")
    style_name: str = Field(
        ...,
        alias="styleName",
        description="A registered web style name, such as `EsriThematicShapesStyle`",
    )
    style_url: str | constr(pattern=r"^\./.+$") | None = Field(
        None, alias="styleUrl", description="URL to a style definition."
    )


class StyleOrigin1(BaseModel):
    """
    The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.
    """

    model_config = common_config
    name: str = Field(..., description="Identifies a symbol in the style by name.")
    style_name: str | None = Field(
        None,
        alias="styleName",
        description="A registered web style name, such as `EsriThematicShapesStyle`",
    )
    style_url: str | constr(pattern=r"^\./.+$") = Field(
        ..., alias="styleUrl", description="URL to a style definition."
    )


class Anchor(Enum):
    bottom = "bottom"
    bottom_left = "bottomLeft"
    bottom_right = "bottomRight"
    center = "center"
    left = "left"
    relative = "relative"
    right = "right"
    top = "top"
    top_left = "topLeft"
    top_right = "topRight"


class Join(Enum):
    """
    Shape of the intersection of two line segments.
    """

    bevel = "bevel"
    miter = "miter"
    round = "round"


class Anchor1(Enum):
    """
    The positioning of the object relative to the geometry.
    """

    bottom = "bottom"
    center = "center"
    origin = "origin"
    relative = "relative"
    top = "top"


class WaterbodySize(Enum):
    """
    Size of the waterbody the symbol layer represents. Applications will display waves that are appropriate for the chosen body of water, for example ocean versus marina versus swimming pool.
    """

    large = "large"
    medium = "medium"
    small = "small"


class WaveStrength(Enum):
    """
    The magnitude of the waves displayed on the waterbody. Strings roughly follow the [Douglas sea scale](https://en.wikipedia.org/wiki/Douglas_sea_scale), currently limited to lower degrees.
    """

    calm = "calm"
    moderate = "moderate"
    rippled = "rippled"
    slight = "slight"


class WaterSymbol3DLayer(BaseModel):
    """
    Symbol Layer that describes a water appearance on surfaces in a SceneView.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] | None = Field(
        [0, 119, 190], description="The dominant water color.", title="color"
    )
    enable: bool | None = None
    type: Literal["Water"] = Field(
        "Water", description="Specifies the type of symbol used."
    )
    waterbody_size: WaterbodySize | None = Field(
        WaterbodySize.medium,
        validate_default=True,
        alias="waterbodySize",
        description="Size of the waterbody the symbol layer represents. Applications will display waves that are appropriate for the chosen body of water, for example ocean versus marina versus swimming pool.",
    )
    wave_direction: confloat(ge=0.0, le=360.0) | None = Field(
        None,
        alias="waveDirection",
        description="Azimuthal bearing for direction of the waves. If ommitted, waves appear directionless. The value is interpreted as 'geographic' rotation, i.e. clockwise starting from north.",
    )
    wave_strength: WaveStrength | None = Field(
        WaveStrength.moderate,
        validate_default=True,
        alias="waveStrength",
        description="The magnitude of the waves displayed on the waterbody. Strings roughly follow the [Douglas sea scale](https://en.wikipedia.org/wiki/Douglas_sea_scale), currently limited to lower degrees.",
    )


class CIMSymbolReference(BaseModel):
    """
    Represents a symbol reference that contains a CIM symbol. In addition to `type` listed below, a symbol reference will contain additional properties. More details on CIM symbol reference and CIM symbols are available on github: [Symbols Overview](https://github.com/Esri/cim-spec/blob/main/docs/v2/Overview-Symbols.md).
    """

    model_config = common_config
    type: Literal["CIMSymbolReference"] = Field(
        "CIMSymbolReference",
        description="Specifies the type of symbol used.",
    )


class StyleSymbolReference(BaseModel):
    """
    The StyleSymbolReference is used to reference a symbol from a portal styleItem
    """

    model_config = common_config
    name: str = Field(..., description="Identifies a symbol in the style by name.")
    style_name: str | None = Field(
        None,
        alias="styleName",
        description="A registered web style name, such as `EsriThematicTreesStyle`",
    )
    style_url: str = Field(
        ..., alias="styleUrl", description="URL to a style definition."
    )
    type: Literal["styleSymbolReference"] = Field(
        "styleSymbolReference", description="The type of the symbol"
    )


class StyleSymbolReference1(BaseModel):
    """
    The StyleSymbolReference is used to reference a symbol from a portal styleItem
    """

    model_config = common_config
    name: str = Field(..., description="Identifies a symbol in the style by name.")
    style_name: str = Field(
        ...,
        alias="styleName",
        description="A registered web style name, such as `EsriThematicTreesStyle`",
    )
    style_url: str | None = Field(
        None, alias="styleUrl", description="URL to a style definition."
    )
    type: Literal["styleSymbolReference"] = Field(
        "styleSymbolReference", description="The type of the symbol"
    )


class Axis(Enum):
    """
    Defines the rotation axis the visual variable should be applied to when rendering features with an ObjectSymbol3DLayer. [Detailed description](static/objectSymbolLayerOrientation.md).
    """

    heading = "heading"
    roll = "roll"
    tilt = "tilt"


class RotationType(Enum):
    """
    Defines the origin and direction of rotation depending on how the angle of rotation was measured. Possible values are `geographic` which rotates the symbol from the north in a clockwise direction and `arithmetic` which rotates the symbol from the east in a counter-clockwise direction.
    """

    arithmetic = "arithmetic"
    geographic = "geographic"


class Axis1(Enum):
    """
    Defines the axis the size visual variable should be applied to when rendering features with an ObjectSymbol3DLayer.
    """

    all = "all"
    depth = "depth"
    height = "height"
    width = "width"
    width_and_depth = "widthAndDepth"


class ValueRepresentation(Enum):
    """
    Specifies how to apply the data value when mapping real-world sizes. See table below for supported values.
    """

    area = "area"
    diameter = "diameter"
    distance = "distance"
    radius = "radius"
    width = "width"


class ValueUnit(Enum):
    """
    A string value indicating the unit of measurement. Defaults to `meters` if not set.
    """

    centimeters = "centimeters"
    decimal_degrees = "decimal-degrees"
    decimeters = "decimeters"
    feet = "feet"
    inches = "inches"
    kilometers = "kilometers"
    meters = "meters"
    miles = "miles"
    millimeters = "millimeters"
    nautical_miles = "nautical-miles"
    unknown = "unknown"
    yards = "yards"


class ColorClassBreakInfo(BaseModel):
    """
    The classBreaksInfo object provides information about the class breaks associated with the renderer.
    """

    model_config = common_config
    class_max_value: float = Field(
        ...,
        alias="classMaxValue",
        description="A numeric value used to specify the maximum value for a break.",
    )
    class_min_value: float = Field(
        ...,
        alias="classMinValue",
        description="A numeric value used to specify discontinuous class breaks. If this value is null or is missing, the map server will calculate the minimum value based on the preceding class' maximum value.",
    )
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    description: str | None = Field(
        None, description="String value used to describe the class."
    )
    label: str | None = Field(None, description="String value used to label the class.")


class ColorModulationInfo(BaseModel):
    """
    Indicates whether modulation should be used to render the point.
    """

    model_config = common_config
    field: str = Field(
        ...,
        description="the attribute to use as a source for the modulation amplitude",
    )
    max_value: float = Field(
        ...,
        alias="maxValue",
        description="maximum value to compute modulation linear mapping",
    )
    min_value: float = Field(
        ...,
        alias="minValue",
        description="minimum value to compute modulation linear mapping",
    )


class ColorStop(BaseModel):
    """
    A colorStop object describes the renderer's color ramp with more specificity than just colors.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="A CSS color string or an array of rbga values. The color to place at the stop indicated by either a ratio or value.",
        title="color",
    )
    label: str | None = Field(
        None,
        description="Value if a label is needed on the legend for a stop.",
    )
    value: float | None = Field(
        None,
        description="The pixel intensity value. Describes the pixel intensity value that the color should be associated with. Just like in colorInfo, using value will ignore `maxPixelIntensity` and `minPixelIntensity` properties. It will actually set those properties to maximum and minimum values you set in the colorStops array. The hard values are converted to ratios to create the color gradient that is used in the heatmap calculations. Setting `minPixelIntensity` or `maxPixelIntensity`, after setting colorStops with values, removes the hard link between the color ramp and the pixel intensity values that were used to create it.",
    )


class ColorUniqueValueInfo(BaseModel):
    """
    The colorUniqueValueInfo object matches a unique value with a specific color.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    description: str | None = Field(
        None, description="String value used to describe the class."
    )
    label: str | None = Field(None, description="String value used to label the class.")
    values: list[str] = Field(
        ..., description="List of string values used to classify points."
    )


class ColormapInfo(BaseModel):
    """
    The colormap information used when rendering raster data.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ..., description="The pixel color.", title="color"
    )
    label: str | None = Field(None, description="The label used for the pixel value.")
    value: float = Field(..., description="The raster pixel value.")


class VoxelAlphaStop(BaseModel):
    """
    Defines transparency stop for a transfer function style.
    """

    model_config = common_config
    alpha: confloat(ge=0.0, le=1.0) = Field(
        ..., description="Opacity of the stop in [0,1]. 1 is fully opaque."
    )
    position: confloat(ge=0.0, le=1.0) = Field(
        ..., description="Normalized position of the stop in [0,1]."
    )


class VoxelColorStop(BaseModel):
    """
    Defines color stop for a transfer function style.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    position: confloat(ge=0.0, le=1.0) = Field(
        ..., description="Normalized position of the stop in [0,1]."
    )


class VoxelRangeFilter(BaseModel):
    """
    Defines the range to filter values from. Voxel Values outside this range will be discarded.
    """

    model_config = common_config
    enabled: bool | None = Field(
        False, description="Determines if the range filter is enabled."
    )
    range: list[float] = Field(
        ...,
        description="Defines the minimum and maximum values of the range. Data outside of the range will be discarded.",
        max_length=2,
        min_length=2,
    )


class SketchEdges(BaseModel):
    """
    The sketch edge rendering configuration of a symbol layer. Edges of type `sketch` are rendered with a hand-drawn look in mind.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    extension_length: float | None = Field(
        None,
        alias="extensionLength",
        description="A size in points by which to extend edges beyond their original end points.",
    )
    size: confloat(ge=0.0) | None = Field(
        None, description="Edge size in points, positive only"
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="The value has to lie between `100` (full transparency) and `0` (full opacity).",
    )
    type: Literal["sketch"] = Field(
        "sketch", description="The type of edge visualization."
    )


class SolidEdges(BaseModel):
    """
    The solid edge rendering configuration of a symbol layer. Edges of type `solid` are rendered in a single color, unaffected by scene lighting.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    extension_length: float | None = Field(
        None,
        alias="extensionLength",
        description="A size in points by which to extend edges beyond their original end points.",
    )
    size: confloat(ge=0.0) | None = Field(
        None, description="Edge size in points, positive only"
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="The value has to lie between `100` (full transparency) and `0` (full opacity).",
    )
    type: Literal["solid"] = Field(
        "solid", description="The type of edge visualization."
    )


class Algorithm(Enum):
    """
    Algorithm used for calculating the ramp.
    """

    esri_cie_lab_algorithm = "esriCIELabAlgorithm"
    esri_hsv_algorithm = "esriHSVAlgorithm"
    esri_lab_l_ch_algorithm = "esriLabLChAlgorithm"


class ColorRampAlgorithmic(BaseModel):
    """
    A colorRamp object is used to specify a range of colors that are applied to a group of symbols.
    """

    model_config = common_config
    algorithm: Algorithm | None = Field(
        None, description="Algorithm used for calculating the ramp."
    )
    from_color: list[conint(ge=0, le=255)] | None = Field(
        None,
        alias="fromColor",
        description="Array representing the initial color to start the ramp from.",
        title="color",
    )
    to_color: list[conint(ge=0, le=255)] | None = Field(
        None,
        alias="toColor",
        description="Array representing the final color to end the ramp with.",
        title="color",
    )
    type: Literal["algorithmic"] | None = Field(
        None, description="Value indicating the type of colorRamp."
    )


class ColorRampMultipart(BaseModel):
    """
    A colorRamp object is used to specify a range of colors that are applied to a group of symbols.
    """

    model_config = common_config
    color_ramps: list[ColorRampAlgorithmic] | None = Field(
        None,
        alias="colorRamps",
        description="A multipart color ramp is defined by a list of constituent color ramps.",
    )
    type: Literal["multipart"] | None = Field(
        None, description="Value indicating the type of colorRamp."
    )


class AuthoringInfoClassBreakInfo(BaseModel):
    """
    The classBreaksInfo object provides information about the class breaks associated with the Relationship renderer.
    """

    model_config = common_config
    max_value: float | None = Field(
        None,
        alias="maxValue",
        description="A numeric value used to specify the maximum value for a break.",
    )
    min_value: float | None = Field(
        None,
        alias="minValue",
        description="A numeric value used to specify the minimum value for a break.",
    )


class ColorMixMode(Enum):
    """
    Controls how symbolLayer or visualVariable color is applied onto the underlying geometry color/texture. This property only applies to FillSymbol3DLayer within MeshSymbol3D.<br><ul><li>`tint`: the feature's appearance (in terms of color) should be altered to match the symbol / visual variable color.</li><li>`replace`: the feature's color is replaced with the symbol / visual variable color.</li><li>`multiply`: the feature's color is multiplied with the symbol color.</li></ul>
    """

    multiply = "multiply"
    replace = "replace"
    tint = "tint"


class Material1(BaseModel):
    """
    The material used to shade the geometry, including colorMixMode options.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] | None = Field(
        None,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    color_mix_mode: ColorMixMode | None = Field(
        None,
        alias="colorMixMode",
        description="Controls how symbolLayer or visualVariable color is applied onto the underlying geometry color/texture. This property only applies to FillSymbol3DLayer within MeshSymbol3D.<br><ul><li>`tint`: the feature's appearance (in terms of color) should be altered to match the symbol / visual variable color.</li><li>`replace`: the feature's color is replaced with the symbol / visual variable color.</li><li>`multiply`: the feature's color is multiplied with the symbol color.</li></ul>",
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="A value between `100` (full transparency) and `0` (full opacity). Ignored if no color is specified.",
    )


class Style2(Enum):
    """
    String value representing predefined styles that can be set as polygon fills.
    """

    backward_diagonal = "backward-diagonal"
    cross = "cross"
    diagonal_cross = "diagonal-cross"
    forward_diagonal = "forward-diagonal"
    horizontal = "horizontal"
    none = "none"
    solid = "solid"
    vertical = "vertical"


class Pattern(BaseModel):
    """
    The pattern used to render the fill of the polygon (only applies to PolygonSymbol3D).
    """

    model_config = common_config
    style: Style2 = Field(
        ...,
        description="String value representing predefined styles that can be set as polygon fills.",
    )
    type: Literal["style"] = Field(
        "style",
        description="The type of pattern applied to the polygon fill.",
    )


class Primitive(Enum):
    """
    Specifies the type of symbol used.
    """

    circle = "circle"
    cross = "cross"
    kite = "kite"
    square = "square"
    triangle = "triangle"
    x = "x"


class IconSymbol3DLayerResource(BaseModel):
    """
    The shape (primitive) or image URL (href) used to visualize the features.
    """

    model_config = common_config
    data_uri: constr(pattern=r"^data:image/(.|\n|\r)+$") | None = Field(
        None,
        alias="dataURI",
        description="an image encoded as base64 string, starting with `data:image/`",
    )
    href: constr(pattern=r"^https?://.+$") | constr(pattern=r"^\./.+$") | None = Field(
        None, description="URL to the returned image."
    )
    primitive: Primitive = Field(..., description="Specifies the type of symbol used.")


class IconSymbol3DLayerResource1(BaseModel):
    """
    The shape (primitive) or image URL (href) used to visualize the features.
    """

    model_config = common_config
    data_uri: constr(pattern=r"^data:image/(.|\n|\r)+$") | None = Field(
        None,
        alias="dataURI",
        description="an image encoded as base64 string, starting with `data:image/`",
    )
    href: constr(pattern=r"^https?://.+$") | constr(pattern=r"^\./.+$") = Field(
        ..., description="URL to the returned image."
    )
    primitive: Primitive | None = Field(
        None, description="Specifies the type of symbol used."
    )


class IconSymbol3DLayerResource2(BaseModel):
    """
    The shape (primitive) or image URL (href) used to visualize the features.
    """

    model_config = common_config
    data_uri: constr(pattern=r"^data:image/(.|\n|\r)+$") = Field(
        ...,
        alias="dataURI",
        description="an image encoded as base64 string, starting with `data:image/`",
    )
    href: constr(pattern=r"^https?://.+$") | constr(pattern=r"^\./.+$") | None = Field(
        None, description="URL to the returned image."
    )
    primitive: Primitive | None = Field(
        None, description="Specifies the type of symbol used."
    )


class LineCap(Enum):
    """
    Shape of the tips at the start and end of each line geometry. This also applies to the tips of each pattern segment along the line.
    """

    butt = "butt"
    round = "round"
    square = "square"


class Placement(Enum):
    """
    Indicates where the marker is placed.
    """

    begin = "begin"
    begin_end = "begin-end"
    end = "end"


class Style3(Enum):
    """
    Style of the marker.
    """

    arrow = "arrow"
    circle = "circle"
    cross = "cross"
    diamond = "diamond"
    square = "square"
    x = "x"


class LineMarker(BaseModel):
    """
    Represents markers placed at the start and end of each line geometry, or both. Markers size is proportional to the width of the line.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] | None = Field(
        None,
        description="An option to color the markers differently from the line. By default the markers inherit the line's color.",
        title="color",
    )
    placement: Placement | None = Field(
        None, description="Indicates where the marker is placed."
    )
    style: Style3 | None = Field(None, description="Style of the marker.")
    type: Literal["style"] = Field(
        "style", description="The type of marker applied to a line."
    )


class Style4(Enum):
    """
    String value representing the pattern used to render a line.
    """

    dash = "dash"
    dash_dot = "dash-dot"
    dash_dot_dot = "dash-dot-dot"
    dot = "dot"
    long_dash = "long-dash"
    long_dash_dot = "long-dash-dot"
    null = "null"
    short_dash = "short-dash"
    short_dash_dot = "short-dash-dot"
    short_dash_dot_dot = "short-dash-dot-dot"
    short_dot = "short-dot"
    solid = "solid"


class LinePattern(BaseModel):
    """
    A pattern used to render a line.
    """

    model_config = common_config
    style: Style4 = Field(
        ...,
        description="String value representing the pattern used to render a line.",
    )
    type: Literal["style"] = Field(
        "style", description="The type of pattern applied to a line."
    )


class Primitive3(Enum):
    cone = "cone"
    cube = "cube"
    cylinder = "cylinder"
    diamond = "diamond"
    inverted_cone = "invertedCone"
    sphere = "sphere"
    tetrahedron = "tetrahedron"


class ObjectSymbol3DLayerResource(BaseModel):
    """
    The primitive shape (primitive) or external 3D model (href) used to visualize the points.
    """

    model_config = common_config
    href: constr(pattern=r"^https?://.+$") | constr(pattern=r"^\./.+$") | None = None
    primitive: Primitive3


class ObjectSymbol3DLayerResource1(BaseModel):
    """
    The primitive shape (primitive) or external 3D model (href) used to visualize the points.
    """

    model_config = common_config
    href: constr(pattern=r"^https?://.+$") | constr(pattern=r"^\./.+$")
    primitive: Primitive3 | None = None


class Anchor2(Enum):
    """
    The position of the extrusion profile with respect to the polyline geometry.
    """

    bottom = "bottom"
    center = "center"
    top = "top"


class Profile(Enum):
    """
    The shape which is extruded along the line.
    """

    circle = "circle"
    quad = "quad"


class ProfileRotation(Enum):
    """
    Specifies the axes about which the profile may be rotated at the joins. Constraining the rotation axes leads to a fixed orientation of the profile for the specified directions.
    """

    all = "all"
    heading = "heading"


class VisualVariableLegendOptions(BaseModel):
    """
    Options available for the legend for visual variables.
    """

    model_config = common_config

    custom_values: list[float] | None = Field(None, alias="customValues")
    show_legend: bool | None = Field(
        None,
        alias="showLegend",
        description="Indicates whether to show the color/size/opacity ramp in the legend.",
    )
    title: str | None = Field(None, description="The title of the legend.")


class SizeStop(BaseModel):
    """
    A `sizeStop` object describes the size of the symbol at various values of the expression.
    """

    model_config = common_config
    label: str | None = Field(
        None,
        description="Value if a label is needed on the legend for a stop.",
    )
    size: float = Field(
        ...,
        description="Specifies the marker size to use for the specified value.",
    )
    value: float = Field(..., description="The value to be mapped to a size.")


class TransparencyStop(BaseModel):
    """
    The transparencyStop object defines the thematic opacity ramp in a sequence of stops. At least two stops are required. The stops must be listed in ascending order based on the value of the `value` property in each stop.
    """

    model_config = common_config
    label: str | None = Field(
        None,
        description="A string value used to label the stop in the legend.",
    )
    transparency: int | float = Field(
        ...,
        description="A numeric transparancy value for a stop ranging from 0-100, where 0 is opaque and 100 is 100% transparent.",
    )
    value: float = Field(
        ...,
        description="The pixel intensity value. Describes the pixel intensity value that the color should be associated with.",
    )


class PointCloudFixedSizeAlgorithm(BaseModel):
    """
    Render points with fixed real world or screen space size.
    """

    model_config = common_config
    size: float = Field(
        ..., description="Symbol size in real world units or display unit"
    )
    type: Literal["pointCloudFixedSizeAlgorithm"] = "pointCloudFixedSizeAlgorithm"
    use_real_world_symbol_sizes: bool | None = Field(
        None,
        alias="useRealWorldSymbolSizes",
        description=" If true symbol size is in meters, display unit (pt) otherwise",
    )


class PointCloudSplatAlgorithm(BaseModel):
    """
    Render points using sizes depending on point density.
    """

    model_config = common_config
    scale_factor: float = Field(
        ...,
        alias="scaleFactor",
        description="Scale factor to applied to the computed point size (in real world)",
    )
    type: Literal["pointCloudSplatAlgorithm"] = "pointCloudSplatAlgorithm"


class PathCap(Enum):
    """
    Shape of the tips at the start and end of each path geometry.
    """

    butt = "butt"
    none = "none"
    round = "round"
    square = "square"


class ClippingArea(BaseModel):
    """
    Defines area to be clipped for display.
    """

    model_config = common_config
    clip: bool | None = Field(None, description="enable / disable clipping")
    geometry: Extent | dict | None = Field(None, description="envelope of clip area")


class MapFloorInfo(BaseModel):
    """
    Contains floor-awareness information for the web scene. Defines the layers and required fields for each layer that are used for floor filtering.
    """

    model_config = common_config
    facility_layer: FacilityLayer = Field(
        ...,
        alias="facilityLayer",
        description="Defines the layer and field properties for the Facility layer used for floor filtering.",
    )
    level_layer: LevelLayer = Field(
        ...,
        alias="levelLayer",
        description="Defines the layer and field properties for the Level layer used for floor filtering.",
    )
    site_layer: SiteLayer | None = Field(
        None,
        alias="siteLayer",
        description="Defines the layer and field properties for the Site layer used for floor filtering. This property is optional.",
    )


class Widgets(BaseModel):
    """
    The widgets object contains widgets that should be exposed to the user.
    """

    model_config = common_config
    floor_filter: FloorFilter | None = Field(
        None,
        alias="floorFilter",
        description="Configurable properties used by the floor filter widget.",
    )
    range: Range | Range1 | None = Field(
        None, description="Active range.", title="Range"
    )
    time_slider: TimeSlider | None = Field(
        None,
        alias="timeSlider",
        description="Configurable properties used by the time slider widget.",
    )


class VectorTileLayerVectorTileLayer(BaseModel):
    """
    A vector tile layer references a set of web-accessible vector tiles and the corresponding style for how those tiles should be drawn.
    """

    model_config = common_config
    blend_mode: BlendMode | None = Field(
        BlendMode.normal, validate_default=True, alias="blendMode"
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="This property is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers, or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["VectorTileLayer"] = Field(
        "VectorTileLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the layer in the layer list",
    )
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.vtpk$") | None = Field(
        None,
        description="For offline data, a path to a vector tile layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.vtpk`. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.vtpk`.",
    )
    style_url: str = Field(
        ...,
        alias="styleUrl",
        description="A url to a JSON file containing the stylesheet information used to render the layer. You may also pass an object containing the stylesheet information identical to the JSON file.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="An array of two numbers representing the start and end time for which the layer is visible. The time is in UNIX epoch time in milliseconds.",
    )


class WMSLayerWMS(BaseModel):
    """
    A layer consuming a Web Map Service (WMS). The WMS specification is an international specification for serving and consuming dynamic maps on the web.
    """

    model_config = common_config
    copyright: str | None = Field(
        None,
        description="A string containing copyright and access information for a WMS layer. This is copied from the capabilities document exposed by the WMS service.",
    )
    custom_layer_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customLayerParameters",
        description="A sequence of custom parameters to WMS layer requests. These parameters are applied to `GetMap` and `GetFeatureInfo` requests. The `customLayerParameters` property takes precedence if `customParameters` is also present.",
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters to all WMS requests. These parameters are applied to `GetCapabilities`, `GetMap`, and `GetFeatureinfo` requests. If used with the `customLayerParameters` property, `customParameters` will not take precedence.",
        title="customParameters",
    )
    extent: list[list[Any]] | None = Field(
        None,
        description="The minimum bounding rectangle, in decimal degrees, of the area covered by the layer as specified in the capabilities.",
        max_length=2,
        min_length=2,
    )
    feature_info_format: str | None = Field(
        None,
        alias="featureInfoFormat",
        description="Format of the feature, e.g.`text/plain`",
    )
    feature_info_url: str | None = Field(
        None,
        alias="featureInfoUrl",
        description="The URL for the WMS GetFeatureInfo call.",
    )
    format: Format1 | None = Field(
        Format1.png,
        validate_default=True,
        description="A string containing the image format to be requested from a WMS service. Default is `png`.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="This is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers, or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Unique string value indicating an item registered in ArcGIS Online or your organization's portal.",
    )
    layers: list[WMSLayerLayer] | None = Field(
        None, description="An array of layer objects from the WMS service."
    )
    layer_type: Literal["WMS"] = Field(
        "WMS",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",
    )
    map_url: str | None = Field(
        None,
        alias="mapUrl",
        description="A string containing the URL of the WMS map. When using a WMS layer, you should also supply the url property. `mapUrl` is the URL returned by the capabilities to be used for the getMap requests.",
    )
    max_height: float | None = Field(
        None,
        alias="maxHeight",
        description="A number defining the maximum height, in pixels, that should be requested from the service.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    max_width: float | None = Field(
        None,
        alias="maxWidth",
        description="A number defining the maximum width, in pixels, that should be requested from the service.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend.",
    )
    spatial_references: list[int] | None = Field(
        None,
        alias="spatialReferences",
        description="An array of numbers containing well-known IDs for spatial references supported by the service.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Boolean value indicating whether the layer supports time animation.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str | None = Field(
        None,
        description="The URL to the WMS service (`getCapabilities` URL).",
    )
    version: str | None = Field(
        None,
        description="A string containing the version number of the service.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web map.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="An array of two numbers representing the start and end time for the visibility of the layer.",
    )
    visible_layers: list[str] | None = Field(
        None,
        alias="visibleLayers",
        description="An array of layers that should appear visible. The array contains the names of the visible layers.",
    )


class OpenStreetMapLayerOpenStreetMap(BaseModel):
    """
    Allows use of OpenStreetMap data for use in basemaps only.
    """

    model_config = common_config
    blend_mode: BlendMode | None = Field(
        BlendMode.normal, validate_default=True, alias="blendMode"
    )
    id: str | None = Field(
        None,
        description="An identifying string for the layer, unique per scene.",
    )
    layer_type: Literal["OpenStreetMap"] = Field(
        "OpenStreetMap",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A numeric property used to determine the maximum scale at which the layer is displayed.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A numeric property used to determine the minimum scale at which the layer is displayed.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    title: str = Field(..., description="String title for the basemap layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="An array of two numbers representing the start and end time for the visibility of the layer.",
    )


class Camera(BaseModel):
    """
    The camera defines the position, tilt, and heading of the point from which the SceneView's visible extent is observed.
    """

    model_config = common_config
    heading: float = Field(
        ...,
        description="The heading of the camera in degrees. Heading is zero when north is the top of the screen. It increases as the view rotates clockwise.",
    )
    position: PointGeometry = Field(
        ..., description="The position of the camera defined by a map point."
    )
    tilt: float = Field(
        ...,
        description="The tilt of the camera in degrees with respect to the surface as projected down from the camera position. Tilt is zero when looking straight down at the surface and 90 degrees when the camera is looking parallel to the surface.",
    )


class LengthDimension(BaseModel):
    """
    Defines the shape of a dimension that measures the distance between two points.
    """

    model_config = common_config
    end_point: PointGeometry = Field(
        ...,
        alias="endPoint",
        description="The position of the point that a dimension is measured to (last input point).",
    )
    measure_type: MeasureType = Field(
        ...,
        alias="measureType",
        description="Defines whether the horizontal, vertical or direct distance between the start and end points is measured.",
    )
    offset: float = Field(
        ...,
        description="The distance of the dimension line from the nearest input point in meters.",
    )
    orientation: float = Field(
        ...,
        description="The direction that the offset of a dimension with a 'direct' measureType extends in.",
    )
    start_point: PointGeometry = Field(
        ...,
        alias="startPoint",
        description="The position of the point that a dimension is measured from (first input point).",
    )
    type: Literal["length"] = Field(
        "length", description="Specifies the type of the dimension."
    )


class MultidimensionalSubset(BaseModel):
    """
    Represents a multidimensional subset of raster data. This includes subsets of both variables and dimensions. When the multidimensionalSubset is defined on a layer, the [layer.multidimensionalDefinition](multidimensionalDefinition.md) or the [mosaicRule.multidimensionalDefinition](mosaicRule.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.
    """

    model_config = common_config
    area_of_interest: Extent | dict | PolygonGeometry | None = Field(
        None,
        alias="areaOfInterest",
        description="An optional area of interest for the entire multidimensional subset.",
    )
    subset_definitions: list[DimensionalDefinition | DimensionalDefinition1] | None = (
        Field(
            None,
            alias="subsetDefinitions",
            description="An optional array of objects representing dimension range subsets for selected variables from multidimensional data.",
        )
    )


class Modification(BaseModel):
    """
    A Modification describes a client-side transformation of the geometry of an [integrated mesh](integratedMeshLayer.md). It defines the shape and type of modification.
    """

    model_config = common_config
    geometry: PolygonGeometry = Field(
        ...,
        description="Polygon geometry shape for the modification. `spatialReference` on geometry is ignored, it is instead taken from the data layer the modification belongs to. Modification polygons are not affected by elevation offsets.",
    )
    type: Type2 = Field(
        ...,
        description="Type of the modification. `clip`: Anything inside of the polygon will be clipped out. `mask`: All areas outside of the polygon will be masked out. `replace`: Flattens or elevates the integrated mesh layer based on the polygon.",
    )


class Georeference(BaseModel):
    """
    The georeference used to place a media in a [MediaLayer](mediaLayer.md).
    """

    model_config = common_config
    coefficients: list[float] = Field(
        ...,
        description="An array of 8 coefficients representing the [projective transformation](https://desktop.arcgis.com/en/arcmap/latest/manage-data/editing-existing-features/about-spatial-adjustment-transformations.htm#ESRI_SECTION1_EBB9C52B96934FE08A32CE852788EA02).",
        max_length=8,
        min_length=8,
    )
    control_points: list[ControlPoint] = Field(
        ...,
        alias="controlPoints",
        description="An array of 4 user defined control points placed on the media.",
        max_length=4,
        min_length=4,
    )
    height: float = Field(
        ...,
        description="Numeric value indicating the height of media in pixels.",
    )
    spatial_reference: SpatialReference = Field(
        ...,
        alias="spatialReference",
        description="The spatial reference can be defined using a well-known ID (wkid) or well-known text (WKT).",
        title="spatialReference",
    )
    width: float = Field(
        ...,
        description="Numeric value indicating the width of media in pixels.",
    )


class TileInfo(BaseModel):
    """
    Tile information, returned from the WMTS OGC Web Service. The tileInfo will contain the spatial reference of the layer. tileInfo is the same json representation as the ArcGIS Map/Image service tileInfo except that it may contain a levelValue on the lod objects that should be used instead of the level in the templateUrl.
    """

    model_config = common_config
    cols: int | None = Field(None, description="Requested tile's column.")
    compression_quality: confloat(ge=0.0, le=100.0) | None = Field(
        None,
        alias="compressionQuality",
        description="Compression quality of the tile.",
    )
    dpi: float | None = Field(None, description="The dpi of the tiling scheme.")
    lods: list[Lod] | None = Field(
        None,
        description="An array of levels of detail that define the tiling scheme.",
    )
    origin: PointGeometry | None = Field(None, description="The tiling scheme origin.")
    rows: int | None = Field(None, description="Requested tile's row.")
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="The spatial reference of the tiling schema.",
        title="spatialReference",
    )


class ElevationInfo(BaseModel):
    """
    Elevation info defines how features are aligned to ground or other layers.
    """

    model_config = common_config
    feature_expression: FeatureExpression | None = Field(
        None,
        alias="featureExpression",
        description='<em>Deprecated with 1.8, use `featureExpressionInfo` instead</em>. `{"value":0}` ignores geometry z-values.',
    )
    feature_expression_info: FeatureExpressionInfo | None = Field(
        None,
        alias="featureExpressionInfo",
        description="An object that defines an expression for per-feature elevation. If not set, geometry.z values are used for elevation. `unit` is applied to the resulting expression value.",
    )
    mode: Mode = Field(
        ...,
        description="Determines how the service elevation values are combined with the elevation of the scene.",
    )
    offset: float | None = Field(
        None,
        description="Offset is always added to the result of the above logic except for onTheGround where offset is ignored.",
    )
    unit: str | None = Field(
        "meter",
        description='A string value indicating the unit for the values in elevationInfo. Applies to both `offset` and `featureExpressionInfo`. Defaults to `meter` if not set. <a href="#unit"><sup>1</sup></a>',
    )


class FieldInfo(BaseModel):
    """
    Defines how a field in the dataset participates (or does not participate) in a popup window.
    """

    model_config = common_config
    field_name: str | None = Field(
        None,
        alias="fieldName",
        description="A string containing the field name as defined by the service. Anywhere that a fieldname is referenced as `{field-name} in popupInfo, an Arcade expression can also be referenced as `{expression/<expression-name>}`.",
    )
    format: Format2 | None = Field(
        None,
        description="A format object used with numerical or date fields to provide more detail about how the value should be displayed in a popup window.",
    )
    is_editable: bool | None = Field(
        True,
        alias="isEditable",
        description="A Boolean determining whether users can edit this field. Not applicable to Arcade expressions.",
    )
    label: str | None = Field(
        None,
        description="A string containing the field alias. This can be overridden by the web map author. Not applicable to Arcade expressions as `title` is used instead.",
    )
    statistic_type: StatisticType | None = Field(
        None,
        alias="statisticType",
        description="Used in a 1:many or many:many relationship to compute the statistics on the field to show in the popup.",
    )
    string_field_option: StringFieldOption | None = Field(
        StringFieldOption.textbox,
        validate_default=True,
        alias="stringFieldOption",
        description="A string determining what type of input box editors see when editing the field. Applies only to string fields. Not applicable to Arcade expressions.",
    )
    tooltip: str | None = Field(
        None,
        description="A string providing an editing hint for editors of the field. Not applicable to Arcade expressions.",
    )
    visible: bool | None = Field(
        None,
        description="A Boolean determining whether the field is visible in the popup window.",
    )


class RelatedRecordsInfo(BaseModel):
    """
    The sort in the popupInfo for the parent feature. This impacts the sorting order for the returned child records.
    """

    model_config = common_config

    order_by_fields: list[OrderByField] | None = Field(
        None,
        alias="orderByFields",
        description="Array of orderByField objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.",
    )
    show_related_records: bool | None = Field(
        None,
        alias="showRelatedRecords",
        description="Required boolean value indicating whether to display related records. If <code>true</code>, client should let the user navigate to the related records. Defaults to <code>true</code> if the layer participates in a relationship AND the related layer/table has already been added to the scene (either as an operationalLayer or as a table).",
    )


class SearchLayer(BaseModel):
    """
    Layer configuration for search.
    """

    model_config = common_config
    field: SearchField
    id: str | None = Field(None, description="A string identifying the layer.")
    sub_layer: int | None = Field(
        None, alias="subLayer", description="Optional index for a sublayer."
    )


class SearchTable(BaseModel):
    """
    Search configuration for table.
    """

    model_config = common_config
    field: SearchField
    id: str | None = Field(None, description="A string identifying the table.")


class RasterFunctionTemplate(BaseModel):
    """
    Raster function template containing one or more raster functions chained together to produce a processing workflow.
    """

    model_config = common_config
    field_object_id: int | None = Field(
        None,
        alias="_object_id",
        description="The id of the raster function template.",
    )
    aliases: list[str] | None = Field(
        None,
        description="Aliases for the `function` referenced in the template.",
    )
    arguments: RasterFunctionTemplateArguments | RasterFunctionTemplateArguments1 = (
        Field(
            ...,
            description="The arguments for the `function` referenced in the raster function template.",
            title="Raster Function Template Arguments",
        )
    )
    definition: str | None = Field(
        None,
        description="A query definition to filter rasters when the layer's data source is a mosaic dataset.",
    )
    description: str = Field(
        ..., description="The description of the raster function template."
    )
    function: RasterFunctionInformation = Field(
        ...,
        description="The raster function referenced by the raster function template.",
    )
    function_type: FunctionType = Field(
        ...,
        alias="functionType",
        description="Defines whether the `function` is applied to a mosaic dataset. Indicates the level of the mosaic processing is used. Only applies to mosaic based image services. `0` - function is applied after mosaicking; `1` - function is applied on each raster item before mosaicking; `2` - function is applied to a group of raster items before mosaicking.",
    )
    group: str | None = Field(
        None,
        description="Group field name for item group function template ",
    )
    help: str | None = Field(
        None,
        description="The short description of what the raster function template does.",
    )
    name: str = Field(..., description="Name of the raster function template.")
    properties: Properties | None = Field(
        None,
        description="The properties used to define multidimensional dataset processing rules.",
    )
    tag: str | None = Field(
        None, description="Tag field name for item group function template ."
    )
    thumbnail: str | None = Field(
        None, description="Thumbnail url of the raster function template."
    )
    thumbnail_ex: str | None = Field(
        None,
        alias="thumbnailEx",
        description="Base64 encoded thumbnail of the raster function template.",
    )
    type: Literal["RasterFunctionTemplate"] | None = Field(
        None, description="Type of the raster function template."
    )


class VoxelSection(BaseModel):
    """
    A section that is locked in for a specific variable and/or time. Allowing you to view the section together with other variables. A locked (or static) section is `float32` raster. This JSON object contains the meta-data needed to display (and potentially re-generate) this raster.

     When creating a web scene the locked sections must have this information as all properties come the web scene. If no section information is given, it is assumed there are no sections in the web scene.
    """

    model_config = common_config
    enabled: bool | None = Field(
        True, description="Boolean defining section visibility."
    )
    href: str = Field(
        ...,
        description="Relative href to the static section raster binary. href is relative to the layer document which contains the href. Inside an SLPK archive, must be of the form `resources/sections/<file>.bin.gz` and the section raster should be gzip-compressed.",
    )
    id: int = Field(
        ...,
        description="The id of the section. Must be unique from other sections in the array.",
    )
    label: str | None = Field(None, description="The label for the section.")
    normal: list[float] = Field(
        ...,
        description="Normal vector to the plane in voxel space.",
        max_length=3,
        min_length=3,
    )
    point: list[float] = Field(
        ...,
        description="Point belonging to the section plane in voxel space.",
        max_length=3,
        min_length=3,
    )
    size_in_pixel: list[int] = Field(
        ...,
        alias="sizeInPixel",
        description="Array of size 2 which describes the dimension of the raster data in pixels.",
        max_length=2,
        min_length=2,
    )
    slices: list[VoxelSlice] | None = Field(
        None,
        description="A copy of the slices that were applied to the volume when the section was created when new UVs are discovered for subsequent time slices.",
    )
    time_id: int | None = Field(
        None,
        alias="timeId",
        description="Time slice id at which the section was created. `timeId` is only applicable when time is the 4th dimension.",
    )
    variable_id: int = Field(..., alias="variableId", description="Id of the variable.")


class MediaInfoChart(BaseModel):
    """
    Defines a chart to be displayed in a popup window.
    """

    model_config = common_config
    alt_text: str | None = Field(
        None,
        alias="altText",
        description="A string providing the alternate text for the media.",
    )
    caption: str | None = Field(
        None, description="A string caption describing the media."
    )
    title: str | None = Field(None, description="A string title for the media.")
    type: Type5 = Field(..., description="A string defining the type of media.")
    value: MediaInfoChartValue


class MediaInfoImage(BaseModel):
    """
    Defines an image to be displayed in a popup window.
    """

    model_config = common_config
    alt_text: str | None = Field(
        None,
        alias="altText",
        description="A string providing the alternate text for the media.",
    )
    caption: str | None = Field(
        None, description="A string caption describing the media."
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled. If the property does not exist, it's equivalent to having a value of 0.",
    )
    title: str | None = Field(None, description="A string title for the media.")
    type: Literal["image"] = Field(
        "image", description="A string defining the type of media."
    )
    value: MediaInfoImageValue


class PopupElementExpression(BaseModel):
    """
    A pop-up element defined by an arcade expression.
    """

    model_config = common_config

    expression_info: PopupElementExpressionInfo | None = Field(
        None,
        alias="expressionInfo",
        description="This property applies to elements of type `expression` . An Arcade expression that defines the pop-up element content. The return type will always be `dictionary` as outlined [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element).",
    )
    type: Literal["expression"] | None = Field(
        None,
        description="String value indicating which elements to use. When an element is of type `expression` then `expressionInfo` is the only valid property.",
    )


class PopupElementFields(BaseModel):
    """
    Configures fields in popup elements.
    """

    model_config = common_config
    attributes: dict[str, Any] | None = Field(
        None,
        description="A dictionary of key value pairs representing attributes to be used instead of fields and their values. This property is only used when an element of type `fields` is being returned inside an element of type `expression` and should be returned as part of the arcade expression itself. This property allows passing arcade derived attribute values into `fields` elements. More details can be found [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element).",
    )
    description: str | None = Field(
        None,
        description="An optional string value describing the element in detail. This property does not apply when the `type` is `text` or `expression`.",
    )
    field_infos: list[FieldInfo] | None = Field(
        None,
        alias="fieldInfos",
        description="It is an array of `fieldInfo` objects representing a field/value pair displayed as a table within the popupElement. If the `fieldInfos` property is not provided, the popupElement will display whatever is specified directly in the `popupInfo.fieldInfos` property.",
    )
    title: str | None = Field(
        None,
        description="An optional string value indicating what the element represents. This property does not apply when the `type` is `text` or `expression`.",
    )
    type: Literal["fields"] = "fields"


class PopupElementRelationship(BaseModel):
    """
    Provides the ability to navigate and view related records from a layer or table associated within the pop-up.
    """

    model_config = common_config
    description: str | None = Field(
        None, description="A string that describes the element in detail."
    )
    display_count: int | None = Field(
        None,
        alias="displayCount",
        description="An integer that indicates the maximum number of records to display.",
    )
    display_type: Literal["list"] = Field(
        "list",
        alias="displayType",
        description="A string that defines how the related records should be displayed.",
    )
    order_by_fields: list[OrderByField] | None = Field(
        None,
        alias="orderByFields",
        description="Array of `orderByField` objects indicating the display order for the related records, and whether they should be sorted in ascending `'asc'` or descending `'desc'` order. If `orderByFields` is not provided, the popupElement will display whatever is specified directly in the `popupInfo.relatedRecordsInfo.orderByFields` property.",
    )
    relationship_id: int = Field(
        ...,
        alias="relationshipId",
        description="The id of the relationship as defined in the feature layer definition",
    )
    title: str | None = Field(
        None,
        description="A string value indicating what the element represents.",
    )
    type: Literal["relationship"] = Field(
        "relationship",
        description="String value indicating which type of element to use.",
    )


class FilterAuthoringInfoForFilterBlocks(BaseModel):
    """
    The filter authoring info object contains metadata about the authoring process for creating a filter block object. This allows the authoring client to save specific, overridable settings.  The next time it is accessed via an authoring client, their selections are remembered. Non-authoring clients can ignore it.
    """

    model_config = common_config
    filter_types: list[FilterType] = Field(
        ...,
        alias="filterTypes",
        description="Array of defined filter types. Each filter type has an array of filter values.",
    )


class LegendOptions(LegendOptions1):
    """
    Options for describing the heatmap in the legend.
    """

    max_label: Any | None = Field(None, alias="maxLabel")
    min_label: Any | None = Field(None, alias="minLabel")
    title: Any | None = None


class HeatmapRenderer(BaseModel):
    """
    The HeatmapRenderer renders point data into a raster visualization that emphasizes areas of higher density or weighted values.
    """

    model_config = common_config
    color_stops: list[HeatmapColorStop] = Field(
        ...,
        alias="colorStops",
        description="An array of colorStop objects describing the renderer's color ramp with more specificity than just colors.",
    )
    field: str | None = Field(
        None,
        description="This is optional as this renderer can be created if no field is specified. Each feature gets the same value/importance/weight or with a field where each feature is weighted by the field's value.",
    )
    legend_options: LegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="Options for describing the heatmap in the legend.",
    )
    max_density: float | None = Field(
        None,
        alias="maxDensity",
        description="The density value assigned to the final color in the `colorStops`.",
    )
    min_density: float | None = Field(
        None,
        alias="minDensity",
        description="The density value assigned to the first color in the `colorStops`.",
    )
    radius: float | None = Field(
        None,
        description="The radius (in points) of the circle representing each point.",
    )
    reference_scale: float | None = Field(
        0,
        alias="referenceScale",
        description="An optional reference scale to make the heatmap visualization static (i.e. then the heatmap radius is defined as points at this certain view scale). A value of 0 means that the heatmap does not have a fixed reference scale.",
    )
    type: Literal["heatmap"] = Field(
        "heatmap", description="Specifies the type of renderer used."
    )


class VolumeStyle(BaseModel):
    """
    The volume style allows you to define the exaggeration and offset.
    """

    model_config = common_config
    dynamic_sections: list[VoxelDynamicSection] | None = Field(
        None,
        alias="dynamicSections",
        description="Array of dynamic sections of the volume. (Only if `style.renderMode = surfaces`). Dynamic sections are planes through the voxel layer used for visual analysis to inspect the voxel layer at specified positions. Sections are visible if the voxel layer shows surfaces. For example, you can define a cross section diagram using dynamic sections.",
    )
    exaggeration_mode: ExaggerationMode | None = Field(
        ExaggerationMode.scale_height,
        validate_default=True,
        alias="exaggerationMode",
        description="Determines how the vertical exaggeration is applied. `scale-height` scales from the voxel dataset origin only, for example, if a voxel layer has its minimum at sea level the layer will be exaggerated starting from sea level. `scale-height` is the default. `scale-position` also scales the space between voxel dataset origin in the coordinate system origin. This exaggeration mode is identical with exaggeration applied to other layers like feature layers (use the scale position option if you want to draw the voxel layer together with feature based data).",
    )
    slices: list[VoxelSlice] | None = Field(
        None,
        description="Array of slices applied to this volume. Slices reduce the voxel volume to an area of interest. Slices are applied to both voxel style renderModes (volume, surfaces).",
        min_length=1,
    )
    vertical_exaggeration: float | None = Field(
        1,
        alias="verticalExaggeration",
        description="Vertical exaggeration factor.",
    )
    vertical_offset: float | None = Field(
        0,
        alias="verticalOffset",
        description="Vertical offset value in vertical unit of the spatial reference of the voxel layer.",
    )
    volume_id: int | None = Field(
        0,
        alias="volumeId",
        description="Id of the volume in the volume array (a maximum of one style per volumeId).",
    )


class CodedValue(BaseModel):
    """
    The coded value domain includes both the actual value that is stored in a database and a description of what the code value means.
    """

    model_config = common_config
    coded_values: list[CodedValue1] = Field(
        ...,
        alias="codedValues",
        description="A set of valid values with unique names.",
    )
    name: str | None = Field(None, description="The domain name.")
    type: Literal["codedValue"] = Field(
        "codedValue",
        description="String value representing the domain type.",
    )


class Callout(BaseModel):
    """
    Callout configuration for a symbol.
    """

    model_config = common_config
    border: Border | None = None
    color: list[conint(ge=0, le=255)] = Field(
        ..., description="The color of the line.", title="color"
    )
    size: confloat(ge=0.0) = Field(..., description="The width of the line in points.")
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="A value between `100` (full transparency) and `0` (full opacity).",
    )
    type: Literal["line"] = Field(
        "line",
        description="The type of the callout. A callout of type `line` connects an offset symbol or label with its location.",
    )


class TextSymbol3DLayer(BaseModel):
    """
    Symbol layer for text and font definitions.
    """

    model_config = common_config
    background: TextBackground | None = None
    enable: bool | None = None
    font: Font | None = None
    halo: Halo | None = None
    horizontal_alignment: HorizontalAlignment | None = Field(
        HorizontalAlignment.center,
        validate_default=True,
        alias="horizontalAlignment",
        description="One of the following string values representing the horizontal alignment of the text.",
    )
    line_height: confloat(ge=0.1, le=4.0) | None = Field(
        1,
        alias="lineHeight",
        description="Multiplier to scale the vertical distance between the baselines of text with multiple lines.",
    )
    material: Material | None = None
    size: confloat(ge=0.0) | None = Field(
        None, description="Font size in points, positive only"
    )
    text: str | None = Field(
        None,
        description="Text content in the label. Typically this property is not set, as text content is read from labeling field.",
    )
    type: Literal["Text"] = Field(
        "Text", description="Specifies the type of symbol used."
    )
    vertical_alignment: VerticalAlignment | None = Field(
        VerticalAlignment.baseline,
        validate_default=True,
        alias="verticalAlignment",
        description="One of the following string values representing the vertical alignment of the text.",
    )


class RasterColormapRenderer(BaseModel):
    """
    A value/color look up table used for rendering raster data only.
    """

    model_config = common_config
    colormap_infos: list[ColormapInfo] = Field(
        ...,
        alias="colormapInfos",
        description="An array of colormapInfo objects.",
    )
    type: Literal["rasterColormap"] = Field(
        "rasterColormap", description="Specifies the type of renderer used."
    )


class VoxelTransferFunctionStyle(BaseModel):
    """
    Defines the mapping between voxel values and color and transparency.
    """

    model_config = common_config
    alpha_stops: list[VoxelAlphaStop] | None = Field(
        None,
        alias="alphaStops",
        description="Describes the transparency stops (transparency mapping).",
        min_length=2,
    )
    color_stops: list[VoxelColorStop] | None = Field(
        None,
        alias="colorStops",
        description="Describes the color stops defining the color ramp.",
        min_length=2,
    )
    interpolation: Interpolation2 | None = Field(None, description="Interpolation mode")
    range_filter: VoxelRangeFilter | None = Field(
        None,
        alias="rangeFilter",
        description="Defines the range of voxels displayed. Voxels with values outside of this range will be discarded.",
    )
    stretch_range: list[float] = Field(
        ...,
        alias="stretchRange",
        description="Describes the low and high point for value-to-color mapping.",
        max_length=2,
        min_length=2,
    )


class AuthoringInfoField(BaseModel):
    """
    Contains information about an attribute field relating to Relationship renderers.
    """

    model_config = common_config
    class_break_infos: list[AuthoringInfoClassBreakInfo] | None = Field(
        None, alias="classBreakInfos"
    )
    field: str | None = Field(None, description="Attribute field used for renderer.")
    label: str | None = Field(
        None,
        description="The label used to describe the field or attribute in the legend.",
    )
    normalization_field: str | None = Field(
        None,
        alias="normalizationField",
        description="Attribute field used to normalize the data.",
    )


class ExtrudeSymbol3DLayer(BaseModel):
    """
    ExtrudeSymbol3DLayer is used to render Polygon geometries by extruding them upward from the ground, creating a 3D volumetric object.
    """

    model_config = common_config
    cast_shadows: bool | None = Field(
        True,
        alias="castShadows",
        description="Boolean to control the shadow casting behaviour of the rendered geometries.",
    )
    edges: SketchEdges | SolidEdges | None = Field(
        None,
        description="Specifies an edge visualization style.",
        title="Edges",
    )
    enable: bool | None = None
    material: Material | None = None
    size: float = Field(..., description="Extrusion height in meters.")
    type: Literal["Extrude"] = Field(
        "Extrude", description="Specifies the type of symbol used."
    )


class LineSymbol3DLayer(BaseModel):
    """
    LineSymbol3DLayer renders Polyline geometries using a flat 2D line with a LineSymbol3D in a 3D SceneView.
    """

    model_config = common_config
    cap: LineCap | None = LineCap.butt
    enable: bool | None = None
    join: Join | None = Field(
        Join.miter,
        validate_default=True,
        description="Shape of the intersection of two line segments.",
    )
    marker: LineMarker | None = None
    material: Material | None = None
    pattern: LinePattern | None = None
    size: confloat(ge=0.0) = Field(
        ..., description="Line width in points, positive only"
    )
    type: Literal["Line"] = Field(
        "Line", description="Specifies the type of symbol used."
    )


class ObjectSymbol3DLayer(BaseModel):
    """
    ObjectSymbol3DLayer is used to render Point geometries using a volumetric 3D shape (e.g., a sphere or cylinder) with a Symbol3D in a SceneView.
    """

    model_config = common_config
    anchor: Anchor1 | None = Field(
        Anchor1.origin,
        validate_default=True,
        description="The positioning of the object relative to the geometry.",
    )
    anchor_position: list[float] | None = Field(
        None,
        alias="anchorPosition",
        description="When `anchor` equals `relative`, this property specifies the positioning of the object relative to the geometry as a fraction of the symbol layer's bounding box. Otherwise it is ignored.",
        max_length=3,
        min_length=3,
    )
    cast_shadows: bool | None = Field(
        True,
        alias="castShadows",
        description="Boolean to control the shadow casting behaviour of the rendered geometries.",
    )
    depth: confloat(ge=0.0) | None = Field(
        None, description="Object depth in meters, positive only"
    )
    enable: bool | None = None
    heading: float | None = Field(
        None,
        description="Rotation angle around Z axis in degrees. At 0 degrees, the model points in the direction of the Y-axis. Positive values indicate clockwise rotation (when looked at from the top). [Detailed description](static/objectSymbolLayerOrientation.md).",
    )
    height: confloat(ge=0.0) | None = Field(
        None, description="Object height in meters, positive only"
    )
    material: Material | None = None
    resource: ObjectSymbol3DLayerResource | ObjectSymbol3DLayerResource1 | None = Field(
        None,
        description="The primitive shape (primitive) or external 3D model (href) used to visualize the points.",
        title="ObjectSymbol3DLayer Resource",
    )
    roll: float | None = Field(
        None,
        description="Rotation angle around Y axis in degrees. At 0 degrees, the model is level. A positive value lifts the left part and lowers the right part of the model. [Detailed description](static/objectSymbolLayerOrientation.md).",
    )
    tilt: float | None = Field(
        None,
        description="Rotation angle around X axis in degrees. At 0 degrees, the model is level. A positive value lifts the front and lowers the back of the model. [Detailed description](static/objectSymbolLayerOrientation.md).",
    )
    type: Literal["Object"] = Field(
        "Object", description="Specifies the type of symbol used."
    )
    width: confloat(ge=0.0) | None = Field(
        None, description="Object width in meters, positive only"
    )


class LegendOptions2(VisualVariableLegendOptions):
    show_legend: Any | None = Field(None, alias="showLegend")
    title: Any | None = None


class ColorInfoVisualVariable(BaseModel):
    """
    The colorInfo visual variable defines how a continuous color ramp is applied to features based on the values of a numeric field attribute.
    """

    model_config = common_config
    field: str | None = Field(
        None,
        description="Attribute field used for color rendering if no valueExpression is provided.",
    )
    legend_options: LegendOptions2 | None = Field(None, alias="legendOptions")
    normalization_field: str | None = Field(
        None,
        alias="normalizationField",
        description="Attribute field used to normalize the data.",
    )
    stops: list[ColorStop] | None = Field(None, description="An array of stop objects.")
    type: Literal["colorInfo"] = Field(
        "colorInfo", description="Specifies the type of visual variable."
    )
    value_expression: str | None = Field(
        None,
        alias="valueExpression",
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) that computes a value in lieu of a value provided by an attribute `field`.",
    )
    value_expression_title: str | None = Field(
        None,
        alias="valueExpressionTitle",
        description="The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property.",
    )


class RotationInfoVisualVariable(BaseModel):
    """
    The rotation visual variable defines how features rendered with marker symbols are rotated. The rotation value is determined by a value in a field or an Arcade expression calculating a value. Use either the `field` property or `valueExpression` when specifying rotation values.
    """

    model_config = common_config
    axis: Axis | None = Field(
        Axis.heading,
        validate_default=True,
        description="Defines the rotation axis the visual variable should be applied to when rendering features with an ObjectSymbol3DLayer. [Detailed description](static/objectSymbolLayerOrientation.md).",
    )
    field: str | None = Field(
        None,
        description="Attribute field used for setting the rotation of a symbol if no `valueExpression` is provided.",
    )
    legend_options: LegendOptions2 | None = Field(None, alias="legendOptions")
    rotation_type: RotationType | None = Field(
        RotationType.geographic,
        validate_default=True,
        alias="rotationType",
        description="Defines the origin and direction of rotation depending on how the angle of rotation was measured. Possible values are `geographic` which rotates the symbol from the north in a clockwise direction and `arithmetic` which rotates the symbol from the east in a counter-clockwise direction.",
    )
    type: Literal["rotationInfo"] = Field(
        "rotationInfo",
        description="A string value indicating the type of visual variable used for the renderer.",
    )
    value_expression: str | None = Field(
        None,
        alias="valueExpression",
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number.",
    )
    value_expression_title: str | None = Field(
        None,
        alias="valueExpressionTitle",
        description="The title identifying and describing the associated [Arcade expression] (https://developers.arcgis.com/arcade/) as defined in the `valueExpression` property.",
    )


class LegendOptions4(VisualVariableLegendOptions):
    custom_values: Any | None = Field(None, alias="customValues")
    show_legend: Any | None = Field(None, alias="showLegend")
    title: Any | None = None


class SizeInfoVisualVariable(BaseModel):
    """
    The sizeInfo visual variable defines how size is applied to features based on the values of a numeric field attribute. The minimum and maximum values of the data should be indicated along with their respective size values. You must specify either `minSize` and `maxSize`, or `stops`, or `valueUnit` to construct the size ramp. All features with values falling in between the specified min and max data values (or stops) will be scaled proportionally between the provided min and max sizes.
    """

    model_config = common_config
    axis: Axis1 | None = Field(
        Axis1.all,
        validate_default=True,
        description="Defines the axis the size visual variable should be applied to when rendering features with an ObjectSymbol3DLayer.",
    )
    field: str | None = Field(
        None,
        description="Attribute field used for size rendering if no valueExpression is provided.",
    )
    legend_options: LegendOptions4 | None = Field(None, alias="legendOptions")
    max_data_value: float | None = Field(
        None, alias="maxDataValue", description="The maximum data value."
    )
    max_size: float | None = Field(
        None,
        alias="maxSize",
        description="Specifies the maximum size to be applied to the symbol. This is required if valueUnit is set to `unknown`.",
    )
    min_data_value: float | None = Field(
        None, alias="minDataValue", description="The minimum data value."
    )
    min_size: float | None = Field(
        None,
        alias="minSize",
        description="Specifies the minimum size to be applied to the symbol. This is required if valueUnit is set to `unknown`.",
    )
    normalization_field: str | None = Field(
        None,
        alias="normalizationField",
        description="Attribute field used to normalize the data.",
    )
    stops: list[SizeStop] | None = Field(
        None,
        description="An array of objects that defines the thematic size ramp in a sequence of data or expression stops. At least two stops are required. The stops must be listed in ascending order based on the value of the `value` property in each stop. This property is required if `minDataValue`, `maxDataValue`, `minSize`, and `maxSize` are not defined.",
    )
    target: Literal["outline"] | None = Field(
        None,
        description="Only used when sizeInfo is used for polygon outlines.",
    )
    type: Literal["sizeInfo"] = Field(
        "sizeInfo", description="Specifies the type of visual variable."
    )
    use_symbol_value: bool | None = Field(
        None,
        alias="useSymbolValue",
        description="When setting a size visual variable on a renderer using an ObjectSymbol3DLayer, this property indicates whether to apply the value defined by the height, width, or depth properties to the corresponding axis of this visual variable instead of proportionally scaling this axis' value after other axes.",
    )
    value_expression: str | None = Field(
        None,
        alias="valueExpression",
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number.",
    )
    value_expression_title: str | None = Field(
        None,
        alias="valueExpressionTitle",
        description="The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property.",
    )
    value_representation: ValueRepresentation | None = Field(
        None,
        alias="valueRepresentation",
        description="Specifies how to apply the data value when mapping real-world sizes. See table below for supported values.",
    )
    value_unit: ValueUnit | None = Field(
        ValueUnit.meters,
        validate_default=True,
        alias="valueUnit",
        description="A string value indicating the unit of measurement. Defaults to `meters` if not set.",
    )


class LegendOptions5(VisualVariableLegendOptions):
    show_legend: Any | None = Field(None, alias="showLegend")
    title: Any | None = None


class TransparencyInfoVisualVariable(BaseModel):
    """
    The transparencyInfo visual variable defines the transparency, or opacity, of each feature's symbol based on a numeric attribute field value.
    """

    model_config = common_config
    field: str | None = Field(
        None,
        description="Attribute field used for setting the transparency of a feature if no `valueExpression` is provided.",
    )
    legend_options: LegendOptions5 | None = Field(None, alias="legendOptions")
    normalization_field: str | None = Field(
        None,
        alias="normalizationField",
        description="Attribute field used to normalize the data.",
    )
    stops: list[TransparencyStop] | None = Field(
        None, description="An array of transparencyStop objects."
    )
    type: Literal["transparencyInfo"] = Field(
        "transparencyInfo",
        description="Specifies the type of visual variable.",
    )
    value_expression: str | None = Field(
        None,
        alias="valueExpression",
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number.",
    )
    value_expression_title: str | None = Field(
        None,
        alias="valueExpressionTitle",
        description="The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property.",
    )


class Outline1(BaseModel):
    """
    The outline of the symbol layer.
    """

    model_config = common_config
    color: list[conint(ge=0, le=255)] = Field(
        ...,
        description="Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
        title="color",
    )
    pattern: LinePattern | None = None
    pattern_cap: LineCap | None = Field(
        LineCap.butt, validate_default=True, alias="patternCap"
    )
    size: confloat(ge=0.0) = Field(
        ..., description="Outline size in points, positive only"
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="The value has to lie between `100` (full transparency) and `0` (full opacity).",
    )


class PathSymbol3DLayer(BaseModel):
    """
    PathSymbol3DLayer renders polyline geometries by extruding a 2D profile along the line, resulting in visualizations like tubes, walls, etc.
    """

    model_config = common_config
    anchor: Anchor2 | None = Field(
        Anchor2.center,
        validate_default=True,
        description="The position of the extrusion profile with respect to the polyline geometry.",
    )
    cap: PathCap | None = PathCap.butt
    cast_shadows: bool | None = Field(
        True,
        alias="castShadows",
        description="Boolean to control the shadow casting behaviour of the rendered geometries.",
    )
    enable: bool | None = None
    height: confloat(ge=0.0) | None = Field(
        None,
        description="Path height in meters. If unspecified, it is equal to `width`.",
    )
    join: Join | None = Field(
        Join.miter,
        validate_default=True,
        description="Shape of the intersection of two line segments.",
    )
    material: Material | None = None
    profile: Profile | None = Field(
        Profile.circle,
        validate_default=True,
        description="The shape which is extruded along the line.",
    )
    profile_rotation: ProfileRotation | None = Field(
        ProfileRotation.all,
        validate_default=True,
        alias="profileRotation",
        description="Specifies the axes about which the profile may be rotated at the joins. Constraining the rotation axes leads to a fixed orientation of the profile for the specified directions.",
    )
    size: confloat(ge=0.0) = Field(
        ...,
        description="Path size (diameter) in meters. Ignored if either `width` or `height` are present.",
    )
    type: Literal["Path"] = Field(
        "Path", description="Specifies the type of symbol used."
    )
    width: confloat(ge=0.0) | None = Field(
        None,
        description="Path width in meters. If unspecified, it is equal to `height`.",
    )


class PathSymbol3DLayer1(BaseModel):
    """
    PathSymbol3DLayer renders polyline geometries by extruding a 2D profile along the line, resulting in visualizations like tubes, walls, etc.
    """

    model_config = common_config
    anchor: Anchor2 | None = Field(
        Anchor2.center,
        validate_default=True,
        description="The position of the extrusion profile with respect to the polyline geometry.",
    )
    cap: PathCap | None = PathCap.butt
    cast_shadows: bool | None = Field(
        True,
        alias="castShadows",
        description="Boolean to control the shadow casting behaviour of the rendered geometries.",
    )
    enable: bool | None = None
    height: confloat(ge=0.0) | None = Field(
        None,
        description="Path height in meters. If unspecified, it is equal to `width`.",
    )
    join: Join | None = Field(
        Join.miter,
        validate_default=True,
        description="Shape of the intersection of two line segments.",
    )
    material: Material | None = None
    profile: Profile | None = Field(
        Profile.circle,
        validate_default=True,
        description="The shape which is extruded along the line.",
    )
    profile_rotation: ProfileRotation | None = Field(
        ProfileRotation.all,
        validate_default=True,
        alias="profileRotation",
        description="Specifies the axes about which the profile may be rotated at the joins. Constraining the rotation axes leads to a fixed orientation of the profile for the specified directions.",
    )
    size: confloat(ge=0.0) | None = Field(
        None,
        description="Path size (diameter) in meters. Ignored if either `width` or `height` are present.",
    )
    type: Literal["Path"] = Field(
        "Path", description="Specifies the type of symbol used."
    )
    width: confloat(ge=0.0) = Field(
        ...,
        description="Path width in meters. If unspecified, it is equal to `height`.",
    )


class PathSymbol3DLayer2(BaseModel):
    """
    PathSymbol3DLayer renders polyline geometries by extruding a 2D profile along the line, resulting in visualizations like tubes, walls, etc.
    """

    model_config = common_config
    anchor: Anchor2 | None = Field(
        Anchor2.center,
        validate_default=True,
        description="The position of the extrusion profile with respect to the polyline geometry.",
    )
    cap: PathCap | None = PathCap.butt
    cast_shadows: bool | None = Field(
        True,
        alias="castShadows",
        description="Boolean to control the shadow casting behaviour of the rendered geometries.",
    )
    enable: bool | None = None
    height: confloat(ge=0.0) = Field(
        ...,
        description="Path height in meters. If unspecified, it is equal to `width`.",
    )
    join: Join | None = Field(
        Join.miter,
        validate_default=True,
        description="Shape of the intersection of two line segments.",
    )
    material: Material | None = None
    profile: Profile | None = Field(
        Profile.circle,
        validate_default=True,
        description="The shape which is extruded along the line.",
    )
    profile_rotation: ProfileRotation | None = Field(
        ProfileRotation.all,
        validate_default=True,
        alias="profileRotation",
        description="Specifies the axes about which the profile may be rotated at the joins. Constraining the rotation axes leads to a fixed orientation of the profile for the specified directions.",
    )
    size: confloat(ge=0.0) | None = Field(
        None,
        description="Path size (diameter) in meters. Ignored if either `width` or `height` are present.",
    )
    type: Literal["Path"] = Field(
        "Path", description="Specifies the type of symbol used."
    )
    width: confloat(ge=0.0) | None = Field(
        None,
        description="Path width in meters. If unspecified, it is equal to `height`.",
    )


class Ground(BaseModel):
    """
    Ground defines the main surface of the web scene, based on elevation layers.
    """

    model_config = common_config
    layers: list[RasterDataElevationLayer | TiledElevationLayer] = Field(
        ...,
        description="An array of elevationLayer objects defining the elevation of the ground in the web scene.",
    )
    navigation_constraint: NavigationConstraint | None = Field(
        None,
        alias="navigationConstraint",
        description="Object determining whether the camera is constrained to navigate only above, or also under the ground surface. If not specified, navigation is constrained to above ground.",
    )
    surface_color: list[conint(ge=0, le=255)] | None = Field(
        None,
        alias="surfaceColor",
        description="Defines the color of the ground surface, displayed underneath the basemap. If no color, the default grid is shown.",
        title="color",
    )
    transparency: conint(ge=0, le=100) | None = Field(
        None,
        description="The transparency of the ground surface. It is used for seeing through the ground, therefore this property also changes the transparency of the basemap. Draped operational layers are not affected by this property. The value has to lie between `100` (full transparency) and `0` (full opacity).",
    )


class Environment(BaseModel):
    """
    Represents settings that affect the environment in which the web scene is displayed. It is entirely stored as part of the initial state of the web scene, and partially in the slides in the presentation.
    """

    model_config = common_config
    atmosphere_enabled: bool | None = Field(
        True,
        alias="atmosphereEnabled",
        description="Whether the atmosphere should be visualized. This includes sky and haze effects.",
    )
    background: ColorBackground | None = Field(
        None,
        description="The background is what is displayed behind any scene objects, stars and atmosphere.",
    )
    lighting: SunLighting | VirtualLighting | None = Field(
        None,
        description="Object containing information on how the scene is lit.",
        title="lighting",
    )
    stars_enabled: bool | None = Field(
        True,
        alias="starsEnabled",
        description="Whether stars should be displayed in the sky.",
    )
    weather: (
        CloudyWeather | FoggyWeather | RainyWeather | SnowyWeather | SunnyWeather | None
    ) = Field(
        {"$ref": "#/definitions/sunnyWeather_schema.json"},
        description="Indicates the type of weather visualization in the scene.",
    )


class Viewpoint(BaseModel):
    """
    The location or camera position from which to view the scene.
    """

    model_config = common_config
    camera: Camera
    rotation: confloat(ge=0.0, le=360.0) | None = Field(
        None,
        description="The rotation of due north in relation to the top of the view in degrees.",
    )
    scale: float | None = Field(None, description="The scale of the viewpoint.")
    target_geometry: (
        Extent
        | dict
        | MultipointGeometry
        | PointGeometry
        | PolygonGeometry
        | PolylineGeometry
        | None
    ) = Field(
        None,
        alias="targetGeometry",
        description="The target geometry framed by the viewpoint.",
    )


class DimensionLayer(BaseModel):
    """
    Dimension layers can be used to visually annotate the lengths between points in a map. They can be created by embedding the dimensions directly in the layer.
    """

    model_config = common_config
    dimensions: list[LengthDimension] | None = Field(
        None,
        description="A collection of dimension objects embedded in the layer.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    layer_type: Literal["ArcGISDimensionLayer"] = Field(
        "ArcGISDimensionLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        None,
        alias="listMode",
        description="To show or hide the layer in the layer list.",
    )
    style: DimensionSimpleStyle | None = Field(
        None,
        description="Specification of how dimensions and their labels are displayed.",
    )
    title: str = Field(
        ...,
        description="A human readable string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is visible in the scene.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",
    )


class MediaLayerMedia(BaseModel):
    """
    Media layer displays one media on the scene positioned by 4 control points.
    """

    model_config = common_config
    georeference: Georeference | None = None
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the `Media Layer` item registered on ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["MediaLayer"] = Field(
        "MediaLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        None,
        alias="listMode",
        description="To show or hide the layer in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        0,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    media_type: Literal["image"] = Field(
        "image",
        alias="mediaType",
        description="Indicates the type of media the url points to. The currently only supported type is `image` in jpeg or png format. It is applicable only when `url` is defined.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        0,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(
        ...,
        description="URL to the media file. The url is relative when the media file is stored as an item resource. It is not applicable when `itemId` is defined. See related `mediaType` property.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",
    )


class MediaLayerMedia1(BaseModel):
    """
    Media layer displays one media on the scene positioned by 4 control points.
    """

    model_config = common_config
    georeference: Georeference | None = None
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str = Field(
        ...,
        alias="itemId",
        description="Optional string containing the item ID of the `Media Layer` item registered on ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["MediaLayer"] = Field(
        "MediaLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        None,
        alias="listMode",
        description="To show or hide the layer in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        0,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    media_type: Literal["image"] | None = Field(
        None,
        alias="mediaType",
        description="Indicates the type of media the url points to. The currently only supported type is `image` in jpeg or png format. It is applicable only when `url` is defined.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        0,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str | None = Field(
        None,
        description="URL to the media file. The url is relative when the media file is stored as an item resource. It is not applicable when `itemId` is defined. See related `mediaType` property.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )


class WebTiledLayerWebTiledLayer(BaseModel):
    """
    A tile layer is a derived from a set of web-accessible tiles which reside on a server. The tiles are accessed by a direct URL request from the web browser. Because the tiles in a tile layer are not available as a service, they must be in a specific format for a web app such as the ArcGIS.com scene viewer to display the layer on a scene.
    """

    model_config = common_config
    blend_mode: BlendMode | None = Field(
        BlendMode.normal, validate_default=True, alias="blendMode"
    )
    copyright: str | None = Field(
        None,
        description="Attribution to the Web Tiled Layer provider. It is displayed in the attribution on the web scene. Input required by the user when the layer is added to the web scene.",
    )
    full_extent: Extent | dict | None = Field(
        None,
        alias="fullExtent",
        description="An extent object representing the full extent envelope for the layer.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="This is applicable if used as a basesceneLayer. A boolean value indicating whether or not the basesceneLayer draws on top (true) of other layers, including operationalLayers , or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Unique string value indicating an item registered in ArcGIS Online or your organization's portal.",
    )
    layer_type: Literal["WebTiledLayer"] = Field(
        "WebTiledLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the layer in the layer list",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    sub_domains: list[str] | None = Field(
        None,
        alias="subDomains",
        description="If subdomains are detected, they must be specified. The scene viewer detects if the Web Tiled Layer has subdomains by parsing the templateURL value for {subDomain}.",
    )
    template_url: str | None = Field(
        None,
        alias="templateUrl",
        description="URL to the Web Tiled Layer. Input required by the user when the layer is added to the web scene. The template URL contains a parameterized URL. The URL can contain the following templated parameters: `level`, `col`, `row`, and `subDomain`.",
    )
    tile_info: TileInfo | None = Field(
        None,
        alias="tileInfo",
        description="Contains the spatial reference and the tiling scheme of the layer. Typically retrieved from a WMTS OGC Web Service. If missing the layer must be in the WGS 1984 Web Mercator (Auxiliary Sphere) tiling scheme.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",
    )
    wmts_info: WebMapTileServiceInfo | None = Field(
        None,
        alias="wmtsInfo",
        description="Object containing information about the chosen WMTS service layer and tiling schema.",
    )


class Environment1(Environment):
    """
    Represents settings that affect the environment in which the web scene is displayed.
    """

    lighting: Any | None = None
    weather: Any | None = None


class Search(BaseModel):
    """
    An object specifying the search parameters set within the web scene.
    """

    model_config = common_config
    enabled: bool | None = Field(
        True,
        description="A boolean value indicating whether search functionality is enabled in the web scene.",
    )
    hint_text: str | None = Field(
        None,
        alias="hintText",
        description="A string value used to indicate the hint provided with the search dialog.",
    )
    layers: list[SearchLayer] | None = Field(
        None,
        description="An array of objects that define search fields and search criteria for layers.",
    )
    tables: list[SearchTable] | None = Field(
        None,
        description="An array of objects that define search fields and search criteria for tables in the web map.",
    )


class RenderingRule(BaseModel):
    """
    Specifies the rendering rule for how the requested image should be rendered.
    """

    model_config = common_config
    output_pixel_type: OutputPixelType | None = Field(
        OutputPixelType.unknown,
        validate_default=True,
        alias="outputPixelType",
        description="the output pixel type defines the output image's pixel type.",
    )
    raster_function: str | None = Field(
        None,
        alias="rasterFunction",
        description="The raster function name identifies the processing or rendering to be performed. For a list of possible types, please see the [Raster Functions](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm) help topic for additional information on this.",
    )
    raster_function_arguments: dict[str, Any] | None = Field(
        None,
        alias="rasterFunctionArguments",
        description="The arguments for the referenced `rasterFunction`. For a description of arguments per raster function type, please see the [Raster Functions](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm) help topic for additional information on this.",
    )
    raster_function_definition: RasterFunctionTemplate | None = Field(
        None,
        alias="rasterFunctionDefinition",
        description="Specifies the raster function template for how the requested image should be processed.",
    )
    variable_name: str | None = Field(
        None,
        alias="variableName",
        description="Variable name for the raster function.",
    )


class LineOfSightObserver(BaseModel):
    """
    Defines the observer of a line of sight analysis in a LineOfSight layer.
    """

    model_config = common_config
    elevation_info: ElevationInfo | None = Field(None, alias="elevationInfo")
    feature: FeatureReference | None = None
    position: PointGeometry = Field(
        ..., description="Position of the line of sight observer."
    )


class LineOfSightTarget(BaseModel):
    """
    Defines the target of a line of sight analysis in a LineOfSight layer.
    """

    model_config = common_config
    elevation_info: ElevationInfo | None = Field(None, alias="elevationInfo")
    feature: FeatureReference | None = None
    position: PointGeometry = Field(
        ..., description="Position of the line of sight target."
    )


class MetadataForCheckboxBasedFilterUI(BaseModel):
    """
    Client UI with checkbox representation for each filter type and filter value.
    """

    model_config = common_config
    filter_blocks: list[FilterAuthoringInfoForFilterBlocks] = Field(
        ...,
        alias="filterBlocks",
        description="Array of filter block authoring infos.",
    )
    type: Literal["checkbox"] = Field(
        "checkbox", description="Type of filter authoring info."
    )


class PopupElementMedia(BaseModel):
    """
    Configures media in popup elements.
    """

    model_config = common_config
    attributes: dict[str, Any] | None = Field(
        None,
        description="A dictionary of key value pairs representing attributes to be used instead of fields and their values.  This property is only used when an element of type  `media` is being returned inside an element of type `expression` and should be returned as part of the arcade expression itself. This property allows passing arcade derived attribute values into `mediaInfos` such as charts. More details can be found [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element).",
    )
    description: str | None = Field(
        None,
        description="An optional string value describing the element in detail. This property does not apply when the `type` is `text` or `expression`.",
    )
    media_infos: list[MediaInfoChart | MediaInfoImage] | None = Field(
        None,
        alias="mediaInfos",
        description="An array of `mediaInfo` objects representing an image or chart for display. If no `mediaInfos` property is provided, the popupElement will display whatever is specified in the `popupInfo.mediaInfos` property.",
    )
    title: str | None = Field(
        None,
        description="An optional string value indicating what the element represents. This property does not apply when the `type` is `text` or `expression`.",
    )
    type: Literal["media"] = "media"


class LabelSymbol3D(BaseModel):
    """
    LabelSymbol3D is used to render labels for features from a FeatureLayer in a 3D SceneView.
    """

    model_config = common_config
    callout: Callout | None = None
    symbol_layers: list[TextSymbol3DLayer] = Field(
        ...,
        alias="symbolLayers",
        description="A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",
    )
    type: Literal["LabelSymbol3D"] = Field(
        "LabelSymbol3D", description="Specifies the type of symbol used."
    )
    vertical_offset: VerticalOffset | None = Field(None, alias="verticalOffset")


class VoxelVariableStyle(BaseModel):
    """
    The voxel variable style defines how the voxel layer will render for a variable. A variable can be discrete (integer values) or continuous (float values).
    """

    model_config = common_config
    isosurfaces: list[VoxelIsosurfaceStyle] | None = Field(
        None,
        description="Array of styles for isosurfaces. (Only if `variable.originalFormat.continuity = continuous`). An isosurface represents a surface at a specific value. A voxel layer can have up to four isosurfaces.",
        min_length=1,
    )
    label: str | None = Field(None, description="Label for the variable.")
    transfer_function: VoxelTransferFunctionStyle | None = Field(
        None,
        alias="transferFunction",
        description="Defines the stretch rendering of the voxel layer. The transfer function maps voxel values to color and transparency. Scalar field only. (Only if `variable.originalFormat.continuity = continuous`).",
    )
    unique_values: list[VoxelUniqueValue] | None = Field(
        None,
        alias="uniqueValues",
        description="Defines the unique value rendering as an array of unique value styles (Only if `variable.originalFormat.continuity = discrete`).",
        min_length=0,
    )
    variable_id: int = Field(
        ...,
        alias="variableId",
        description="Id of the variable. The variable styles will be defined for the variable with the given Id. If the `variableId` and the voxel style `currentVariableId` are identical the style will be shown. ",
    )


class FilterModeWireFrame(BaseModel):
    """
    Draw elements of this filter block in wireframe mode. This mode draws only the edges of the features with the specified edge style.
    """

    model_config = common_config
    edges: SketchEdges | SolidEdges = Field(
        ...,
        description="An object defining edges of a feature.",
        title="Edges",
    )
    type: Literal["wireFrame"] = Field(
        "wireFrame", description="Declares filter mode of type wire frame."
    )


class AuthoringInfo(BaseModel):
    """
    The authoringInfo is an object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via an authoring client, their selections are remembered. Non-authoring clients can ignore it. Properties for color/size/transparency sliders, theme selection, classification information, and additional properties are saved within this object.
    """

    model_config = common_config
    classification_method: ClassificationMethod | None = Field(
        ClassificationMethod.esri_classify_manual,
        validate_default=True,
        alias="classificationMethod",
        description="Used for classed color or size. The default value is `esriClassifyManual`.",
    )
    color_ramp: ColorRampAlgorithmic | ColorRampMultipart | None = Field(
        None,
        alias="colorRamp",
        description="A colorRamp object is used to specify a range of colors that are applied to a group of symbols.",
        title="Color Ramp",
    )
    field1: AuthoringInfoField | None = None
    field2: AuthoringInfoField | None = None
    fields: list[str] | None = Field(
        None,
        description="An array of string values representing field names used for creating predominance renderers.",
    )
    focus: Focus | None = Field(
        None,
        description="Optional. Used for Relationship renderer. If not set, the legend will default to being square.",
    )
    length_unit: LengthUnit | None = Field(
        None,
        alias="lengthUnit",
        description="Unit used in user interfaces to display world/map sizes and distances",
    )
    num_classes: conint(ge=2, le=4) | None = Field(
        None,
        alias="numClasses",
        description="Number of classes to be associated with the relationship. Used for Relationship renderer.",
    )
    standard_deviation_interval: StandardDeviationInterval | None = Field(
        None,
        alias="standardDeviationInterval",
        description="Use this property if the classificationMethod is `esriClassifyStandardDeviation`.",
    )
    type: Type7 | None = None
    visual_variables: list[AuthoringInfoVisualVariable] | None = Field(
        None,
        alias="visualVariables",
        description="An array of visualVariable objects containing additional information needed when authoring the renderer.",
    )


class PointCloudClassBreaksRenderer(BaseModel):
    """
    PointCloudClassBreaksRenderer defines the color of each point in a PointCloudLayer based on the value of a numeric attribute. Colors are assigned based on classes or ranges of data. Each point is assigned a symbol based on the class break in which the value of the attribute falls.
    """

    model_config = common_config
    color_class_break_infos: list[ColorClassBreakInfo] = Field(
        ...,
        alias="colorClassBreakInfos",
        description="Each element in the array is an object that provides information about a class break associated with the renderer.",
    )
    color_modulation: ColorModulationInfo | None = Field(None, alias="colorModulation")
    field: str = Field(
        ...,
        description="  The name of the field that is used for the renderer.",
    )
    field_transform_type: FieldTransformType | None = Field(
        None,
        alias="fieldTransformType",
        description="A transform that is applied to the field value before evaluating the renderer.",
    )
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    point_size_algorithm: (
        PointCloudFixedSizeAlgorithm | PointCloudSplatAlgorithm | None
    ) = Field(
        None,
        alias="pointSizeAlgorithm",
        description="Size algorithms for point cloud performance.",
        title="PointCloud Size Algorithm",
    )
    points_per_inch: float | None = Field(
        None,
        alias="pointsPerInch",
        description="Number of point to draw per display inch.",
    )
    type: Literal["pointCloudClassBreaksRenderer"] = Field(
        "pointCloudClassBreaksRenderer",
        description="Specifies the type of renderer used.",
    )


class PointCloudRGBRenderer(BaseModel):
    """
    PointCloudRGBRenderer defines the color of each point in a PointCloudLayer based on the value of a color attribute.
    """

    model_config = common_config
    color_modulation: ColorModulationInfo | None = Field(None, alias="colorModulation")
    field: str = Field(
        ...,
        description="The name of the field that is used for the renderer.",
    )
    point_size_algorithm: (
        PointCloudFixedSizeAlgorithm | PointCloudSplatAlgorithm | None
    ) = Field(
        None,
        alias="pointSizeAlgorithm",
        description="Size algorithms for point cloud performance.",
        title="PointCloud Size Algorithm",
    )
    points_per_inch: float | None = Field(
        None,
        alias="pointsPerInch",
        description="Number of point to draw per display inch.",
    )
    type: Literal["pointCloudRGBRenderer"] = Field(
        "pointCloudRGBRenderer",
        description="Specifies the type of renderer used.",
    )


class PointCloudStretchRenderer(BaseModel):
    """
    PointCloudStretchRenderer defines the color of each point in a PointCloudLayer based on the value of a numeric attribute. They allow you to easily map continuous color ramps to minimum and maximum data values of one of the layer's numeric attribute fields.
    """

    model_config = common_config
    color_modulation: ColorModulationInfo | None = Field(None, alias="colorModulation")
    field: str = Field(
        ...,
        description="The name of the field that is used for the renderer.",
    )
    field_transform_type: FieldTransformType | None = Field(
        None,
        alias="fieldTransformType",
        description="A transform that is applied to the field value before evaluating the renderer.",
    )
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    point_size_algorithm: (
        PointCloudFixedSizeAlgorithm | PointCloudSplatAlgorithm | None
    ) = Field(
        None,
        alias="pointSizeAlgorithm",
        description="Size algorithms for point cloud performance.",
        title="PointCloud Size Algorithm",
    )
    points_per_inch: float | None = Field(
        None,
        alias="pointsPerInch",
        description="Number of points to draw per display inch.",
    )
    stops: list[ColorStop] = Field(..., description="An array of color value pairs.")
    type: Literal["pointCloudStretchRenderer"] = Field(
        "pointCloudStretchRenderer",
        description="Specifies the type of renderer used.",
    )


class PointCloudUniqueValueRenderer(BaseModel):
    """
    PointCloudUniqueValueRenderer allows you to colorize points in a PointCloudLayer based on an attribute. This is done by using unique colors to represent points with equal attribute values.
    """

    model_config = common_config
    color_modulation: ColorModulationInfo | None = Field(None, alias="colorModulation")
    color_unique_value_infos: list[ColorUniqueValueInfo] = Field(
        ..., alias="colorUniqueValueInfos", description="Unique value infos."
    )
    field: str = Field(
        ...,
        description="The name of the field that is used for the renderer.",
    )
    field_transform_type: FieldTransformType | None = Field(
        None,
        alias="fieldTransformType",
        description="A transform that is applied to the field value before evaluating the renderer.",
    )
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    point_size_algorithm: (
        PointCloudFixedSizeAlgorithm | PointCloudSplatAlgorithm | None
    ) = Field(
        None,
        alias="pointSizeAlgorithm",
        description="Size algorithms for point cloud performance.",
        title="PointCloud Size Algorithm",
    )
    points_per_inch: float | None = Field(
        None,
        alias="pointsPerInch",
        description="Number of points to draw per display inch.",
    )
    type: Literal["pointCloudUniqueValueRenderer"] = Field(
        "pointCloudUniqueValueRenderer",
        description="Specifies the type of renderer used.",
    )


class RasterClassBreaksRenderer(BaseModel):
    """
    Defines the color of each raster cell based on the value of a numeric attribute. Colors are assigned based on classed ranges of data.
    """

    model_config = common_config
    authoring_info: AuthoringInfo | None = Field(
        None,
        alias="authoringInfo",
        description="An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",
    )
    color_class_break_infos: list[ColorClassBreakInfo] = Field(
        ...,
        alias="colorClassBreakInfos",
        description="Each element in the array is an object that provides information about a class break associated with the renderer.",
    )
    default_color: list[conint(ge=0, le=255)] | None = Field(
        None,
        alias="defaultColor",
        description="The defaultColor on the renderer that get assigned to features with no value or features that do not fall within the configured data.",
        title="color",
    )
    default_label: str | None = Field(
        None,
        alias="defaultLabel",
        description="Default label for the default symbol used to draw unspecified values.",
    )
    field: str = Field(
        ...,
        description="The name of the field that is used for the renderer.",
    )
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    normalization_field: str | None = Field(
        None,
        alias="normalizationField",
        description="Used when normalizationType is field. The string value indicating the attribute field by which the data value is normalized.",
    )
    normalization_total: float | None = Field(
        None,
        alias="normalizationTotal",
        description="Used when normalizationType is percent-of-total, this number property contains the total of all data values.",
    )
    normalization_type: NormalizationType | None = Field(
        None,
        alias="normalizationType",
        description="Determine how the data was normalized.",
    )
    type: Literal["rasterClassBreaksRenderer"] = Field(
        "rasterClassBreaksRenderer",
        description="Specifies the type of renderer used.",
    )


class RasterShadedReliefRenderer(BaseModel):
    """
    This renderer produces a grayscale or a colored 3D representation of a shaded relief. This renderer can be used on either layers of type: [ImageServiceLayer](imageServiceLayer.md) or [TiledImageServiceLayer](tiledImageServiceLayer.md).
    """

    model_config = common_config
    altitude: float = Field(
        ...,
        description="Used when the `hillshadeType` is `traditional`. The  altitude of the illumination source.",
    )
    azimuth: float = Field(
        ...,
        description="Used when the `hillshadeType` is `traditional`. The azimuth of the illumination source.",
    )
    color_ramp: ColorRampAlgorithmic | ColorRampMultipart | None = Field(
        None,
        alias="colorRamp",
        description="The color ramp to display the shaded relief. When not specified, a grayscale hillshade is produced. When this is specified, grayscale hillshade is fused with colorized elevation.",
        title="Color Ramp",
    )
    hillshade_type: HillshadeType = Field(
        ...,
        alias="hillshadeType",
        description="Use single (traditional), or multiple illumination sources to generate hillshade.",
    )
    pixel_size_factor: float | None = Field(
        None,
        alias="pixelSizeFactor",
        description="Accounts for changes in scale as the viewer zooms in and out on the map display.",
    )
    pixel_size_power: float | None = Field(
        None,
        alias="pixelSizePower",
        description="Accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.",
    )
    scaling_type: ScalingType = Field(
        ...,
        alias="scalingType",
        description="Apply a constant or adjusted z-factor based on resolution changes. The `adjusted` type is suitable for worldwide elevation dataset. An adjusted `zFactor` is determined using this equation: `Adjusted Z-Factor = (Z Factor) + (Pixel Size)` <sup>(Pixel Size Power)</sup> `x Pixel Size Factor`.",
    )
    type: Literal["rasterShadedRelief"] = Field(
        "rasterShadedRelief",
        description="Specifies the type of renderer used.",
    )
    z_factor: float = Field(
        ...,
        alias="zFactor",
        description="A conversion factor that adjusts the units of measure for the vertical (or elevation) units when they are different from the horizontal coordinate (x,y) units of the input surface. It is the number of ground x,y units in one surface z-unit.",
    )


class RasterStretchRenderer(BaseModel):
    """
    Displays continuous raster cell values across a gradual ramp of colors. Use this renderer to draw a single band of continuous data. This renderer works well when you have a large range of values to display, such as with imagery or scientific data.
    """

    model_config = common_config
    color_ramp: ColorRampAlgorithmic | ColorRampMultipart | None = Field(
        None,
        alias="colorRamp",
        description="A colorRamp object is used to specify a range of colors that are applied to a group of symbols.",
        title="Color Ramp",
    )
    compute_gamma: bool | None = Field(
        None,
        alias="computeGamma",
        description="Indicates if gamma values should be computed by default.",
    )
    dra: bool | None = Field(
        None,
        description="Indicates if Dynamic Range Adjustment should be applied.",
    )
    gamma: list[float] | None = Field(None, description="The list of Gamma value(s).")
    max: float | None = Field(None, description="The current maximum output value.")
    max_percent: float | None = Field(
        None,
        alias="maxPercent",
        description="The current maximum percent value.",
    )
    min: float | None = Field(None, description="The current minimum output value.")
    min_percent: float | None = Field(
        None,
        alias="minPercent",
        description="The current minimum percent value.",
    )
    number_of_standard_deviations: float | None = Field(
        None,
        alias="numberOfStandardDeviations",
        description="The number of standard deviations for standard deviation stretch.",
    )
    sigmoid_strength_level: float | None = Field(
        None,
        alias="sigmoidStrengthLevel",
        description="Set this from (1 to 6) to adjust the curvature of Sigmoid curve used in color stretch.",
    )
    statistics: list[list[float]] | None = Field(
        None, description="The custom raster stretch statistics."
    )
    stretch_type: StretchType | None = Field(
        None,
        alias="stretchType",
        description="The stretch types for stretch raster function.",
    )
    type: Literal["rasterStretch"] | None = Field(
        None, description="Specifies the type of renderer used."
    )
    use_gamma: bool | None = Field(
        None,
        alias="useGamma",
        description="Indicates if the renderer applies Gamma stretch.",
    )


class RasterUniqueValueRenderer(BaseModel):
    """
    Defines the color of each raster cell based on an attribute. This is done by using unique colors to represent cells with equal attribute values.
    """

    model_config = common_config
    authoring_info: AuthoringInfo | None = Field(
        None,
        alias="authoringInfo",
        description="An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",
    )
    color_unique_value_infos: list[ColorUniqueValueInfo] = Field(
        ..., alias="colorUniqueValueInfos", description="Unique value infos."
    )
    default_color: list[conint(ge=0, le=255)] | None = Field(
        None,
        alias="defaultColor",
        description="The defaultColor on the renderer that get assigned to features with no value or features that do not fall within the configured data.",
        title="color",
    )
    default_label: str | None = Field(
        None,
        alias="defaultLabel",
        description="Default label for the default symbol used to draw unspecified values.",
    )
    field: str = Field(
        ...,
        description="The name of the field that is used for the renderer.",
    )
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    type: Literal["rasterUniqueValueRenderer"] = Field(
        "rasterUniqueValueRenderer",
        description="Specifies the type of renderer used.",
    )


class MarkerPlacement(Enum):
    """
    Indicates where the marker is placed.
    """

    begin = "begin"
    begin_end = "begin-end"
    end = "end"


class SimpleLineSymbolStyle(Enum):
    """
    String value representing the simple line symbol type.
    """

    esri_sls_dash = "esriSLSDash"
    esri_sls_dash_dot = "esriSLSDashDot"
    esri_sls_dash_dot_dot = "esriSLSDashDotDot"
    esri_sls_dot = "esriSLSDot"
    esri_sls_long_dash = "esriSLSLongDash"
    esri_sls_long_dash_dot = "esriSLSLongDashDot"
    esri_sls_null = "esriSLSNull"
    esri_sls_short_dash = "esriSLSShortDash"
    esri_sls_short_dash_dot = "esriSLSShortDashDot"
    esri_sls_short_dash_dot_dot = "esriSLSShortDashDotDot"
    esri_sls_short_dot = "esriSLSShortDot"
    esri_sls_solid = "esriSLSSolid"


class SimpleFillSymbolStyle(Enum):
    """
    String value representing the simple fill symbol type.
    """

    esri_sfs_backward_diagonal = "esriSFSBackwardDiagonal"
    esri_sfs_cross = "esriSFSCross"
    esri_sfs_diagonal_cross = "esriSFSDiagonalCross"
    esri_sfs_forward_diagonal = "esriSFSForwardDiagonal"
    esri_sfs_horizontal = "esriSFSHorizontal"
    esri_sfs_null = "esriSFSNull"
    esri_sfs_solid = "esriSFSSolid"
    esri_sfs_vertical = "esriSFSVertical"


class SimpleMarkerSymbolStyle(Enum):
    """
    String value representing the simple marker type.
    """

    esri_sms_circle = "esriSMSCircle"
    esri_sms_cross = "esriSMSCross"
    esri_sms_diamond = "esriSMSDiamond"
    esri_sms_square = "esriSMSSquare"
    esri_sms_triangle = "esriSMSTriangle"
    esri_smsx = "esriSMSX"


class TextDecoration(Enum):
    """
    The text decoration.
    """

    line_through = "line-through"
    none = "none"
    underline = "underline"


class TextStyle(Enum):
    """
    The text style.
    """

    italic = "italic"
    normal = "normal"
    oblique = "oblique"


class TextWeight(Enum):
    """
    The text weight.
    """

    bold = "bold"
    bolder = "bolder"
    lighter = "lighter"
    normal = "normal"


class HorizontalAlignment(Enum):
    """
    One of the following string values representing the horizontal alignment of the text.
    """

    center = "center"
    justify = "justify"
    left = "left"
    right = "right"


class VerticalAlignment(Enum):
    """
    One of the following string values representing the vertical alignment of the text.
    """

    baseline = "baseline"
    bottom = "bottom"
    middle = "middle"
    top = "top"


class Marker(BaseModel):
    """
    Represents markers placed along the line. Markers will have the same color as the line, and their size will be proportional to the width of the line.
    """

    model_config = common_config
    placement: Optional[MarkerPlacement] = Field(
        None, description="Indicates where the marker is placed."
    )
    style: Optional[Literal["arrow"]] = Field(None, description="Style of the marker.")


class SimpleLineSymbolEsriSLS(BaseModel):
    """
    Simple line symbols can be used to symbolize polyline geometries or outlines for polygon fills.
    """

    model_config = common_config

    color: Optional[list[Annotated[int, Field(ge=0, le=255)]]] = Field(
        None,
        description="Color is represented as a four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
    )
    marker: Optional[Marker] = None
    style: Optional[SimpleLineSymbolStyle] = Field(
        None,
        description="String value representing the simple line symbol type.",
    )
    type: Optional[Literal["esriSLS"]] = Field(
        "esriSLS", description="Specifies the type of symbol used."
    )
    width: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the width of the line in points.",
    )


class CimSymbolReference(BaseModel):
    """
    Represents a symbol reference that contains a CIM symbol. In addition to `type` listed below, a symbol reference will contain additional properties.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    type: Literal["CIMSymbolReference"] = Field(
        "CIMSymbolReference",
        description="Specifies the type of symbol used.",
    )


class PictureFillSymbolsEsriPFS(BaseModel):
    """
    Picture fill symbols can be used to symbolize polygon geometries.
    """

    model_config = common_config

    angle: Optional[Annotated[float, Field(ge=0.0, le=360.0)]] = Field(
        None,
        description="Numeric value that defines the number of degrees ranging from 0-360, that a marker symbol is rotated. The rotation is from East in a counter-clockwise direction where East is the 0� axis.",
    )
    content_type: Optional[str] = Field(
        None,
        alias="contentType",
        description="String value indicating the content type for the image.",
    )
    height: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value used if needing to resize the symbol. Specify a value in points. If images are to be displayed in their original size, leave this blank.",
    )
    image_data: Optional[str] = Field(
        None,
        alias="imageData",
        description="String value indicating the base64 encoded data.",
    )
    outline: Optional[SimpleLineSymbolEsriSLS] = Field(
        None, description="Sets the outline of the symbol."
    )
    type: Literal["esriPFS"] = Field(
        "esriPFS", description="Specifies the type of symbol used."
    )
    url: Optional[str] = Field(
        None,
        description="String value indicating the URL of the image. The URL should be relative if working with static layers. A full URL should be used for map service dynamic layers. A relative URL can be dereferenced by accessing the map layer image resource or the feature layer image resource.",
    )
    width: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value used if needing to resize the symbol. Specify a value in points. If images are to be displayed in their original size, leave this blank.",
    )
    xoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the x-axis in points.",
    )
    xscale: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the scale factor in x direction.",
    )
    yoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the y-axis in points.",
    )
    yscale: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the scale factor in y direction.",
    )


class PictureMarkerSymbolEsriPMS(BaseModel):
    """
    Picture marker symbols can be used to symbolize point geometries.
    """

    model_config = common_config

    angle: Optional[Annotated[float, Field(ge=0.0, le=360.0)]] = Field(
        None,
        description="Numeric value that defines the number of degrees ranging from 0-360, that a marker symbol is rotated. The rotation is from East in a counter-clockwise direction where East is the 0� axis.",
    )
    content_type: Optional[str] = Field(
        None,
        alias="contentType",
        description="String value indicating the content type for the image.",
    )
    height: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value used if needing to resize the symbol. Specify a value in points. If images are to be displayed in their original size, leave this blank.",
    )
    image_data: Optional[str] = Field(
        None,
        alias="imageData",
        description="String value indicating the base64 encoded data.",
    )
    type: Literal["esriPMS"] = Field(
        "esriPMS", description="Specifies the type of symbol used."
    )
    url: Optional[str] = Field(
        None,
        description="String value indicating the URL of the image. The URL should be relative if working with static layers. A full URL should be used for map service dynamic layers. A relative URL can be dereferenced by accessing the map layer image resource or the feature layer image resource.",
    )
    width: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value used if needing to resize the symbol. Specify a value in points. If images are to be displayed in their original size, leave this blank.",
    )
    xoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the x-axis in points.",
    )
    yoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the y-axis in points.",
    )


class SimpleFillSymbolEsriSFS(BaseModel):
    """
    Simple fill symbols that can be used to symbolize polygon geometries.
    """

    model_config = common_config

    color: list[Annotated[int, Field(ge=0, le=255)]] | None = Field(
        None,
        description="Color is represented as a four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
    )
    outline: Optional[SimpleLineSymbolEsriSLS] = Field(
        None, description="Sets the outline of the fill symbol."
    )
    style: SimpleFillSymbolStyle = Field(
        ...,
        description="String value representing the simple fill symbol type.",
    )
    type: Literal["esriSFS"] = Field(
        "esriSFS", description="Specifies the type of symbol used."
    )


class SimpleMarkerSymbolEsriSMS(BaseModel):
    """
    Simple marker symbols can be used to symbolize point geometries.
    """

    model_config = common_config

    angle: Optional[Annotated[float, Field(ge=0.0, le=360.0)]] = Field(
        None,
        description="Numeric value used to rotate the symbol. The symbol is rotated counter-clockwise. For example, The following, angle=-30, in will create a symbol rotated -30 degrees counter-clockwise; that is, 30 degrees clockwise.",
    )
    color: list[Annotated[int, Field(ge=0, le=255)]] = Field(
        ...,
        description="Color is represented as a four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
    )
    outline: Optional[SimpleLineSymbolEsriSLS] = Field(
        None, description="Sets the outline of the marker symbol."
    )
    size: Optional[Union[float, int]] = Field(
        None, description="Numeric size of the symbol given in points."
    )
    style: SimpleMarkerSymbolStyle = Field(
        ..., description="String value representing the simple marker type."
    )
    type: Literal["esriSMS"] = Field(
        "esriSMS", description="Specifies the type of symbol used."
    )
    xoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the x-axis in points.",
    )
    yoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the y-axis in points.",
    )


class TextFont(BaseModel):
    """
    Font used for text symbols
    """

    model_config = common_config

    decoration: Optional[TextDecoration] = Field(
        None, description="The text decoration."
    )
    family: Optional[str] = Field(None, description="The font family.")
    size: Optional[Union[float, int]] = Field(
        None, description="The font size in points."
    )
    style: Optional[TextStyle] = Field(None, description="The text style.")
    weight: Optional[TextWeight] = Field(None, description="The text weight.")


class TextSymbolEsriTS(BaseModel):
    """
    Text symbols are used to add text to a feature (labeling).
    """

    model_config = common_config

    angle: Optional[Annotated[float, Field(ge=0.0, le=360.0)]] = Field(
        None,
        description="A numeric value that defines the number of degrees (0 to 360) that a text symbol is rotated. The rotation is from East in a counter-clockwise direction where East is the 0� axis.",
    )
    background_color: Optional[list[Annotated[int, Field(ge=0, le=255)]]] = Field(
        None,
        alias="backgroundColor",
        description="Background color is represented as a four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
    )
    border_line_color: Optional[list[Annotated[int, Field(ge=0, le=255)]]] = Field(
        None,
        alias="borderLineColor",
        description="Borderline color is represented as a four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
    )
    border_line_size: Optional[Union[float, int]] = Field(
        None,
        alias="borderLineSize",
        description="Numeric value indicating the the size of the border line in points.",
    )
    color: list[Annotated[int, Field(ge=0, le=255)]] = Field(
        ...,
        description="Color is represented as a four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",
    )
    font: Optional[TextFont] = Field(
        None,
        description="An object specifying the font used for the text symbol.",
    )
    halo_color: Optional[list[Annotated[int, Field(ge=0, le=255)]]] = Field(
        None,
        alias="haloColor",
        description="Color of the halo around the text.",
    )
    halo_size: Optional[Union[float, int]] = Field(
        None,
        alias="haloSize",
        description="Numeric value indicating the point size of a halo around the text symbol.",
    )
    horizontal_alignment: Optional[HorizontalAlignment] = Field(
        None,
        alias="horizontalAlignment",
        description="One of the following string values representing the horizontal alignment of the text.",
    )
    kerning: Optional[bool] = Field(
        None,
        description="Boolean value indicating whether to adjust the spacing between characters in the text string.",
    )
    right_to_left: Optional[bool] = Field(
        None,
        alias="rightToLeft",
        description="Boolean value, set to true if using Hebrew or Arabic fonts.",
    )
    rotated: Optional[bool] = Field(
        None,
        description="Boolean value indicating whether every character in the text string is rotated.",
    )
    text: Optional[str] = Field(
        None,
        description="only applicable when specified as a client-side graphic.",
    )
    type: Literal["esriTS"] = Field(
        "esriTS", description="Specifies the type of symbol used."
    )
    vertical_alignment: Optional[VerticalAlignment] = Field(
        None,
        alias="verticalAlignment",
        description="One of the following string values representing the vertical alignment of the text.",
    )
    xoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the x-axis in points.",
    )
    yoffset: Optional[Union[float, int]] = Field(
        None,
        description="Numeric value indicating the offset on the y-axis in points.",
    )


class FillSymbol3DLayer(BaseModel):
    """
    FillSymbol3DLayer is used to render the surfaces of flat 2D Polygon geometries and 3D volumetric meshes in a SceneView.
    """

    model_config = common_config
    cast_shadows: bool | None = Field(
        True,
        alias="castShadows",
        description="Boolean to control the shadow casting behaviour of the rendered geometries (only applies to MeshSymbol3D).",
    )
    edges: SketchEdges | SolidEdges | None = Field(
        None,
        description="Specifies an edge visualization style (only applies to MeshSymbol3D). Edges describe the style applied to visually important edges of 3D objects.",
        title="Edges",
    )
    enable: bool | None = None
    material: Material1 | None = None
    outline: Outline1 | None = Field(
        None,
        description="The outline of the symbol layer (only applies to PolygonSymbol3D).",
    )
    pattern: Pattern | None = None
    type: Literal["Fill"] = Field(
        "Fill", description="Specifies the type of symbol used."
    )


class Outline(Outline1):
    """
    Sets properties of the outline of the IconSymbol3DLayer.
    """

    color: Any | None = None
    size: Any | None = None
    transparency: Any | None = None


class IconSymbol3DLayer(BaseModel):
    """
    IconSymbol3DLayer is used to render Point geometries using a flat 2D icon (e.g. a circle) with a PointSymbol3D in a SceneView.
    """

    model_config = common_config
    anchor: Anchor | None = Anchor.center.value
    anchor_position: list[float] | None = Field(
        None,
        alias="anchorPosition",
        description="When `anchor` equals `relative`, this property specifies the position within the icon that should coincide with the feature geometry. Otherwise it is ignored. The position is defined as a factor of the icon dimensions that is added to the icon center: `positionInIcon = (0.5 + anchorPosition) * size`, where `size` is the original size of the icon resource.",
        max_length=2,
        min_length=2,
    )
    enable: bool | None = None
    material: Material | None = None
    outline: Outline | None = Field(
        None,
        description="Sets properties of the outline of the IconSymbol3DLayer.",
    )
    resource: (
        IconSymbol3DLayerResource
        | IconSymbol3DLayerResource1
        | IconSymbol3DLayerResource2
        | None
    ) = Field(
        None,
        description="The shape (primitive) or image URL (href) used to visualize the features.",
        title="IconSymbol3DLayer Resource",
    )
    size: confloat(ge=0.0) = Field(
        ..., description="Icon size in points, positive only"
    )
    type: Literal["Icon"] = Field(
        "Icon", description="Specifies the type of symbol used."
    )


class LineSymbol3D(BaseModel):
    """
    LineSymbol3D is used to render features with Polyline geometry in a 3D SceneView.
    """

    model_config = common_config
    style_origin: StyleOrigin | StyleOrigin1 | None = Field(
        None,
        alias="styleOrigin",
        description="The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.",
        title="styleOrigin",
    )
    symbol_layers: list[
        LineSymbol3DLayer | PathSymbol3DLayer | PathSymbol3DLayer1 | PathSymbol3DLayer2
    ] = Field(
        ...,
        alias="symbolLayers",
        description="A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",
    )
    type: Literal["LineSymbol3D"] = Field(
        "LineSymbol3D", description="Specifies the type of symbol used."
    )


class MeshSymbol3D(BaseModel):
    """
    MeshSymbol3D is used to render 3D mesh features in a SceneLayer in a 3D SceneView.
    """

    model_config = common_config
    style_origin: StyleOrigin | StyleOrigin1 | None = Field(
        None,
        alias="styleOrigin",
        description="The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.",
        title="styleOrigin",
    )
    symbol_layers: list[FillSymbol3DLayer] = Field(
        ...,
        alias="symbolLayers",
        description="A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",
    )
    type: Literal["MeshSymbol3D"] = Field(
        "MeshSymbol3D", description="Specifies the type of symbol used"
    )


class PointSymbol3D(BaseModel):
    """
    PointSymbol3D is used to render features with Point geometry in a 3D SceneView.
    """

    model_config = common_config
    callout: Callout | None = None
    style_origin: StyleOrigin | StyleOrigin1 | None = Field(
        None,
        alias="styleOrigin",
        description="The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.",
        title="styleOrigin",
    )
    symbol_layers: list[IconSymbol3DLayer | ObjectSymbol3DLayer | TextSymbol3DLayer] = (
        Field(
            ...,
            alias="symbolLayers",
            description="A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",
        )
    )
    type: Literal["PointSymbol3D"] = Field(
        "PointSymbol3D", description="Specifies the type of symbol used"
    )
    vertical_offset: VerticalOffset | None = Field(None, alias="verticalOffset")


class InitialState(BaseModel):
    """
    An object that provides information about the initial environment settings and viewpoint of the web scene.
    """

    model_config = common_config
    environment: Environment
    time_extent: list[int] | None = Field(
        None,
        alias="timeExtent",
        description="Represents the time extent for the data displayed on the scene.",
    )
    viewpoint: Viewpoint = Field(
        ...,
        description="Describes a point of view for a 2D or 3D view. In a 3D view, it is determined using a camera position.",
    )


class Viewing(BaseModel):
    """
    View-specific properties of application and UI elements.
    """

    model_config = common_config
    search: Search | None = None


class LineOfSightLayer(BaseModel):
    """
    The LineOfSight layer is a layer for storing line of sight analyses in web scenes.
    """

    model_config = common_config
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    layer_type: Literal["LineOfSightLayer"] = Field(
        "LineOfSightLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        None,
        alias="listMode",
        description="To show or hide the layer in the layer list.",
    )
    observer: LineOfSightObserver
    targets: list[LineOfSightTarget] = Field(
        ...,
        description="A Collection of LineOfSight target objects used for visibility analysis from observer position.",
    )
    title: str = Field(
        ...,
        description="A human readable string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is visible in the scene.",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Time extent for the visibility of the layer.",
    )


class MosaicRule(BaseModel):
    """
    The image service uses a mosaic rule to mosaick multiple rasters on the fly. The mosaic rule parameter is used by many image service operations, for example, export image and identify operations.
    """

    model_config = common_config
    ascending: bool | None = Field(
        True,
        description="Indicate whether to use ascending or descending order.",
    )
    fids: list[int] | None = Field(
        None,
        description="A list that defines a subset of rasters used in the mosaic, be aware that the rasters may not be visible at all scales.",
    )
    item_rendering_rule: RenderingRule | None = Field(
        None,
        alias="itemRenderingRule",
        description="The rendering rule applies on items before mosaicking.",
    )
    lock_raster_ids: list[int] | None = Field(
        None,
        alias="lockRasterIds",
        description="Lock a few rasters in the image service. Used together with `esriMosaicLockRaster`.",
    )
    mosaic_method: MosaicMethod = Field(
        ...,
        alias="mosaicMethod",
        description="A string value that determines how the selected rasters are ordered.",
    )
    mosaic_operation: MosaicOperation | None = Field(
        MosaicOperation.mt_first,
        validate_default=True,
        alias="mosaicOperation",
        description="Use the mosaic operation to resolve overlap pixel values: from first or last raster, use the min, max or mean of the pixel values, or blend them.",
    )
    multidimensional_definition: (
        list[DimensionalDefinition | DimensionalDefinition1] | None
    ) = Field(
        None,
        alias="multidimensionalDefinition",
        description="An array of objects representing a slice from multidimensional data or multiple slices that are dynamically mosaicked and processed by the server. The dimensional definitions in this array are used to filter display data based on slices in one or more dimensions.",
    )
    sort_field: str | None = Field(
        None,
        alias="sortField",
        description="The field name used together with `esriMosaicAttribute` method.",
    )
    sort_value: float | str | None = Field(
        0,
        alias="sortValue",
        description="The base sort value used together with `esriMosaicAttribute` method and `sortField` parameter.",
    )
    viewpoint: PointGeometry | None = Field(
        None,
        description="Use a view point along with `esriMosaicViewpoint`.",
    )
    where: str | None = Field(
        None,
        description="<em>Deprecated. Use `layerDefinition.definitionExpression` instead.</em>",
    )


class FieldModel(BaseModel):
    """
    Contains information about an attribute field.
    """

    model_config = common_config
    alias: str | None = Field(None, description="A string defining the field alias.")
    default_value: float | str | None = Field(
        None,
        alias="defaultValue",
        description="The default value set for the field.",
    )
    domain: CodedValue | InheritedDomain | RangeDomain | None = Field(
        None, description="The domain objects if applicable.", title="domain"
    )
    editable: bool | None = Field(
        None,
        description="A Boolean defining whether this field is editable.",
    )
    length: int | None = Field(
        None,
        description="A number defining how many characters are allowed in a string field.",
    )
    name: str | None = Field(None, description="A string defining the field name.")
    nullable: bool | None = Field(
        None,
        description="A Boolean defining whether this field can have a null value.",
    )
    type: Type3 | None = Field(None, description="A string defining the field type.")


class LabelingInfo(BaseModel):
    """
    The labelingInfo object specifies the label definition for a layer.
    """

    model_config = common_config
    label_expression: str | None = Field(
        None,
        alias="labelExpression",
        description="Defines the expression for text of labels. <em>Deprecated on FeatureLayer, Use `labelExpressionInfo` instead.</em>",
    )
    label_expression_info: LabelExpressionInfo | None = Field(
        None, alias="labelExpressionInfo"
    )
    label_placement: LabelPlacement | None = Field(
        None,
        alias="labelPlacement",
        description="This string property specifies the label placement with respect to that of its feature. A list of label placement values categorized by feature geometry types.",
    )
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="Represents the maximum scale at which the layer definition will be applied.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="Represents the minimum scale at which the layer definition will be applied.",
    )
    name: str | None = Field(None, description="The name of the label class.")
    symbol: LabelSymbol3D | None = Field(
        None, description="The text symbol used to label."
    )
    use_coded_values: bool | None = Field(
        None,
        alias="useCodedValues",
        description="Boolean value indicating whether to display the coded values for the specified field name(s).",
    )
    where: str | None = Field(
        None,
        description="String template used to determine which features to label.",
    )


class VoxelStyle(BaseModel):
    """
    A voxel style allows you to define the visualization of the voxel layer. A voxel layer can be represented as volume or surface depending on your analysis needs. You can change the visibility of the different representations and change their drawing options.
    """

    model_config = common_config
    current_variable_id: int = Field(
        ...,
        alias="currentVariableId",
        description="Id of the currently visible variable.",
    )
    enable_dynamic_sections: bool | None = Field(
        None,
        alias="enableDynamicSections",
        description='Enable dynamic section (only if `renderMode = "surfaces"`).',
    )
    enable_isosurfaces: bool | None = Field(
        None,
        alias="enableIsosurfaces",
        description='Enable isosurfaces (only if `renderMode = "surfaces"`).',
    )
    enable_sections: bool | None = Field(
        None, alias="enableSections", description="Enable static sections."
    )
    enable_slices: bool | None = Field(
        None, alias="enableSlices", description="Enable slices."
    )
    render_mode: RenderMode | None = Field(
        None,
        alias="renderMode",
        description="Current rendering mode for the voxel layer. Depending on the rendering mode different voxel layer representations can be shown. `volume` draws the voxel layer as rectangular cuboid (but any slices defined for the voxel layer will change the volume to the area of interest). `surfaces` will represent the layer as a set of surfaces, for example, dynamic sections can define a plane through the volume or isosurfaces can show a specific value as surface.",
    )
    shading: VoxelShading | None = Field(
        None, description="Defines the shading properties."
    )
    variable_styles: list[VoxelVariableStyle] | None = Field(
        None,
        alias="variableStyles",
        description="Array of variable styles. Each variable can have one variable style.",
        min_length=1,
    )
    volume_styles: list[VolumeStyle] | None = Field(
        None,
        alias="volumeStyles",
        description="Array of volume styles. Currently only one volume style is allowed.",
        min_length=1,
    )


class PolygonSymbol3D(BaseModel):
    """
    PolygonSymbol3D is used to render features with Polygon geometry in a 3D SceneView. Polygon features may also be rendered as points with icons or objects at the centroid of each polygon.
    """

    model_config = common_config
    style_origin: StyleOrigin | StyleOrigin1 | None = Field(
        None,
        alias="styleOrigin",
        description="The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.",
        title="styleOrigin",
    )
    symbol_layers: list[
        ExtrudeSymbol3DLayer
        | FillSymbol3DLayer
        | IconSymbol3DLayer
        | LineSymbol3DLayer
        | ObjectSymbol3DLayer
        | TextSymbol3DLayer
        | WaterSymbol3DLayer
    ] = Field(
        ...,
        alias="symbolLayers",
        description="A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",
    )
    type: Literal["PolygonSymbol3D"] = Field(
        "PolygonSymbol3D", description="Specifies the type of symbol used."
    )


class UniqueValueInfo(BaseModel):
    """
    Info item for the Unique Value Renderer. Its symbol gets selected if the feature's field matches its value.
    """

    model_config = common_config
    description: str | None = Field(
        None, description="String value used to describe the drawn symbol."
    )
    label: str | None = Field(
        None, description="String value used to label the drawn symbol."
    )
    symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
        | None
    ) = Field(
        None,
        description="An object used to display the value.",
        title="Symbol3D",
    )
    value: str | int | float | None = Field(
        None, description="String value indicating the unique value."
    )


class UniqueValueClass(BaseModel):
    """
    The following is a list of properties found on the uniqueValueClass object. The uniqueValueClass object contains the symbology for grouped unique values in the renderer.
    """

    model_config = common_config
    description: str | None = Field(
        None, description="String value used to describe the drawn symbol."
    )
    label: str | None = Field(
        None, description="String value used to label the drawn symbol."
    )
    symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
    ) = Field(
        ...,
        description="An object used to display the value.",
        title="Symbol3D",
    )
    values: list[list[str]] = Field(
        ...,
        description="A list of unique values that should be rendered with the same symbol. Each item in the list represents a set of value combinations represented by the given symbol. The inner array must contain only one value if only field1 is specified, two values if field1 and field2 are specified, or three values if field1, field2, and field3 are specified. The inner arrays must not contain more than three values.",
    )


class ApplicationProperties(BaseModel):
    """
    Configuration of application and UI elements.
    """

    model_config = common_config
    viewing: Viewing | None = None


class PopupInfo(BaseModel):
    """
    Defines the look and feel of popup windows when a user clicks or queries a feature.
    """

    model_config = common_config
    description: str | None = Field(
        None,
        description="A string that appears in the body of the popup window as a description. A basic subset of HTML may also be used to enrich the text. The supported HTML for ArcGIS Online can be seen in the [Supported HTML](https://doc.arcgis.com/en/arcgis-online/reference/supported-html.htm) page.",
    )
    expression_infos: list[PopupExpressionInfo] | None = Field(
        None,
        alias="expressionInfos",
        description="List of Arcade expressions added to the pop-up.",
    )
    field_infos: list[FieldInfo] | None = Field(
        None,
        alias="fieldInfos",
        description="Array of fieldInfo information properties. This information is provided by the service layer definition.",
    )
    layer_options: LayerOptions | None = Field(
        None,
        alias="layerOptions",
        description="Additional options that can be defined for the popup layer.",
    )
    media_infos: list[MediaInfoChart | MediaInfoImage] | None = Field(
        None,
        alias="mediaInfos",
        description="Array of various mediaInfo to display. Can be of type `image`, `piechart`, `barchart`, `columnchart`, or `linechart`. The order given is the order in which it displays.",
    )
    popup_elements: (
        list[
            PopupElementAttachments
            | PopupElementExpression
            | PopupElementFields
            | PopupElementMedia
            | PopupElementRelationship
            | PopupElementText
        ]
        | None
    ) = Field(
        None,
        alias="popupElements",
        description="An array of popupElement objects that represent an ordered list of popup elements.",
    )
    related_records_info: RelatedRecordsInfo | None = Field(
        None,
        alias="relatedRecordsInfo",
        description="Indicates whether to enable related records if they exist on a layer.",
    )
    show_attachments: bool | None = Field(
        None,
        alias="showAttachments",
        description="Indicates whether attachments will be loaded for feature layers that have attachments.",
    )
    show_last_edit_info: bool | None = Field(
        True,
        alias="showLastEditInfo",
        description="Indicates whether popup will display information about when and who last edited the feature. Applicable only to layers that have been configured to keep track of such information.",
    )
    title: str | None = Field(
        None,
        description="A string that appears at the top of the popup window as a title.",
    )


class VoxelLayerDefinition(BaseModel):
    """
    The voxelLayerDefinition contains drawing information for a voxel layer.
    """

    model_config = common_config
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification.",
    )
    sections: list[VoxelSection] | None = Field(
        None,
        description="Array of metadata about sections. A section is a static plane through the voxel layer showing the variable the section was created at.",
        min_length=0,
    )
    style: VoxelStyle | None = Field(
        None,
        description="Voxel style describes how the layer will be drawn including rendering and voxel representions that will be visible.",
    )


class BuildingSceneLayerFilterBlock(BaseModel):
    """
    A filter block defines what elements will be filtered with a specific filter mode.  To ensure performance on client applications, it is not recommended to declare multiple filter blocks with the same filter mode. Filter blocks are contained in a filter for a building scene layer. Each filter includes at least one filter block.
    """

    model_config = common_config
    filter_expression: str = Field(
        ...,
        alias="filterExpression",
        description="SQL expression to select features that belong to this filter block.",
    )
    filter_mode: FilterModeSolid | FilterModeWireFrame | FilterModeXRay = Field(
        ...,
        alias="filterMode",
        description="Filter mode represents the way elements draw when participating in a filter block.",
        title="Filter Mode",
    )
    title: str = Field(..., description="Title of the filter block.")


class SimpleRenderer(BaseModel):
    """
    A simple renderer is a renderer that uses one symbol only.
    """

    model_config = common_config
    authoring_info: AuthoringInfo | None = Field(
        None,
        alias="authoringInfo",
        description="An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",
    )
    description: str | None = Field(None, description="Description of the renderer.")
    label: str | None = Field(
        None,
        description="The text string that is displayed in the table of contents.",
    )
    symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
    ) = Field(
        ...,
        description="An object that represents how all features will be drawn.",
        title="Symbol3D",
    )
    type: Literal["simple"] = Field(
        "simple", description="Specifies the type of renderer used."
    )
    visual_variables: (
        list[
            ColorInfoVisualVariable
            | RotationInfoVisualVariable
            | SizeInfoVisualVariable
            | TransparencyInfoVisualVariable
        ]
        | None
    ) = Field(
        None,
        alias="visualVariables",
        description="An array of objects used to set rendering properties.",
    )


class UniqueValueFromStyleRenderer(BaseModel):
    """
    This is a special version of UniqueValue Renderer, which loads uniqueValue infos from a style definition.
    """

    model_config = common_config
    default_label: str | None = Field(
        None,
        alias="defaultLabel",
        description="Default string for the default symbol used to draw unspecified values.",
    )
    default_symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
        | None
    ) = Field(
        None,
        alias="defaultSymbol",
        description="The defaultSymbol on the renderer that get assigned to features with no value or features that do not fall within the configured data.",
        title="Symbol3D",
    )
    field1: str | None = Field(
        None,
        description="String value specifying the first field used to render matching values.",
    )
    style_name: str = Field(
        ...,
        alias="styleName",
        description="A registered web style name. Can not be used in conjunction with styleUrl.",
    )
    style_url: str | None = Field(
        None,
        alias="styleUrl",
        description="URL that points to the web style definition. Can not be used in conjunction with styleName.",
    )
    type: Literal["uniqueValue"] = Field(
        "uniqueValue", description="Specifies the type of renderer used."
    )
    visual_variables: (
        list[
            ColorInfoVisualVariable
            | RotationInfoVisualVariable
            | SizeInfoVisualVariable
            | TransparencyInfoVisualVariable
        ]
        | None
    ) = Field(
        None,
        alias="visualVariables",
        description="An array of visualVariable objects used for continuous color or size, simple, and unique values with feature opacity.",
    )


class UniqueValueFromStyleRenderer1(BaseModel):
    """
    This is a special version of UniqueValue Renderer, which loads uniqueValue infos from a style definition.
    """

    model_config = common_config
    default_label: str | None = Field(
        None,
        alias="defaultLabel",
        description="Default string for the default symbol used to draw unspecified values.",
    )
    default_symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
        | None
    ) = Field(
        None,
        alias="defaultSymbol",
        description="The defaultSymbol on the renderer that get assigned to features with no value or features that do not fall within the configured data.",
        title="Symbol3D",
    )
    field1: str | None = Field(
        None,
        description="String value specifying the first field used to render matching values.",
    )
    style_name: str | None = Field(
        None,
        alias="styleName",
        description="A registered web style name. Can not be used in conjunction with styleUrl.",
    )
    style_url: str = Field(
        ...,
        alias="styleUrl",
        description="URL that points to the web style definition. Can not be used in conjunction with styleName.",
    )
    type: Literal["uniqueValue"] = Field(
        "uniqueValue", description="Specifies the type of renderer used."
    )
    visual_variables: (
        list[
            ColorInfoVisualVariable
            | RotationInfoVisualVariable
            | SizeInfoVisualVariable
            | TransparencyInfoVisualVariable
        ]
        | None
    ) = Field(
        None,
        alias="visualVariables",
        description="An array of visualVariable objects used for continuous color or size, simple, and unique values with feature opacity.",
    )


class Feature(BaseModel):
    """
    Contains information about an attribute field and feature geometry.
    """

    model_config = common_config
    attributes: dict[str, Any] | None = Field(
        None,
        description="The feature attributes. A JSON object that contains a dictionary of name-value pairs. The names are the feature field names. The values are the field values, and they can be any of the standard JSON types: string, number, and boolean. Note that date values are encoded as numbers. The number represents the number of milliseconds since epoch (January 1, 1970) in UTC.",
    )
    geometry: (
        MultipointGeometry | PointGeometry | PolygonGeometry | PolylineGeometry | None
    ) = Field(
        None,
        description="It can be any of the supported geometry types.",
        title="geometry",
    )
    symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
        | None
    ) = Field(
        None,
        description="Symbol used for drawing the feature.",
        title="Symbol3D",
    )


class ClassBreakInfo(BaseModel):
    """
    The classBreaksInfo object provides information about the class breaks associated with the renderer.
    """

    model_config = common_config
    class_max_value: float | None = Field(
        None,
        alias="classMaxValue",
        description="A numeric value used to specify the maximum value for a break.",
    )
    class_min_value: float | None = Field(
        None,
        alias="classMinValue",
        description="A numeric value used to specify discontinuous class breaks. If this value is null or is missing, the map server will calculate the minimum value based on the preceding class' maximum value.",
    )
    description: str | None = Field(
        None, description="String value used to describe the drawn symbol."
    )
    label: str | None = Field(
        None, description="String value used to label the drawn symbol."
    )
    symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
    ) = Field(
        ...,
        description="An object used to display the value.",
        title="Symbol3D",
    )


class UniqueValueGroup(BaseModel):
    """
    Represents a group of unique value classes (i.e. symbols). This is used to group symbols under a common heading and/or when representing multiple unique values with a single symbol.
    """

    model_config = common_config
    classes: list[UniqueValueClass] = Field(
        ...,
        description="Specifies the classes (i.e. symbols) to group under a common heading. Classes may be included here without a heading when representing multiple values with a single symbol.",
    )
    heading: str | None = Field(
        None,
        description="The heading to be displayed for the collection of classes defined in this group.",
    )


class VoxelLayer(BaseModel):
    """
    A voxel layer represents multidimensional spatial and temporal information in a 3D volumetric visualization. You can analyze the volume by slicing, creating sections or isosurfaces. For example, you can visualize atmospheric or oceanic data, a geological underground model, or space-time cubes as voxel layers.
    """

    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="disablePopups allows a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: VoxelLayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing of a voxel layer.",
    )
    layer_type: Literal["Voxel"] = Field(
        "Voxel",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the layer in the layer list.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        True,
        alias="timeAnimation",
        description="Boolean value indicating whether to enable time animation.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible",
    )
    visibility_time_extent: list[int] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Time extent for the visibility of the layer.",
    )


class BuildingSceneLayerFilter(BaseModel):
    """
    Filter allows client applications to reduce the drawn elements of a building to specific types and values. Filters on the webscene override the list of existing filters on the service.
    """

    model_config = common_config
    description: str | None = Field(None, description="Description of the filter.")
    filter_authoring_info: MetadataForCheckboxBasedFilterUI | None = Field(
        None,
        alias="filterAuthoringInfo",
        description="Metadata about the athoring process for this filter.",
    )
    filter_blocks: list[BuildingSceneLayerFilterBlock] = Field(
        ...,
        alias="filterBlocks",
        description="Array of filter blocks defining the filter. A filter contains at least one filter block.",
        min_length=1,
    )
    id: str | None = Field(
        None,
        description="unique filter id (uuid). Either a new id to extend the list of filters, or an existing id to override properties of an existing filter.",
    )
    name: str = Field(..., description="Name of the filter.")


class Template(BaseModel):
    """
    Templates describe features that can be created in a layer. They are generally used with feature collections and editable web-based CSV layers. Templates are not used with ArcGIS feature services as these already have templates defined in the service. They are also defined as properties of the layer definition when there are no defined types. Otherwise, templates are defined as properties of the types.
    """

    model_config = common_config
    description: str | None = Field(
        None,
        description="A string value containing a detailed description of the template.",
    )
    drawing_tool: DrawingTool | None = Field(
        None,
        alias="drawingTool",
        description="An optional string that can define a client-side drawing tool to be used with this feature.",
    )
    name: str | None = Field(
        None,
        description="A string containing a user-friendly name for the template.",
    )
    prototype: Feature | None = Field(
        None,
        description="A feature object representing a prototypical feature for the template.",
    )


class ClassBreaksRenderer(BaseModel):
    """
    A class breaks renderer symbolizes based on the value of some numeric attribute. The classBreakInfo define the values at which the symbology changes.
    """

    model_config = common_config
    authoring_info: AuthoringInfo | None = Field(
        None,
        alias="authoringInfo",
        description="An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",
    )
    background_fill_symbol: PolygonSymbol3D | None = Field(
        None,
        alias="backgroundFillSymbol",
        description="Supported only for polygon features.",
    )
    class_break_infos: list[ClassBreakInfo] = Field(
        ...,
        alias="classBreakInfos",
        description="Array of classBreakInfo objects.",
    )
    default_label: str | None = Field(
        None,
        alias="defaultLabel",
        description="Label for the default symbol used to draw unspecified values.",
    )
    default_symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
        | None
    ) = Field(
        None,
        alias="defaultSymbol",
        description="Symbol used when a value cannot be classified.",
        title="Symbol3D",
    )
    field: str | None = Field(None, description="Attribute field used for renderer.")
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    min_value: float | None = Field(
        None,
        alias="minValue",
        description="The minimum numeric data value needed to begin class breaks.",
    )
    normalization_field: str | None = Field(
        None,
        alias="normalizationField",
        description="Used when normalizationType is field. The string value indicating the attribute field by which the data value is normalized.",
    )
    normalization_total: float | None = Field(
        None,
        alias="normalizationTotal",
        description="Used when normalizationType is percent-of-total, this number property contains the total of all data values.",
    )
    normalization_type: NormalizationType | None = Field(
        None,
        alias="normalizationType",
        description="Determine how the data was normalized.",
    )
    type: Literal["classBreaks"] = Field(
        "classBreaks", description="Specifies the type of renderer used."
    )
    value_expression: str | None = Field(
        None,
        alias="valueExpression",
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number.",
    )
    value_expression_title: str | None = Field(
        None,
        alias="valueExpressionTitle",
        description="The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property.",
    )
    visual_variables: (
        list[
            ColorInfoVisualVariable
            | RotationInfoVisualVariable
            | SizeInfoVisualVariable
            | TransparencyInfoVisualVariable
        ]
        | None
    ) = Field(
        None,
        alias="visualVariables",
        description="An array of objects used to set rendering properties.",
    )


class UniqueValueRenderer(BaseModel):
    """
    A unique value renderer symbolizes based on the value of an attribute.
    """

    model_config = common_config
    authoring_info: AuthoringInfo | None = Field(
        None,
        alias="authoringInfo",
        description="An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",
    )
    background_fill_symbol: PolygonSymbol3D | None = Field(
        None,
        alias="backgroundFillSymbol",
        description="Used for polygon features. It can only be used for bivariate types and size rendering.",
    )
    default_label: str | None = Field(
        None,
        alias="defaultLabel",
        description="Default label for the default symbol used to draw unspecified values.",
    )
    default_symbol: (
        CIMSymbolReference
        | LineSymbol3D
        | MeshSymbol3D
        | PointSymbol3D
        | PolygonSymbol3D
        | StyleSymbolReference
        | StyleSymbolReference1
        | PictureFillSymbolsEsriPFS
        | PictureMarkerSymbolEsriPMS
        | SimpleFillSymbolEsriSFS
        | SimpleLineSymbolEsriSLS
        | SimpleMarkerSymbolEsriSMS
        | TextSymbolEsriTS
        | None
    ) = Field(
        None,
        alias="defaultSymbol",
        description="The defaultSymbol on the renderer that get assigned to features with no value or features that do not fall within the configured data.",
        title="Symbol3D",
    )
    field1: str | None = Field(
        None, description="Attribute field renderer uses to match values."
    )
    field2: str | None = Field(
        None,
        description="If needed, specify an additional attribute field the renderer uses to match values.",
    )
    field3: str | None = Field(
        None,
        description="If needed, specify an additional attribute field the renderer uses to match values.",
    )
    field_delimiter: str | None = Field(
        ", ",
        alias="fieldDelimiter",
        description="String inserted between the values if multiple attribute fields are specified.",
    )
    legend_options: RendererLegendOptions | None = Field(
        None,
        alias="legendOptions",
        description="A legend containing one title, which is a string describing the renderer in the legend.",
    )
    type: Literal["uniqueValue"] = Field(
        "uniqueValue", description="Specifies the type of renderer used."
    )
    unique_value_groups: list[UniqueValueGroup] | None = Field(
        None,
        alias="uniqueValueGroups",
        description="An array of uniqueValueGroup objects. If present, then `uniqueValueGroups` should be used in favor of `uniqueValueInfos`.",
    )
    unique_value_infos: list[UniqueValueInfo] = Field(
        ...,
        alias="uniqueValueInfos",
        description="An array of uniqueValueInfo objects. If `uniqueValueGroups` is present, then this property should be ignored on rad. In the future, this property will be deprecated and eventually removed.",
    )
    value_expression: str | None = Field(
        None,
        alias="valueExpression",
        description="An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to either a string or a number.",
    )
    value_expression_title: str | None = Field(
        None,
        alias="valueExpressionTitle",
        description="The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property.",
    )
    visual_variables: (
        list[
            ColorInfoVisualVariable
            | RotationInfoVisualVariable
            | SizeInfoVisualVariable
            | TransparencyInfoVisualVariable
        ]
        | None
    ) = Field(
        None,
        alias="visualVariables",
        description="An array of objects used to set rendering properties.",
    )


class Type4(BaseModel):
    """
    Contains information about an attribute field.
    """

    model_config = common_config
    domains: (
        dict[constr(pattern=r".*"), CodedValue | InheritedDomain | RangeDomain] | None
    ) = Field(
        None,
        description="A comma-delimited series of domain objects for each domain in the type.",
    )
    id: float | str = Field(
        ..., description="A unique string or numerical ID for the type."
    )
    name: str | None = Field(None, description="A user-friendly name for the type.")
    templates: list[Template] | None = Field(
        None,
        description="Defined as a property of the layer definition when there are no types defined; otherwise, templates are defined as properties of the types.",
    )


class DrawingInfo(BaseModel):
    """
    The drawingInfo object contains drawing information for a feature layer.
    """

    model_config = common_config
    labeling_info: list[LabelingInfo] | None = Field(
        None,
        alias="labelingInfo",
        description="An array of labelingInfo objects.",
    )
    renderer: (
        ClassBreaksRenderer
        | HeatmapRenderer
        | SimpleRenderer
        | UniqueValueRenderer
        | PointCloudClassBreaksRenderer
        | PointCloudRGBRenderer
        | PointCloudStretchRenderer
        | PointCloudUniqueValueRenderer
        | RasterClassBreaksRenderer
        | RasterColormapRenderer
        | RasterShadedReliefRenderer
        | RasterStretchRenderer
        | RasterUniqueValueRenderer
        | UniqueValueFromStyleRenderer
        | UniqueValueFromStyleRenderer1
        | None
    ) = Field(
        None,
        description="The renderer object contains the drawing information for the operationalLayer. This is a list of links to all the renderer objects.",
        title="renderer",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it.",
    )
    transparency: confloat(ge=0.0, le=100.0) | None = Field(
        None,
        description="Number value ranging between 0 (no transparency) to 100 (completely transparent).",
    )


class Webscene(BaseModel):
    """
    The web scene data lists the basemap, operational layers, and presentation slides to be used in the web scene. It also contains information about pop-up windows and layer styling overrides to be used in the web scene. A version property allows you to supply the version of the web scene JSON format being used.
    """

    model_config = common_config
    application_properties: ApplicationProperties | None = Field(
        None, alias="applicationProperties"
    )
    authoring_app: str | None = Field(
        None,
        alias="authoringApp",
        description="String value indicating the application which authored the webmap",
    )
    authoring_app_version: str | None = Field(
        None,
        alias="authoringAppVersion",
        description="String value indicating the authoring App's version number.",
    )
    base_map: BaseMap | None = Field(
        None,
        alias="baseMap",
        description="Basemaps give the web scene a geographic context.",
    )
    clipping_area: ClippingArea | None = Field(None, alias="clippingArea")
    ground: Ground
    height_model_info: HeightModelInfo | None = Field(None, alias="heightModelInfo")
    initial_state: InitialState | None = Field(None, alias="initialState")
    map_floor_info: MapFloorInfo | None = Field(None, alias="mapFloorInfo")
    map_range_info: MapRangeInformation | None = Field(
        None, alias="mapRangeInfo", description="Map Range Information"
    )
    operational_layers: list[
        BuildingSceneLayerBuildingSceneLayer
        | CSVLayerCSV
        | DimensionLayer
        | FeatureLayerArcGISFeatureLayer
        | GeoJSONLayerGeoJSON
        | GroupLayer
        | ImageServiceLayerArcGISImageServiceLayer
        | LineOfSightLayer
        | MapServiceLayerArcGISMapServiceLayer
        | OGCFeatureLayer
        | RasterDataLayer
        | TiledImageServiceLayerArcGISTiledImageServiceLayer
        | VoxelLayer
        | WebTiledLayerWebTiledLayer
        | WebFeatureServiceLayerWFS
        | WMSLayerWMS
        | IntegratedMesh3DTilesLayer
        | IntegratedMesh3DTilesLayer1
        | IntegratedMeshLayerIntegratedMeshLayer
        | IntegratedMeshLayerIntegratedMeshLayer1
        | KMLLayerKML
        | KMLLayerKML1
        | MediaLayerMedia
        | MediaLayerMedia1
        | PointCloudLayer
        | PointCloudLayer1
        | SceneLayerArcGISSceneServiceLayer
        | TiledMapServiceLayerArcGISTiledMapServiceLayer
        | VectorTileLayerVectorTileLayer
        | CatalogLayerCatalogLayer
    ] = Field(
        ...,
        alias="operationalLayers",
        description="Operational layers contain business data which are used to make thematic scenes.",
    )
    presentation: Presentation | None = None
    spatial_reference: SpatialReference = Field(
        ...,
        alias="spatialReference",
        description="An object used to specify the spatial reference of the given geometry.",
        title="spatialReference",
    )
    tables: list[Table] | None = Field(None, description="An array of table objects.")
    transportation_networks: list[TransportationNetwork] | None = Field(
        None,
        alias="transportationNetworks",
        description="Used to specify the transportation networks of the scene.",
    )
    version: str = Field("1.34")
    viewing_mode: ViewingMode = Field(..., alias="viewingMode")
    widgets: Widgets | None = Field(
        None,
        description="the widgets object contains widgets that should be exposed to the user.",
    )


class BaseMap(BaseModel):
    """
    The basemap provides geographic context to the map and scene.
    """

    model_config = common_config
    base_map_layers: list[
        ImageServiceLayerArcGISImageServiceLayer
        | MapServiceLayerArcGISMapServiceLayer
        | OpenStreetMapLayerOpenStreetMap
        | RasterDataLayer
        | TiledImageServiceLayerArcGISTiledImageServiceLayer
        | WebTiledLayerWebTiledLayer
        | WMSLayerWMS
        | SceneLayerArcGISSceneServiceLayer
        | TiledMapServiceLayerArcGISTiledMapServiceLayer
        | VectorTileLayerVectorTileLayer
    ] = Field(
        ...,
        alias="baseMapLayers",
        description="An array of baseMapLayer objects defining the basemaps used in the web scene.",
    )
    elevation_layers: list[RasterDataElevationLayer | TiledElevationLayer] | None = (
        Field(
            None,
            alias="elevationLayers",
            description='<em>Deprecated, use `ground.layers` instead <a href="#elevationLayers"><sup>1</sup></a>.</em> An array of elevationLayer objects defining the basemaps used in the web scene.',
        )
    )
    id: str | None = Field(
        None, description="A unique identifying string for the basemap."
    )
    title: str = Field(
        ...,
        description="Required string title for the basemap that can be used in a table of contents.",
    )


class Presentation(BaseModel):
    """
    A presentation consists of multiple slides, where each slide is a specific view into the web scene.
    """

    model_config = common_config
    slides: list[Slide] | None = Field(None, description="Array of slide objects.")


class Table(BaseModel):
    """
    Root element in the web scene specifying an array of table objects.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(None, description="Unique string identifier for the table.")
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Unique string value indicating an item registered in ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define configurations for the table.",
    )
    path: constr(pattern=r"^file:.+\.geodatabase\?itemId=\d+$") | None = Field(
        None,
        description="For offline data, a path to a geodatabase file. A URI format is used starting with `file:` followed by a file system path with a `.geodatabase` extension. A query parameter `itemId` must also be present specifying the ID of a table in the geodatabase's `GDB_ServiceItems` table. A relative path must be from the file which defines the table. For example `file:../p20/northamerica.geodatabase?itemId=5`.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="An object defining the content of popup windows when you query a record and the sort option for child related records.",
    )
    title: str = Field(..., description="String value for the title of the table.")
    url: str = Field(
        ...,
        description="String value indicating the URL reference of the hosted table.",
    )


class BuildingSceneLayerBuildingSceneLayer(BaseModel):
    """
    The BuildingSceneLayer is a layer type designed for on-demand streaming and displaying building data.
    """

    model_config = common_config
    active_filter_id: str | None = Field(
        None,
        alias="activeFilterId",
        description="specifies the id of the currently active filter",
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    filters: list[BuildingSceneLayerFilter] | None = Field(
        None,
        description="A list of filters available for this layer. Overrides filters defined on the service.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that can define an elevation offset for the layer.",
    )
    layer_type: Literal["BuildingSceneLayer"] = Field(
        "BuildingSceneLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    sublayers: list[BuildingSceneSublayer] | None = Field(
        None,
        description="An array of objects specifying overrides for building scene layer sublayers",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the service.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
        title="timeExtent",
    )


class CSVLayerCSV(BaseModel):
    """
    The CSV layer type references a CSV or TXT file from a publicly-accessible web server. It then dynamically loads into the scene at run time. The CSV layer will maintain a reference to the CSV resource.
    """

    model_config = common_config
    column_delimiter: ColumnDelimiter | None = Field(
        None,
        alias="columnDelimiter",
        description="A string defining the character used to separate columns in a CSV file.",
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    layer_type: Literal["CSV"] = Field(
        "CSV",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the layer in the layer list",
    )
    location_info: LocationInfo | None = Field(
        None,
        alias="locationInfo",
        description="A locationInfo object defining how location information will be retrieved from a CSV file.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click or query a feature.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    screen_size_perspective: bool | None = Field(
        True,
        alias="screenSizePerspective",
        description="Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
        title="timeExtent",
    )


class FeatureLayerArcGISFeatureLayer(BaseModel):
    """
    Feature layers can be created by referencing a layer from either a map service or a feature service. Use a map service if you just want to retrieve geometries and attributes from the server and symbolize them yourself. Use a feature service if you want to take advantage of symbols from the service's source map document. Also, use a feature service if you plan on doing editing with the feature layer. Feature layers honor any feature templates configured in the source map document. Feature collection objects are used to create a feature layer based on the supplied definition.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    layer_type: Literal["ArcGISFeatureLayer"] = Field(
        "ArcGISFeatureLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.geodatabase\?itemId=\d+$") | None = Field(
        None,
        description="For offline data, a path to a geodatabase file. A URI format is used starting with `file:` followed by a file system path with a `.geodatabase` extension. A query parameter `itemId` must also be present specifying the ID of a table in the geodatabase's `GDB_ServiceItems` table. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.geodatabase?itemId=1`.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click or query a feature.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    screen_size_perspective: bool | None = Field(
        True,
        alias="screenSizePerspective",
        description="Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web scene config and in the item/data.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str | None = Field(
        None,
        description="The URL to the layer. If the layer is not from a web service but rather a feature collection, then the url property is omitted.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
        title="timeExtent",
    )


class GeoJSONLayerGeoJSON(BaseModel):
    """
    The GeoJSON layer type references a GeoJSON file from a publicly-accessible web server. It then dynamically loads into the scene at run time. The GeoJSON layer will maintain a reference to the GeoJSON resource.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    layer_type: Literal["GeoJSON"] = Field(
        "GeoJSON",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click or query a feature.",
    )
    refresh_interval: float | None = Field(
        0,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    screen_size_perspective: bool | None = Field(
        True,
        alias="screenSizePerspective",
        description="Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web scene config.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class GroupLayer(BaseModel):
    """
    GroupLayer provides the ability to organize several sublayers into one common layer. Suppose there are several FeatureLayers that all represent water features in different dimensions. For example, wells (points), streams (lines), and lakes (polygons). The GroupLayer provides the functionality to treat them as one layer called Water Features even though they are stored as separate feature layers.<br><br>With respect to scale visibility, sublayers of a group layer will be visible only within the scale range defined for the group layer. A sublayer may further restrict itself to a narrow scale range. In other words, a sublayer will be visible only when the current map scale intersects the scale range of that sublayer as well as the scale range of all its parent group layers.
    """

    model_config = common_config
    blend_mode: BlendMode | None = Field(
        BlendMode.normal, validate_default=True, alias="blendMode"
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer"
    )
    layers: (
        list[
            BuildingSceneLayerBuildingSceneLayer
            | CSVLayerCSV
            | DimensionLayer
            | FeatureLayerArcGISFeatureLayer
            | GeoJSONLayerGeoJSON
            | GroupLayer
            | ImageServiceLayerArcGISImageServiceLayer
            | LineOfSightLayer
            | MapServiceLayerArcGISMapServiceLayer
            | OGCFeatureLayer
            | RasterDataLayer
            | TiledImageServiceLayerArcGISTiledImageServiceLayer
            | VoxelLayer
            | WebTiledLayerWebTiledLayer
            | WebFeatureServiceLayerWFS
            | WMSLayerWMS
            | IntegratedMesh3DTilesLayer
            | IntegratedMesh3DTilesLayer1
            | IntegratedMeshLayerIntegratedMeshLayer
            | IntegratedMeshLayerIntegratedMeshLayer1
            | KMLLayerKML
            | KMLLayerKML1
            | MediaLayerMedia
            | MediaLayerMedia1
            | PointCloudLayer
            | PointCloudLayer1
            | SceneLayerArcGISSceneServiceLayer
            | TiledMapServiceLayerArcGISTiledMapServiceLayer
            | VectorTileLayerVectorTileLayer
            | CatalogLayerCatalogLayer
        ]
        | None
    ) = Field(None, description="list of child operationalLayers")
    layer_type: Literal["GroupLayer"] = Field(
        "GroupLayer",
        alias="layerType",
        description="String indicating the layer type",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the group layer in the layer list",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: float | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene",
    )
    visibility_mode: VisibilityMode | None = Field(
        None,
        alias="visibilityMode",
        description="How visibility of children is affected:independent, exclusive, inherited",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class CatalogLayerCatalogLayer(BaseModel):
    """
    A catalog layer is a collection of item references to local and shared datasets, layers, services, and workspaces from across
    various work environments. The references, which are catalogued and added as items in a dataset, can be visualized, filtered, and queried dynamically in maps and scenes.
    """

    model_config = common_config

    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    dynamic_group_layer: CatalogDynamicGroupLayerCatalogDynamicGroupLayer | None = (
        Field(
            None,
            alias="dynamicGroupLayer",
            description="Object representing the dynamic group layer that loads layers selectively based on the current map extent, their scale visibility, time and range.",
        )
    )
    footprint_layer: CatalogFoodprintLayerCatalogFootprintLayer | None = Field(
        None,
        alias="footprintLayer",
        description="bject representing the dynamic group layer to visualize the catalog layer items as a polygon feature layer.",
    )
    id: str | None = Field(
        ..., description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    layer_type: Literal["CatalogLayer"] = Field(
        "CatalogLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the group layer in the layer list",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    time_animation: bool | None = Field(
        False,
        alias="timeAnimation",
        description="Indicates whether to disable time animation if the layer supports it.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str | None = Field(
        None,
        description="The URL to the layer. If the layer is not from a web service but rather a feature collection, then the url property is omitted.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
        title="timeExtent",
    )


class CatalogDynamicGroupLayerCatalogDynamicGroupLayer(BaseModel):
    """
    The Catalog Dynamic Group Layer automatically changes to show catalog items that are currently visible in the view. Can exist only as a child of a catalog layer.
    """

    model_config = common_config

    id: str | None = Field(
        ..., description="A unique identifying string for the layer."
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the group layer in the layer list",
    )
    maximum_visible_sublayers: confloat(ge=0.0) | None = Field(
        10,
        alias="maximumVisibleSublayers",
        description="Gets or sets upper bound for number of layers in view for the dynamic group layer.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene",
    )


class CatalogFoodprintLayerCatalogFootprintLayer(BaseModel):
    """
    The Footprints feature layer shows the boundary or extent of each item participating in the catalog dataset. Can exist only as a child of a catalog layer.
    """

    model_config = common_config

    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Boolean property indicating whether to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        ..., description="A unique identifying string for the layer."
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the group layer in the layer list",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click or query a feature.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene",
    )


class ImageServiceLayerArcGISImageServiceLayer(BaseModel):
    """
    An image service provides access to raster data through a web service. Multiple rasters can be served as one image service through mosaic dataset technology, dynamically processed and mosaicked on the fly. An image service supports accessing both the mosaicked image and its catalog, as well as individual rasters in the catalog. Also, image services can be cached (tiled) or uncached (dynamic). This object specifically details properties within uncached image services.
    """

    model_config = common_config
    band_ids: list[int] | None = Field(
        None,
        alias="bandIds",
        description="An array of bandIds that are visible for a multispectral dataset. It can specify bands to export or rearrange band order(from image service).",
    )
    compression_quality: confloat(ge=0.0, le=100.0) | None = Field(
        None,
        alias="compressionQuality",
        description="Controls how much loss the image will be subjected to by the compression algorithm (from image service).",
    )
    compression_tolerance: confloat(ge=0.0) | None = Field(
        0.01,
        alias="compressionTolerance",
        description="Controls the tolerance of the lerc compression algorithm. The tolerance defines the maximum possible error of pixel values in the compressed image.",
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Boolean property indicating whether to ignore popups defined by the service item.",
    )
    format: Format | None = Field(
        Format.png,
        validate_default=True,
        description="String value representing image format.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    interpolation: Interpolation | None = Field(
        None, description="The algorithm used for interpolation."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="This is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers , or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(None, alias="layerDefinition")
    layer_type: Literal["ArcGISImageServiceLayer"] = Field(
        "ArcGISImageServiceLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    mosaic_rule: MosaicRule | None = Field(
        None,
        alias="mosaicRule",
        description="An object representing a subset from multidimensional data. The data is based on slices or ranges in one or more dimensions in [mosaicRule.description](mosaicRule.md). When the [multidimensionalSubset](multidimensionalSubset.md) is defined then the [mosaicRule.multidimensionalDefinition](mosaicRule.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",
    )
    multidimensional_subset: MultidimensionalSubset | None = Field(
        None,
        alias="multidimensionalSubset",
        description="An object representing a subset from multidimensional data. The data is based on slices or ranges in one or more dimensions in [mosaicRule.description](mosaicRule.md). When the [multidimensionalSubset](multidimensionalSubset.md) is defined then the [mosaicRule.multidimensionalDefinition](mosaicRule.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",
    )
    no_data: int | None = Field(
        None,
        alias="noData",
        description="The pixel value that represents no information.",
    )
    no_data_interpretation: NoDataInterpretation | None = Field(
        NoDataInterpretation.esri_no_data_match_any,
        validate_default=True,
        alias="noDataInterpretation",
        description="A string value of interpretation of noData setting. Default is `esriNoDataMatchAny` when noData is a number, and `esriNoDataMatchAll` when noData is an array.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    pixel_type: PixelType | None = Field(
        None,
        alias="pixelType",
        description="Pertains to the type of values stored in the raster, such as signed integer, unsigned integer, or floating point.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click on the image.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    rendering_rule: RenderingRule | None = Field(
        None,
        alias="renderingRule",
        description="Specifies the rendering rule for how the requested image should be rendered.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Indicates whether to allow map authors the ability to control what layers should be shown in a client's legend control.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web map.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class IntegratedMesh3DTilesLayer(BaseModel):
    """
    An integrated mesh layer with OGC 3D Tiles as the data source. This layer type can represent built and natural 3D features, such as building walls, trees, valleys, and cliffs, and can include realistic textures and elevation information.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["IntegratedMesh3DTilesLayer"] = Field(
        "IntegratedMesh3DTilesLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.3tz$") | None = Field(
        None,
        description="For offline data, a path to integrated mesh data in a 3D Tiles Archive Format file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.3tz`. A relative path must be from the file which defines the layer. For example `file:../p20/edinburgh.3tz`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str = Field(
        ...,
        description="The URL to the tileset JSON file for the OGC 3D Tiles layer.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class IntegratedMesh3DTilesLayer1(BaseModel):
    """
    An integrated mesh layer with OGC 3D Tiles as the data source. This layer type can represent built and natural 3D features, such as building walls, trees, valleys, and cliffs, and can include realistic textures and elevation information.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["IntegratedMesh3DTilesLayer"] = Field(
        "IntegratedMesh3DTilesLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.3tz$") = Field(
        ...,
        description="For offline data, a path to integrated mesh data in a 3D Tiles Archive Format file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.3tz`. A relative path must be from the file which defines the layer. For example `file:../p20/edinburgh.3tz`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str | None = Field(
        None,
        description="The URL to the tileset JSON file for the OGC 3D Tiles layer.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class IntegratedMeshLayerIntegratedMeshLayer(BaseModel):
    """
    An integrated mesh can represent built and natural 3D features, such as building walls, trees, valleys, and cliffs, with realistic textures and includes elevation information.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["IntegratedMeshLayer"] = Field(
        "IntegratedMeshLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    modifications: str | None = Field(
        None,
        description="URL to modifications json file, typically stored in `ITEM/resources`. Content of the file follows the $ref:[Modifications schema](modifications_schema.json).",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.slpk$") | None = Field(
        None,
        description="For offline data, a path to integrated mesh data in a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/edinburgh.slpk`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str = Field(
        ...,
        description="The URL to the layer. If the layer is not from a web service but rather a feature collection, then the url property is omitted.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class IntegratedMeshLayerIntegratedMeshLayer1(BaseModel):
    """
    An integrated mesh can represent built and natural 3D features, such as building walls, trees, valleys, and cliffs, with realistic textures and includes elevation information.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["IntegratedMeshLayer"] = Field(
        "IntegratedMeshLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    modifications: str | None = Field(
        None,
        description="URL to modifications json file, typically stored in `ITEM/resources`. Content of the file follows the $ref:[Modifications schema](modifications_schema.json).",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.slpk$") = Field(
        ...,
        description="For offline data, a path to integrated mesh data in a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/edinburgh.slpk`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str | None = Field(
        None,
        description="The URL to the layer. If the layer is not from a web service but rather a feature collection, then the url property is omitted.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class MapServiceLayerArcGISMapServiceLayer(BaseModel):
    """
    Map services can be cached (tiled) or uncached (dynamic). This object specifically details properties within uncached map services.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="This is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers, or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layers: list[Layer] | None = Field(
        None,
        description="An array of layer objects defining the styling, geometry, and attribute information for the features.",
    )
    layer_type: Literal["ArcGISMapServiceLayer"] = Field(
        "ArcGISMapServiceLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene.",
    )
    visible_layers: list[int] | None = Field(
        None,
        alias="visibleLayers",
        description="An array of sublayer ids that should appear visible. Used with map service layers that are not tiled.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class OGCFeatureLayer(BaseModel):
    """
    OGC Feature Layer is a dynamic feature service that follows the specifications of OGC API - Features.
    """

    model_config = common_config
    collection_id: str | None = Field(
        None,
        alias="collectionId",
        description="A unique identifying string for a feature collection.",
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of parameters used to append custom or vendor specific parameters to all OGC API - Features requests.",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether popup is enabled or not.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Unique string value indicating an item registered in ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    layer_type: Literal["OGCFeatureLayer"] = Field(
        "OGCFeatureLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click or query a feature.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    screen_size_perspective: bool | None = Field(
        True,
        alias="screenSizePerspective",
        description="Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="Boolean value indicating whether to display labels for this layer.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display legend for this layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(
        ...,
        description="The URL of the OGC API Features service landing page.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web map.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class PointCloudLayer(BaseModel):
    """
    Point cloud data is post-processed spatially organized lidar data that consists of large collections of 3D points. Elevations for the ground, buildings, forest canopy, highway overpasses, and anything else encountered during the lidar survey make up the point cloud data. Point cloud layers allow for fast visualisation of point cloud data in the browser.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="disablePopups allows a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer"
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["PointCloudLayer"] = Field(
        "PointCloudLayer",
        alias="layerType",
        description="String indicating the layer type",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the layer in the layer list",
    )
    path: constr(pattern=r"^file:.+\.slpk$") | None = Field(
        None,
        description="For offline data, a path to point cloud layer data in a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/zurich.slpk`.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of pop-up windows when you click a point.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service",
    )
    url: str = Field(..., description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class PointCloudLayer1(BaseModel):
    """
    Point cloud data is post-processed spatially organized lidar data that consists of large collections of 3D points. Elevations for the ground, buildings, forest canopy, highway overpasses, and anything else encountered during the lidar survey make up the point cloud data. Point cloud layers allow for fast visualisation of point cloud data in the browser.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="disablePopups allows a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer"
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["PointCloudLayer"] = Field(
        "PointCloudLayer",
        alias="layerType",
        description="String indicating the layer type",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the layer in the layer list",
    )
    path: constr(pattern=r"^file:.+\.slpk$") = Field(
        ...,
        description="For offline data, a path to point cloud layer data in a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/zurich.slpk`.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of pop-up windows when you click a point.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service",
    )
    url: str | None = Field(None, description="The URL to the layer.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible",
    )


class SceneLayerArcGISSceneServiceLayer(BaseModel):
    """
    The SceneLayer is a layer type designed for on-demand streaming and displaying large amounts of data in a SceneView. SceneLayers support two geometry types: Point and 3D Objects (e.g. buildings). The SceneLayer displays data published to a Scene Service. Scene Services can hold large volumes of features in an open format that is suitable for web streaming. The SceneLayer loads these features progressively, starting from coarse representations and refines them to higher detail as necessary for close-up views.
    """

    model_config = common_config
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="disablePopups allows a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="A layerDefinition object defining the attribute schema and drawing information for the layer.",
    )
    layer_type: Literal["ArcGISSceneServiceLayer"] = Field(
        "ArcGISSceneServiceLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.slpk$") | None = Field(
        None,
        description="For offline data, a path to a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.slpk`.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of pop-up windows when you click or query a feature.",
    )
    screen_size_perspective: bool | None = Field(
        True,
        alias="screenSizePerspective",
        description="Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="If the layer has a labelingInfo property then labels show on the scene only if the showLabels property it true.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        True,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="The URL to the service.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web scene",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class TiledImageServiceLayerArcGISTiledImageServiceLayer(BaseModel):
    """
    An ArcGIS Tiled Image Service layer displays content from an ArcGIS Server Image service that has been cached (tiled).
    """

    model_config = common_config
    band_ids: list[int] | None = Field(
        None,
        alias="bandIds",
        description="An array of bandIds that are visible for a multispectral dataset. It can specify bands to export or rearrange band order(from image service).",
        min_length=1,
    )
    blend_mode: BlendMode | None = Field(
        BlendMode.normal, validate_default=True, alias="blendMode"
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    interpolation: Interpolation1 | None = Field(
        None, description="String indicating the interpolation type."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="Applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers , or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(None, alias="layerDefinition")
    layer_type: Literal["ArcGISTiledImageServiceLayer"] = Field(
        "ArcGISTiledImageServiceLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        alias="listMode",
        description="To show or hide the sublayer in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    multidimensional_definition: (
        list[DimensionalDefinition | DimensionalDefinition1] | None
    ) = Field(
        None,
        alias="multidimensionalDefinition",
        description="An array of objects representing a slice from multidimensional data. The dimensional definitions in this array are used to filter display data based on slices in one or more dimensions.",
    )
    multidimensional_subset: MultidimensionalSubset | None = Field(
        None,
        alias="multidimensionalSubset",
        description="An object representing a subset from multidimensional data. This includes subsets of both variables and dimensions. When the multidimensionalSubset is defined on a tiled image service layer, the layer's [multidimensionalDefinition](multidimensionalDefinition.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click on the image.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    rendering_rule: RenderingRule | None = Field(
        None,
        alias="renderingRule",
        description="Specifies the rendering rule for how the requested image should be rendered.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(..., description="URL to the ArcGIS Server Image Service.")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class TiledMapServiceLayerArcGISTiledMapServiceLayer(BaseModel):
    """
    An ArcGIS Tiled Map Service layer displays map content from an ArcGIS Server Map service that has been cached (tiled).
    """

    model_config = common_config
    blend_mode: BlendMode | None = Field(
        BlendMode.normal, validate_default=True, alias="blendMode"
    )
    custom_parameters: dict[constr(pattern=r".*"), str] | None = Field(
        None,
        alias="customParameters",
        description="A sequence of custom parameters appended to the URL of all requests related to a layer.",
        title="customParameters",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    is_reference: bool | None = Field(
        None,
        alias="isReference",
        description="This property is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers , or below (false).",
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal.",
    )
    layers: list[Layer] | None = Field(
        None,
        description="An array of layer objects defining a URL for queries and the popup window content.",
    )
    layer_type: Literal["ArcGISTiledMapServiceLayer"] = Field(
        "ArcGISTiledMapServiceLayer",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",
    )
    max_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: confloat(ge=0.0) | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    path: constr(pattern=r"^file:.+\.tpkx?$") | None = Field(
        None,
        description="For offline data, a path to a tile package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.tpk` or `.tpkx`. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.tpk`.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display the layer in the legend. Default value is `true`.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service.",
    )
    url: str = Field(..., description="URL to the ArcGIS Server tiled Map Service")
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the map.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )


class WebFeatureServiceLayerWFS(BaseModel):
    """
    OGC Web Feature Service (WFS) is a dynamic feature service that follows the specifications of OGC.
    """

    model_config = common_config
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore popups defined by the service item.",
    )
    id: str | None = Field(
        None, description="A unique identifying string for the layer."
    )
    item_id: str | None = Field(
        None,
        alias="itemId",
        description="Unique string value indicating an item registered in ArcGIS Online or your organization's portal.",
    )
    layer_definition: LayerDefinition | None = Field(None, alias="layerDefinition")
    layer_type: Literal["WFS"] = Field(
        "WFS",
        alias="layerType",
        description="String indicating the layer type.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        validate_default=True,
        alias="listMode",
        description="To show or hide layers in the layer list",
    )
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of popup windows when you click or query a feature.",
    )
    refresh_interval: float | None = Field(
        None,
        alias="refreshInterval",
        description="Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",
    )
    screen_size_perspective: bool | None = Field(
        True,
        alias="screenSizePerspective",
        description="Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",
    )
    show_labels: bool | None = Field(
        False,
        alias="showLabels",
        description="Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web map config and in the item/data.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="Boolean value indicating whether to display legend for this layer.",
    )
    time_animation: bool | None = Field(
        None,
        alias="timeAnimation",
        description="Indicates whether to enable time animation for the layer.",
    )
    title: str = Field(
        ...,
        description="A user-friendly string title for the layer that can be used in a table of contents.",
    )
    url: str = Field(
        ...,
        description="The URL to the layer. If the layer is not from a web service but rather a feature collection, than the url property is omitted.",
    )
    visibility: bool | None = Field(
        True,
        description="Boolean property determining whether the layer is initially visible in the web map.",
    )
    visibility_time_extent: list[int | None] | None = Field(
        None,
        alias="visibilityTimeExtent",
        description="Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.  ",
    )
    wfs_info: WebFeatureServiceInfo | None = Field(
        None,
        alias="wfsInfo",
        description="Object that defines and provides information about layers in a WFS service.",
    )


class Slide(BaseModel):
    """
    A slide object used within a presentation.
    """

    model_config = common_config
    base_map: BaseMap | None = Field(
        None,
        alias="baseMap",
        description="The basemap to be displayed on the slide.",
    )
    description: Description | None = Field(
        None,
        description="Text description of the individual presentation slide.",
    )
    environment: Environment1 | None = Field(
        None,
        description="Represents settings that affect the environment in which the web scene is displayed.",
    )
    ground: Ground1 | None = Field(
        None, description="The ground properties to be set in the slide."
    )
    hidden: bool | None = Field(
        False,
        description="A boolean used to indicate the visibility of the slide inside a presentation.",
    )
    id: str | None = Field(
        None,
        description="The unique id of a slide within the slides property of a Presentation.",
    )
    thumbnail: Thumbnail
    time_extent: list[int | None] | None = Field(
        None,
        alias="timeExtent",
        description="The time extent of the slide.",
    )
    title: Title
    viewpoint: Viewpoint
    visible_layers: list[VisibleLayer] = Field(
        ...,
        alias="visibleLayers",
        description="An array of objects used to indicate the visible layers of the web scene.",
    )


class TimeIntervalUnits(Enum):
    """
    Temporal unit in which the time interval is measured.
    """

    esri_time_units_centuries = "esriTimeUnitsCenturies"
    esri_time_units_days = "esriTimeUnitsDays"
    esri_time_units_decades = "esriTimeUnitsDecades"
    esri_time_units_hours = "esriTimeUnitsHours"
    esri_time_units_milliseconds = "esriTimeUnitsMilliseconds"
    esri_time_units_minutes = "esriTimeUnitsMinutes"
    esri_time_units_months = "esriTimeUnitsMonths"
    esri_time_units_seconds = "esriTimeUnitsSeconds"
    esri_time_units_unknown = "esriTimeUnitsUnknown"
    esri_time_units_weeks = "esriTimeUnitsWeeks"
    esri_time_units_years = "esriTimeUnitsYears"


class TimeInfoExportOptions(BaseModel):
    """
    The default time-related export options for a layer.
    """

    model_config = common_config
    time_data_cumulative: bool | None = Field(
        None,
        alias="timeDataCumulative",
        description="If true, draw all the features from the beginning of time for that data.",
    )
    time_offset: float | None = Field(
        None,
        alias="timeOffset",
        description="Time offset value for this layer so that it can be overlaid on the top of a previous or future time period.",
    )
    time_offset_units: TimeIntervalUnits | None = Field(
        None,
        alias="timeOffsetUnits",
        description="Temporal unit in which the time offset is measured.",
    )
    use_time: bool | None = Field(
        None,
        alias="useTime",
        description="If true, use the time extent specified by the time parameter.",
    )


class TimeReference(BaseModel):
    """
    Defines information about daylight savings time and the time zone in which data was collected.
    """

    model_config = common_config

    respects_daylight_saving: bool | None = Field(
        None,
        alias="respectsDaylightSaving",
        description="If true, dates will honor the daylight savings equivalent of 'timeZone'. This applies to certain 'timeZone' values only.",
    )
    time_zone: str | None = Field(
        None,
        alias="timeZone",
        description="The time zone in which the data was captured.",
    )
    time_zone_IANA: str | None = Field(
        None,
        alias="timeZoneIANA",
        description="The time zone in which the data was captured. Values are in IANA format.\nClients should use 'timeZoneIANA' over 'timeZone'/'respectsDaylightSaving'.",
    )


class LayerTimeInfo(BaseModel):
    """
    Time info if the layer/table supports querying and exporting maps based on time.
    """

    model_config = common_config
    end_time_field: str | None = Field(
        None,
        alias="endTimeField",
        description="The name of the attribute field that contains the end time information.",
    )
    export_options: TimeInfoExportOptions | None = Field(
        None,
        alias="exportOptions",
        description="The default time-related export options for this layer.",
    )
    has_live_data: bool | None = Field(
        None,
        alias="hasLiveData",
        description="Indicates whether service has live data.",
    )
    start_time_field: str | None = Field(
        None,
        alias="startTimeField",
        description="The name of the attribute field that contains the start time information.",
    )
    time_extent: list[float] | None = Field(
        None,
        alias="timeExtent",
        description="The time extent for all the data in the layer.",
        max_length=2,
        min_length=2,
    )
    time_interval: float | None = Field(
        None,
        alias="timeInterval",
        description="Time interval of the data in the layer. Typically used for the TimeSlider when animating the layer.",
    )
    time_interval_units: TimeIntervalUnits | None = Field(
        None,
        alias="timeIntervalUnits",
        description="Temporal unit in which the time interval is measured.",
    )
    time_reference: TimeReference | None = Field(
        None,
        alias="timeReference",
        description="Information about how the time was measured.",
    )
    track_id_field: str | None = Field(
        None,
        alias="trackIdField",
        description="The field that contains the trackId.",
    )


class LayerDefinition(BaseModel):
    """
    An object that defines the attribute schema and drawing information for a layer drawn using client-side graphics.
    """

    model_config = common_config
    allow_geometry_updates: bool | None = Field(
        None,
        alias="allowGeometryUpdates",
        description="Boolean value indicating whether the geometry of the features in the layer can be edited.",
    )
    capabilities: str | None = Field(
        None,
        description="A comma separated list of supported capabilities, e.g. `Query,Editing`.",
    )
    copyright_text: str | None = Field(
        None,
        alias="copyrightText",
        description="String value for the copyright text information for the layer.",
    )
    current_version: float | None = Field(
        None,
        alias="currentVersion",
        description="Numeric value indicating the server version of the layer.",
    )
    default_visibility: bool | None = Field(
        None,
        alias="defaultVisibility",
        description="Boolean value indicating whether the layer's visibility is turned on.",
    )
    definition_expression: str | None = Field(
        None,
        alias="definitionExpression",
        description="SQL-based definition expression string that narrows the data to be displayed in the layer.",
    )
    description: str | None = Field(
        None,
        description="String value of the layer as defined in the map service.",
    )
    display_field: str | None = Field(
        None,
        alias="displayField",
        description="A string value that summarizes the feature.",
    )
    drawing_info: DrawingInfo | None = Field(
        None,
        alias="drawingInfo",
        description="Contains the drawing and labeling information.",
    )
    elevation_info: ElevationInfo | None = Field(None, alias="elevationInfo")
    exclude_object_ids: list[float] | None = Field(
        None,
        alias="excludeObjectIds",
        description="Excludes features with the given object ids from rendering.",
    )
    extent: Extent | dict | None = Field(
        None, description="An object defining the rectangular area."
    )
    feature_reduction: FeatureReductionSelection | None = Field(
        None,
        alias="featureReduction",
        description="An object that specifies how features are reduced or aggregated, with the goal of decluttering the view or presenting the user with an aggregate visualization. If unset, every feature is rendered individually.",
        title="Feature Reduction",
    )
    fields: list[FieldModel] | None = Field(
        None,
        description="An array of field objects containing information about the attribute fields for the feature collection or layer.",
    )
    filters: (
        list[
            PointCloudReturnFilter
            | PointCloudValueFilter
            | PointCloudBitfieldFilter
            | PointCloudBitfieldFilter1
        ]
        | None
    ) = Field(None, description="Filters for PointCloud layers")
    floor_info: FloorInfo | None = Field(
        None,
        alias="floorInfo",
        description="Contains floor-awareness information for the layer.",
    )
    geometry_type: str | None = Field(
        None,
        alias="geometryType",
        description="A string defining the type of geometry. Possible geometry types are: `esriGeometryPoint`, `esriGeometryMultipoint`, `esriGeometryPolyline`, `esriGeometryPolygon`, and `esriGeometryEnvelope`.",
    )
    global_id_field: str | None = Field(
        None,
        alias="globalIdField",
        description="The unique identifier for a feature or table row within a geodatabase.",
    )
    has_attachments: bool | None = Field(
        None,
        alias="hasAttachments",
        description="Indicates whether attachments should be loaded for the layer.",
    )
    has_m: bool | None = Field(
        None,
        alias="hasM",
        description="Boolean value indicating whether layer has M values.",
    )
    has_static_data: bool | None = Field(
        None,
        alias="hasStaticData",
        description="Boolean value indicating whether data changes. True if it does not.",
    )
    has_z: bool | None = Field(
        None,
        alias="hasZ",
        description="Boolean value indicating whether layer has Z values.",
    )
    html_popup_type: HtmlPopupType | None = Field(
        None,
        alias="htmlPopupType",
        description="String value indicating the HTML popup type.",
    )
    id: int | None = Field(None, description="The identifier assigned to the layer.")
    is_data_versioned: bool | None = Field(
        None,
        alias="isDataVersioned",
        description="Boolean value indicating whether the data is versioned.",
    )
    max_record_count: int | None = Field(
        None,
        alias="maxRecordCount",
        description="Numeric value indicating tbe maximum number of records that will be returned at once for a query.",
    )
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification.",
    )
    name: str | None = Field(
        None,
        description="Contains a unique name for the layer that can be displayed in a legend.",
    )
    object_id_field: str | None = Field(
        None,
        alias="objectIdField",
        description="Indicates the name of the object ID field in the dataset.",
    )
    order_by: list[OrderByField] | None = Field(
        None,
        alias="orderBy",
        description="An array of orderByFields objects that define the sorting of the layer.",
    )
    override_symbols: bool | None = Field(
        None,
        alias="overrideSymbols",
        description="Dictates whether a client can support having an end user modify symbols on individual features.",
    )
    polygon_filter: PolygonFilter | None = Field(
        None,
        alias="polygonFilter",
        description="[Polygon filter](polygonFilter.md) for scene layer features.",
    )
    range_infos: list[RangeInformation] | None = Field(
        None,
        alias="rangeInfos",
        description="Indicates range information",
        min_length=1,
    )
    source: DynamicDataLayer | DynamicMapLayer | None = Field(
        None,
        description="An object indicating the layerDefinition's layer source.",
        title="source",
    )
    spatial_reference: SpatialReference | None = Field(
        None,
        alias="spatialReference",
        description="An object containing the WKID or WKT identifying the spatial reference of the layer's geometry.",
        title="spatialReference",
    )
    supported_query_formats: str | None = Field(
        None,
        alias="supportedQueryFormats",
        description="String value indicating the output formats that are supported in a query.",
    )
    supports_advanced_queries: bool | None = Field(
        None,
        alias="supportsAdvancedQueries",
        description="Boolean value indicating whether the layer supports orderByFields in a query operation.",
    )
    supports_attachments_by_upload_id: bool | None = Field(
        None,
        alias="supportsAttachmentsByUploadId",
        description="Boolean value indicating whether the layer supports uploading attachments with the Uploads operation. This can then be used in the Add Attachment and Update Attachment operations.",
    )
    supports_calculate: bool | None = Field(
        None,
        alias="supportsCalculate",
        description="Boolean value indicating whether the layer supports the Calculate REST operation when updating features.",
    )
    supports_rollback_on_failure_parameter: bool | None = Field(
        None,
        alias="supportsRollbackOnFailureParameter",
        description="Boolean value indicating whether the layer supports rolling back edits made on a feature layer if some of the edits fail.",
    )
    supports_statistics: bool | None = Field(
        None,
        alias="supportsStatistics",
        description="Boolean value indicating whether feature layer query operations support statistical functions.",
    )
    supports_validate_sql: bool | None = Field(
        None,
        alias="supportsValidateSql",
        description="Boolean value indicating whether the validateSQL operation is supported across a feature service layer.",
    )
    type: Type1 | None = Field(
        Type1.feature_layer,
        validate_default=True,
        description="Indicates whether the layerDefinition applies to a Feature Layer or a Table.",
    )
    type_id_field: str | None = Field(
        None,
        alias="typeIdField",
        description="Contains the name of the field holding the type ID for the features.",
    )
    time_info: LayerTimeInfo | None = Field(
        None,
        alias="timeInfo",
        description="An object that defines properties to enable time on a layer or table. Refer to the individual layerDefinition properties to see which layer types support this.",
    )
    types: list[Type4] | None = Field(
        None, description="Contains information about an attribute field."
    )
    visibility_field: str | None = Field(
        None,
        alias="visibilityField",
        description="String value indicating the attribute field that is used to control the visibility of a feature. If applicable, when rendering a feature the client should use this field to control visibility. The field's values are 0 = do not display, 1 = display.",
    )


class BuildingSceneSublayer(BaseModel):
    """
    The BuildingSceneLayer sublayer is a part of a building scene layer.
    """

    model_config = common_config
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="disablePopups allows a client to ignore popups defined by the service item.",
    )
    id: int = Field(
        ...,
        description="Identifies the sublayer inside the building scene layer.",
    )
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that can define drawing information and a definition expression for the sublayer.",
    )
    list_mode: ListMode | None = Field(
        ListMode.show.value,
        alias="listMode",
        description="To show or hide the sublayer in the layer list.",
    )
    opacity: confloat(ge=0.0, le=1.0) | None = Field(
        1,
        description="The degree of transparency applied to the sublayer on the client side, where 0 is full transparency and 1 is no transparency. This is multiplied with the opacity of the containing layers.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the content of pop-up windows when you click or query a feature.",
    )
    title: str | None = Field(
        None,
        description="A user-friendly string title for the sublayer that can be used in a table of contents.",
    )
    visibility: bool | None = Field(
        None,
        description="Boolean property determining whether the sublayer is initially visible in the web scene",
    )


class Layer(BaseModel):
    """
    A layer object may allow overrides on popup content and drawing behavior for individual layers of a web service. This object also contains geographic features and their attributes when used in a feature collection.
    """

    model_config = common_config
    default_visibility: bool | None = Field(
        None,
        alias="defaultVisibility",
        description="Default visibility of the layers in the map service.",
    )
    disable_popup: bool | None = Field(
        False,
        alias="disablePopup",
        description="Indicates whether to allow a client to ignore the popups defined on the layer. The popupInfo object could be saved in the map or item.",
    )
    id: int | None = Field(None, description="The layer id, as a numeric value.")
    layer_definition: LayerDefinition | None = Field(
        None,
        alias="layerDefinition",
        description="Additional properties that define drawing information and other configurations for the layer.",
    )
    layer_item_id: str | None = Field(
        None,
        alias="layerItemId",
        description="The associated query layer's itemId. Only available when there is a `layerUrl`.  You will see this if [popups are configured](https://doc.arcgis.com/en/arcgis-online/manage-data/publish-tiles-from-features.htm) on it.",
    )
    layer_url: str | None = Field(
        None,
        alias="layerUrl",
        description="A URL to a service that should be used for all queries against the layer.",
    )
    list_mode: ListMode | None = Field(
        None,
        alias="listMode",
        description="To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",
    )
    max_scale: float | None = Field(
        None,
        alias="maxScale",
        description="A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    min_scale: float | None = Field(
        None,
        alias="minScale",
        description="A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",
    )
    name: str | None = Field(None, description="The name of the layer.")
    parent_layer_id: int | None = Field(
        None,
        alias="parentLayerId",
        description="If working with nested layers, this is the numeric value indicating the layer id of the next layer (parent) directly above the current referenced layer.",
    )
    popup_info: PopupInfo | None = Field(
        None,
        alias="popupInfo",
        description="A popupInfo object defining the popup window content for the layer.",
    )
    show_legend: bool | None = Field(
        True,
        alias="showLegend",
        description="A Boolean indicating if the layer should be shown in the legend in client applications.",
    )
    sub_layer_ids: list[int] | None = Field(
        None,
        alias="subLayerIds",
        description="If the layer is a parent layer, it will have one or more sub layers included in an array.",
    )


class DynamicDataLayer(BaseModel):
    """
    A dynamic data layer derived from a registered workspace. More information on this can be found in the [ArcGIS REST API help](http://resources.arcgis.com/en/help/rest/apiref/layersource.html).
    """

    model_config = common_config
    data_source: (
        JoinTableDataSource | QueryTableDataSource | RasterDataSource | TableDataSource
    ) = Field(
        ...,
        alias="dataSource",
        description="The layer's data source.",
        title="dataSource",
    )
    fields: list[FieldModel] | None = Field(
        None,
        description="An array of objects specifying information about an attribute field.",
    )
    type: Literal["dataLayer"] = Field(
        "dataLayer", description="A string value indicating the type."
    )


class JoinTableDataSource(BaseModel):
    """
    Join Table data source is the result of a join operation. Nested joins are supported. To use nested joins, set either leftTableSource or rightTableSource to be a joinTable.
    """

    model_config = common_config
    join_type: JoinType | None = Field(
        None,
        alias="joinType",
        description="The type of join (left outer or left inner).",
    )
    left_table_key: str | None = Field(
        None,
        alias="leftTableKey",
        description="Field name from the left table.",
    )
    left_table_source: DynamicDataLayer | DynamicMapLayer | None = Field(
        None,
        alias="leftTableSource",
        description="The left source. If the leftTableSource is a table, the resulting joinTable is a table. If the leftTableSource is a layer, the resulting joinTable is a layer.",
        title="source",
    )
    right_table_key: str | None = Field(
        None,
        alias="rightTableKey",
        description="Field name from the right table.",
    )
    right_table_source: DynamicDataLayer | DynamicMapLayer | None = Field(
        None,
        alias="rightTableSource",
        description="The right table source.",
        title="source",
    )
    type: Literal["joinTable"] = Field(
        "joinTable",
        description="String value indicating the type for the dataSource.",
    )


Webscene.model_rebuild()
BaseMap.model_rebuild()
Presentation.model_rebuild()
Table.model_rebuild()
BuildingSceneLayerBuildingSceneLayer.model_rebuild()
CSVLayerCSV.model_rebuild()
FeatureLayerArcGISFeatureLayer.model_rebuild()
GeoJSONLayerGeoJSON.model_rebuild()
ImageServiceLayerArcGISImageServiceLayer.model_rebuild()
IntegratedMesh3DTilesLayer.model_rebuild()
IntegratedMesh3DTilesLayer1.model_rebuild()
IntegratedMeshLayerIntegratedMeshLayer.model_rebuild()
IntegratedMeshLayerIntegratedMeshLayer1.model_rebuild()
MapServiceLayerArcGISMapServiceLayer.model_rebuild()
OGCFeatureLayer.model_rebuild()
PointCloudLayer.model_rebuild()
PointCloudLayer1.model_rebuild()
SceneLayerArcGISSceneServiceLayer.model_rebuild()
TiledImageServiceLayerArcGISTiledImageServiceLayer.model_rebuild()
TiledMapServiceLayerArcGISTiledMapServiceLayer.model_rebuild()
WebFeatureServiceLayerWFS.model_rebuild()
Slide.model_rebuild()
LayerDefinition.model_rebuild()
BuildingSceneSublayer.model_rebuild()
Layer.model_rebuild()
DynamicDataLayer.model_rebuild()
